/***************************************************************************
 *   Copyright (C) 2007 by David Torres   *
 *   aiken669@gmail.com   *
 ***************************************************************************/
#include "eval_functions.h"
#include "omweso.h"
#include <math.h>
#include <iostream>

using namespace std;


/**
* Returns a random value no related in any way with the board status or contents.
* @param owboard Pointer to the board to evaluate
* @param boardside Side of the board owned by the player you want to evaluate for.
* @return random value 0->1000
*/
int evalfunc_random(ow_board *owboard, int boardside) {
	int random_integer = rand()%1000;
	return random_integer;
}

/**
* Makes a defensive evaluation of the board. A column is undefended if both their cells are not empty.
* Bd = (d1-d2)/(1+d1+d2); d1 is count of defended columns for boardside defined by params.
* @param owboard Pointer to the board to evaluate
* @param boardside Side of the board owned by the player you want to evaluate for.
* @return Eval value -1000->1000  (9999 if won, -9999 if lost
*/
int evalfunc_defense(ow_board *owboard, int boardside) {
	int d1,d2;
	int columnx, colcount;
	int boardside_enemy;
	if (boardside==BRD_SIDE_NORTH) { boardside_enemy = BRD_SIDE_SOUTH;} else {boardside_enemy = BRD_SIDE_NORTH;}

  if (!owboard->checkavailablemoves(boardside)) {return -9999;}
	if (!owboard->checkavailablemoves(boardside_enemy)) return 9999;

	//owboard->draw_txt();
	colcount = owboard->getcolcount(); 
	d1=0;d2=0;
	for (columnx=0;columnx<colcount;columnx++)  {
		if ((owboard->countseeds(boardside, columnx, BRD_INNER_ROW)==0)||
				(owboard->countseeds(boardside, columnx, BRD_OUTER_ROW)==0)) { d1++;}
		if ((owboard->countseeds(boardside_enemy, columnx, BRD_INNER_ROW)==0)||
				(owboard->countseeds(boardside_enemy, columnx, BRD_OUTER_ROW)==0)) {d2++;}
	}
  //AQUII
	//cout << "EVAL_DEFENSE(" << d1 << "," << d2 << "): " << floor((float)((d1-d2)*1000)/(float)(d1+d2+1)) << endl;
	return  (int)floor((float)((d1-d2)*1000)/(float)(d1+d2+1));
}

/**
* Makes a defensive evaluation of the board. A column is undefended if both their cells are not empty.
* Bd = (d1-d2)/(1+d1+d2); d1 is count of defended columns for boardside defined by params.
* @param owboard Pointer to the board to evaluate
* @param boardside Side of the board owned by the player you want to evaluate for.
* @return Eval value -1000->1000  (9999 if won, -9999 if lost
*/
int evalfunc_bkdefense(ow_board *owboard, int boardside) {
	int d1,d2;
	int columnx, colcount;
  int plfront, plback, enfront, enback;
	int boardside_enemy;
	if (boardside==BRD_SIDE_NORTH) { boardside_enemy = BRD_SIDE_SOUTH;} else {boardside_enemy = BRD_SIDE_NORTH;}

  if (!owboard->checkavailablemoves(boardside)) {return -9999;}
	if (!owboard->checkavailablemoves(boardside_enemy)) return 9999;

	//owboard->draw_txt();
	colcount = owboard->getcolcount(); 
	d1=0;d2=0;
	for (columnx=0;columnx<colcount;columnx++)  {
		plfront = owboard->countseeds(boardside, columnx, BRD_INNER_ROW);
		plback	= owboard->countseeds(boardside, columnx, BRD_OUTER_ROW);
		enfront = owboard->countseeds(boardside_enemy, columnx, BRD_INNER_ROW);
		enback	= owboard->countseeds(boardside_enemy, columnx, BRD_OUTER_ROW);
	
//		if ((plfront!=0)||(plback!=0)) {d1++;}
		if ((plfront>0)&&(plback==0)) {d1++;}

//		if ((enfront!=0)||(enback!=0)) {d2++;}
		if ((enfront>0)&&(enback==0)) {d2++;}
		
	}
  //AQUII
	//cout << "EVAL_DEFENSE(" << d1 << "," << d2 << "): " << floor((float)((d1-d2)*1000)/(float)(d1+d2+1)) << endl;
	return  (int)floor((float)((d1-d2)*1000)/(float)(d1+d2+1));
}

/**
* Makes a evaluation about the dispersion of the seeds in the board. A board with a dispersed distribution is stronger.
* Disp = (d1-d2)/(1+d1+d2); d1 is the dispersion for boardside defined by params.
* @param owboard Pointer to the board to evaluate
* @param boardside Side of the board owned by the player you want to evaluate for.
* @return Eval value -1000->1000  (9999 if won, -9999 if lost
*/
int evalfunc_dispersion(ow_board *owboard, int boardside) {
  int plfront, plback, enfront, enback;
	int columnx, colcount;
	float sigma_max, sigma_pl, sigma_en, mu;
	int c1, c2;
	float d1, d2;
	int boardside_enemy;
	if (boardside==BRD_SIDE_NORTH) { boardside_enemy = BRD_SIDE_SOUTH;} else {boardside_enemy = BRD_SIDE_NORTH;}

  if (!owboard->checkavailablemoves(boardside)) {return -9999;}
	if (!owboard->checkavailablemoves(boardside_enemy)) return 9999;

  //owboard->draw_txt();

	colcount = owboard->getcolcount(); 
	mu = 8;
	sigma_max = 8*sqrt(colcount-1);

	sigma_pl=0;sigma_en=0;
	for (columnx=0;columnx<colcount;columnx++)  {
		plfront = owboard->countseeds(boardside, columnx, BRD_INNER_ROW);
		plback	= owboard->countseeds(boardside, columnx, BRD_OUTER_ROW);
		enfront = owboard->countseeds(boardside_enemy, columnx, BRD_INNER_ROW);
		enback	= owboard->countseeds(boardside_enemy, columnx, BRD_OUTER_ROW);
		
		c1 = plfront + plback;
		c2 = enfront + enback;

		sigma_pl += pow((c1-mu),2);
		sigma_en += pow((c2-mu),2);
		
	}
	//cout << endl;

	//cout << "sigma_en(pre):  " << sigma_en << endl;
  //cout << "sigma_pl(pre):  " << sigma_pl << endl;

	sigma_pl = sqrt(sigma_pl/colcount);
	sigma_en = sqrt(sigma_en/colcount);

	d1 = sigma_max - sigma_pl;
	d2 = sigma_max - sigma_en;

	//cout << "sigma_max: " << sigma_max << endl;
	//cout << "sigma_en:  " << sigma_en << endl;
  //cout << "sigma_pl:  " << sigma_pl << endl;
  //cout << "dispersion:" << (int)floor((float)((d1-d2)*1000)/(float)(d1+d2+1)) << endl;
	return  (int)floor((float)((d1-d2)*1000)/(float)(d1+d2+1));
}

/**
* Makes a evaluation about the mobility of the seeds in the board. The mobility measures the number of alternative
* moves available each turn.
* Mob = (m1-m2)/(1+m1+m2); m1 is mobility for boardside defined by params.
* @param owboard Pointer to the board to evaluate
* @param boardside Side of the board owned by the player you want to evaluate for.
* @return Eval value -1000->1000  (9999 if won, -9999 if lost
*/
int evalfunc_mobility(ow_board *owboard, int boardside) {
	int d1,d2;
	int columnx,colcount;
  int plfront, plback, enfront, enback;
	int boardside_enemy;
	if (boardside==BRD_SIDE_NORTH) { boardside_enemy = BRD_SIDE_SOUTH;} else {boardside_enemy = BRD_SIDE_NORTH;}

  if (!owboard->checkavailablemoves(boardside)) {return -9999;}
	if (!owboard->checkavailablemoves(boardside_enemy)) return 9999;

	colcount = owboard->getcolcount();
//owboard->draw_txt();
	d1=0;d2=0;
	for (columnx=0;columnx<colcount;columnx++)  {
		plfront = owboard->countseeds(boardside, columnx, BRD_INNER_ROW);
		plback	= owboard->countseeds(boardside, columnx, BRD_OUTER_ROW);
		enfront = owboard->countseeds(boardside_enemy, columnx, BRD_INNER_ROW);
		enback	= owboard->countseeds(boardside_enemy, columnx, BRD_OUTER_ROW);

		// avaible movements
		if (plfront>1) {d1++;}	if (plback>1) {d1++;}
		if (enfront>1) {d2++;}	if (enback>1) {d2++;}
		
	}
	return  (int)floor((float)((d1-d2)*1000)/(float)(d1+d2+1));
}




/**
* The board evaluation is higher as the number of seeds grows in the specified boardside.
* Bs = (s1-s2)/(1+s1+s2); s1 is count of seeds in boardside defined by params.
* @param owboard Pointer to the board to evaluate
* @param boardside Side of the board owned by the player you want to evaluate for.
* @return Eval value -1000->1000  (9999 if won, -9999 if lost
*/
int evalfunc_material(ow_board *owboard, int boardside) {
	int seeds1,seeds2;
	int columnx, colcount;
	int boardside_enemy;
	if (boardside==BRD_SIDE_NORTH) { boardside_enemy = BRD_SIDE_SOUTH;} else {boardside_enemy = BRD_SIDE_NORTH;}

  if (!owboard->checkavailablemoves(boardside)) {return -9999;}
	if (!owboard->checkavailablemoves(boardside_enemy)) return 9999;

	//owboard->draw_txt();
	colcount = owboard->getcolcount(); 
	seeds1=0;seeds2=0;
	for (columnx=0;columnx<colcount;columnx++)  {
		seeds1+=owboard->countseeds(boardside, columnx, BRD_INNER_ROW);
		seeds1+=owboard->countseeds(boardside, columnx, BRD_OUTER_ROW);
		seeds2+=owboard->countseeds(boardside_enemy, columnx, BRD_INNER_ROW);
		seeds2+=owboard->countseeds(boardside_enemy, columnx, BRD_OUTER_ROW);
	}
  //AQUII
	//cout << "EVAL_SEEDS(" << seeds1 << "," << seeds2 << "): " << (int)floor((float)(seeds1-seeds2+(colcount*4))*1000/(float)(seeds1+seeds2+(colcount*4)+1)) << endl;
	return  (int)floor((float)(seeds1-seeds2)*1000/(float)(seeds1+seeds2+1));
}

/**
* Makes a evaluation about the cells with seeds in range. A cell has seeds in range if moving from
* this cell the movement ends in a cell of the front row.
* Reach = (r1-r2)/(1+r1+r2); r1 is the reach evaluation for boardside defined by params.
* @param owboard Pointer to the board to evaluate
* @param boardside Side of the board owned by the player you want to evaluate for.
* @return Eval value -1000->1000  (9999 if won, -9999 if lost
*/
int evalfunc_reachfront(ow_board *owboard, int boardside) {
	int d1,d2;
	int columnx,colcount;
  int plfront, plback, enfront, enback;
	int boardside_enemy;
	if (boardside==BRD_SIDE_NORTH) { boardside_enemy = BRD_SIDE_SOUTH;} else {boardside_enemy = BRD_SIDE_NORTH;}

  if (!owboard->checkavailablemoves(boardside)) {return -9999;}
	if (!owboard->checkavailablemoves(boardside_enemy)) return 9999;

	colcount = owboard->getcolcount();
//owboard->draw_txt();
	d1=0;d2=0;
	for (columnx=0;columnx<colcount;columnx++)  {
		plfront = owboard->countseeds(boardside, columnx, BRD_INNER_ROW);
		plback	= owboard->countseeds(boardside, columnx, BRD_OUTER_ROW);
		enfront = owboard->countseeds(boardside_enemy, columnx, BRD_INNER_ROW);
		enback	= owboard->countseeds(boardside_enemy, columnx, BRD_OUTER_ROW);

		// avaible movements
		if (plfront<columnx) { d1++; }
		if (plback>(colcount-columnx)) {d1++;}
		if (enfront<columnx) { d2++; }
		if (enback>(colcount-columnx)) {d2++;}
	}
	return  (int)floor((float)((d1-d2)*1000)/(float)(d1+d2+1));
}

/**
* This is a linear combination of some of the previous characteristics.
* The weight of each characteristic has been calculated throught experimentation.
* @param owboard Pointer to the board to evaluate
* @param boardside Side of the board owned by the player you want to evaluate for.
* @return Eval value -1000->1000  (9999 if won, -9999 if lost
*/
int evalfunc_F1comb(ow_board *owboard, int boardside) {
	int boardside_enemy;
	if (boardside==BRD_SIDE_NORTH) { boardside_enemy = BRD_SIDE_SOUTH;} else {boardside_enemy = BRD_SIDE_NORTH;}
  if (!owboard->checkavailablemoves(boardside)) {return -9999;}
	if (!owboard->checkavailablemoves(boardside_enemy)) return 9999;

	int mateval = evalfunc_material(owboard,boardside);
	int dispeval = evalfunc_dispersion(owboard,boardside);
	int mobileval = evalfunc_mobility(owboard,boardside);
	int reachfronteval = evalfunc_reachfront(owboard,boardside);

	mateval = (int)(1.33*mateval);
	dispeval = (int)(1.60*dispeval);
	mobileval = (int)(1.44*mobileval);
	reachfronteval = (int)(1.0*reachfronteval);

//	if (mateval==9999)||(dispeval


	int feval = (mateval+dispeval+mobileval+reachfronteval);
	feval = feval/5.37; // normalize div by (1.33+1.60+1.44+1.0 = 5.37) so -1000<feval<1000

	return (feval);
}

/**
* This is a linear combination of some of the previous characteristics.
* The weight of each characteristic has been calculated throught experimentation.
* @param owboard Pointer to the board to evaluate
* @param boardside Side of the board owned by the player you want to evaluate for.
* @return Eval value -1000->1000  (9999 if won, -9999 if lost
*/
int evalfunc_F2comb(ow_board *owboard, int boardside) {
	int boardside_enemy;
	if (boardside==BRD_SIDE_NORTH) { boardside_enemy = BRD_SIDE_SOUTH;} else {boardside_enemy = BRD_SIDE_NORTH;}
  if (!owboard->checkavailablemoves(boardside)) {return -9999;}
	if (!owboard->checkavailablemoves(boardside_enemy)) return 9999;

	int mateval = evalfunc_material(owboard,boardside);
	int dispeval = evalfunc_dispersion(owboard,boardside);
	int mobileval = evalfunc_mobility(owboard,boardside);

	mateval = (int)(1.00*mateval);
	dispeval = (int)(1.07*dispeval);
	mobileval = (int)(1.05*mobileval);

	int feval = (mateval+dispeval+mobileval);
	feval = feval/3.12; // normalize div by (1.0+1.07+1.05 = 3.12) so -1000<feval<1000

	return (feval);
}
