/***************************************************************************
 *   Copyright (C) 2007 by David Torres   *
 *   aiken669@gmail.com   *
 ***************************************************************************/
#include "ow_aisearch_alphabeta.h"
#include "eval_functions.h"

#include <iostream>
#include <sys/time.h>
#include <fstream>

using namespace std;

/**
* Creates a new ow_aisearch_alphabeta object width default values.
* @param owboard Pointer to the board used during the game, where to look for moves.
* @return New ow_aisearch_alphabeta object.
*/
ow_aisearch_alphabeta::ow_aisearch_alphabeta(ow_board *owboard)
 : ow_aisearch(owboard)
{
	this->owboard = owboard;
	this->evalfunct = evalfunc_random;
}


/**
* Deletes the ow_aisearch_alphabeta object
*/
ow_aisearch_alphabeta::~ow_aisearch_alphabeta()
{
	//Carefull with this is virtual method, executes both this and parent, shit.
	//cout << "delete ow_aisearch_negamax" << endl;
}

/**
* AlphaBeta search algoritm
* @param prinvar Principal variation found during the IA search.
* @param board Pointer to the board.
* @param boardside Side of the board where to look for best move.
* @param depth  Depth of the search algoritm
* @param nodecount Nodes (positions) analyzed during AI search.
* @return -1 if not found yet, >=0 if best move found, also it is the max eval value.
*/
int ow_aisearch_alphabeta::_alphabeta(ow_move prinvar[], ow_board *board, int boardside, int depth, int alpha, int beta, int *nodecount) {
	int idxrow,idxcol,val,best=alpha;prinvar[0].boardside = -1;
  int idxprvarchild;
  ow_move prinvar_child[AI_MAXDEPTH];

	(*nodecount)++;
	if ((!depth)||(!board->checkavailablemoves(boardside))) {
		return this->evalfunct(board,boardside);
  } else {
		//Begin analyzing regular moves
		for (idxrow=0;idxrow<BRD_ROWS_PER_PLAYER;idxrow++) {
			for (idxcol=0; idxcol<(board->getcolcount());idxcol++) {
				ow_board *newboard = new ow_board(board);
				if (newboard->move_d(boardside, idxcol, idxrow)!=-1) {
					val = -this->_alphabeta(prinvar_child,newboard,(boardside==BRD_SIDE_NORTH)?BRD_SIDE_SOUTH:BRD_SIDE_NORTH,depth-1, -beta, -best, nodecount);
					if (val>best) {
							best=val;prinvar[0].boardside = boardside; prinvar[0].dir = 1; prinvar[0].x = idxcol; prinvar[0].y = idxrow;
							//append the best child principal variation	
							idxprvarchild=0;while (prinvar_child[idxprvarchild].boardside!=-1) {prinvar[idxprvarchild+1].x = prinvar_child[idxprvarchild].x;prinvar[idxprvarchild+1].y = prinvar_child[idxprvarchild].y;prinvar[idxprvarchild+1].boardside = prinvar_child[idxprvarchild].boardside;prinvar[idxprvarchild+1].dir = prinvar_child[idxprvarchild].dir;idxprvarchild++;}prinvar[idxprvarchild+1].boardside=-1;
							// alphabeta cutoff
							if (best>=beta) { 
								delete newboard; return best;
							}
					}
				}
				delete newboard;
			}
		}
		//End analyzing regular moves


		//Begin analyzing reverse capture moves
		for (idxrow=0;idxrow<BRD_ROWS_PER_PLAYER;idxrow++) {
			for (idxcol=0; idxcol<2;idxcol++) { // reverse capture only from 2 most left cols
				//cout << endl << endl << "RALL(depth,col,row): (" << depth << "," << idxcol << "," << idxrow << ")" << endl << endl;
				ow_board *newboard = new ow_board(board);
				if (newboard->rcapture_d(boardside, idxcol, idxrow)!=-1) {
					val = -this->_alphabeta(prinvar_child,newboard,(boardside==BRD_SIDE_NORTH)?BRD_SIDE_SOUTH:BRD_SIDE_NORTH,depth-1, -beta, -best, nodecount);
					if (val>best) {
							best=val;prinvar[0].boardside = boardside; prinvar[0].dir = -1; prinvar[0].x = idxcol; prinvar[0].y = idxrow;
							//append the best child principal variation	
							idxprvarchild=0;while (prinvar_child[idxprvarchild].boardside!=-1) {prinvar[idxprvarchild+1].x = prinvar_child[idxprvarchild].x;prinvar[idxprvarchild+1].y = prinvar_child[idxprvarchild].y;prinvar[idxprvarchild+1].boardside = prinvar_child[idxprvarchild].boardside;prinvar[idxprvarchild+1].dir = prinvar_child[idxprvarchild].dir;idxprvarchild++;}prinvar[idxprvarchild+1].boardside=-1;
							// alphabeta cutoff
							if (best>=beta) { 
								delete newboard; return best;
							}
					}
				}
				delete newboard;
			}
		}
		//End analyzing reverse capture moves

	}

	return best;
}

/**
* AlphaBeta search algoritm (random version)
* If two nodes have the same evaluation it chooses one randomly.
* @param prinvar Principal variation found during the IA search.
* @param board Pointer to the board.
* @param boardside Side of the board where to look for best move.
* @param depth  Depth of the search algoritm
* @param nodecount Nodes (positions) analyzed during AI search.
* @return -1 if not found yet, >=0 if best move found, also it is the max eval value.
*/
int ow_aisearch_alphabeta::_alphabeta_rand(ow_move prinvar[], ow_board *board, int boardside, int depth, int alpha, int beta, int *nodecount) {
	int idxrow,idxcol,val,best=alpha;prinvar[0].boardside = -1;
  int idxprvarchild;
  ow_move prinvar_child[AI_MAXDEPTH];

	(*nodecount)++;
	if ((!depth)||(!board->checkavailablemoves(boardside))) {
		//prinvar[0].boardside = -1;
		return this->evalfunct(board,boardside);
  } else {
		//Begin analyzing regular moves
		for (idxrow=0;idxrow<BRD_ROWS_PER_PLAYER;idxrow++) {
			for (idxcol=0; idxcol<(board->getcolcount());idxcol++) {
				ow_board *newboard = new ow_board(board);
				if (newboard->move_d(boardside, idxcol, idxrow)!=-1) {
					val = -this->_alphabeta_rand(prinvar_child,newboard,(boardside==BRD_SIDE_NORTH)?BRD_SIDE_SOUTH:BRD_SIDE_NORTH,depth-1, -beta, -best, nodecount);
					//if (val==best) { cout << "EQUAL_EVAL: "; }
					if ((val>best)||((val==best)&&((rand()%100)>50))) {
						//if (val==best) { cout << "newRandomOk" << endl; }
						best=val;prinvar[0].boardside = boardside; prinvar[0].dir = 1; prinvar[0].x = idxcol; prinvar[0].y = idxrow;
						//append the best child principal variation	
						idxprvarchild=0;while (prinvar_child[idxprvarchild].boardside!=-1) {prinvar[idxprvarchild+1].x = prinvar_child[idxprvarchild].x;prinvar[idxprvarchild+1].y = prinvar_child[idxprvarchild].y;prinvar[idxprvarchild+1].boardside = prinvar_child[idxprvarchild].boardside;prinvar[idxprvarchild+1].dir = prinvar_child[idxprvarchild].dir;idxprvarchild++;}prinvar[idxprvarchild+1].boardside=-1;
						// alphabeta cutoff
						if (best>=beta) { 
							delete newboard; return best;
						}
					} //else {if (val==best) {cout << "NOnewRandom" << endl;}}
				}
				delete newboard;
			}
		}
		//End analyzing regular moves


		//Begin analyzing reverse capture moves
		for (idxrow=0;idxrow<BRD_ROWS_PER_PLAYER;idxrow++) {
			for (idxcol=0; idxcol<2;idxcol++) { // reverse capture only from 2 most left cols
				//cout << endl << endl << "RALL(depth,col,row): (" << depth << "," << idxcol << "," << idxrow << ")" << endl << endl;
				ow_board *newboard = new ow_board(board);
				if (newboard->rcapture_d(boardside, idxcol, idxrow)!=-1) {
					val = -this->_alphabeta_rand(prinvar_child,newboard,(boardside==BRD_SIDE_NORTH)?BRD_SIDE_SOUTH:BRD_SIDE_NORTH,depth-1, -beta, -best, nodecount);
					if ((val>best)||((val==best)&&((rand()%100)>50))) {
						best=val;prinvar[0].boardside = boardside; prinvar[0].dir = -1; prinvar[0].x = idxcol; prinvar[0].y = idxrow;
						//append the best child principal variation	
						idxprvarchild=0;while (prinvar_child[idxprvarchild].boardside!=-1) {prinvar[idxprvarchild+1].x = prinvar_child[idxprvarchild].x;prinvar[idxprvarchild+1].y = prinvar_child[idxprvarchild].y;prinvar[idxprvarchild+1].boardside = prinvar_child[idxprvarchild].boardside;prinvar[idxprvarchild+1].dir = prinvar_child[idxprvarchild].dir;idxprvarchild++;}prinvar[idxprvarchild+1].boardside=-1;
						// alphabeta cutoff
						if (best>=beta) { 
							delete newboard; return best;
						}
					}
				}
				delete newboard;
			}
		}
		//End analyzing reverse capture moves

	}

	return best;
}

/**
* Abstract method to the search algoritm (alphabeta)
* @param prinvar Principal variation found during the IA search.
* @param board Pointer to the board.
* @param boardside Side of the board where to look for best move.
* @param depth  Depth of the search algoritm
* @return -1 if not found yet, >=0 if best move found, also it is the max eval value.
*/
int ow_aisearch_alphabeta::search(ow_move prinvar[], ow_board *board, int boardside, int depth) {
	struct timeval tv_ini;struct timeval tv_end;
  int usecs_consumed; float secs_consumed; int nodecount=0; char strspeed[100];
  int best; 

	gettimeofday(&tv_ini, NULL);
	srand((unsigned)time(0));
	best = this->_alphabeta_rand(prinvar, board, boardside, depth, -10001, 10001, &nodecount);
	gettimeofday(&tv_end, NULL);
	usecs_consumed = 1000000*(tv_end.tv_sec - tv_ini.tv_sec) + (tv_end.tv_usec - tv_ini.tv_usec);
	secs_consumed = (float)usecs_consumed/(float)1000000;

	int nspeed = usecs_consumed/nodecount; sprintf(strspeed," at %d usecs/node", nspeed);
	cout << "[alphabeta] Nodes analyzed: " << nodecount << " (time " << secs_consumed << " secs)" << endl; //<< strspeed << ")" << endl;
	return best;
}
