/***************************************************************************
 *   Copyright (C) 2007 by David Torres   *
 *   aiken669@gmail.com   *
 ***************************************************************************/
#include "ow_aisearch_scout.h"
#include "eval_functions.h"

#include <iostream>
#include <fstream>
#include <sys/time.h>
using namespace std;

/**
* Creates a new ow_aisearch_scout object width default values.
* @param owboard Pointer to the board used during the game, where to look for moves.
* @return New ow_aisearch_scout object.
*/
ow_aisearch_scout::ow_aisearch_scout(ow_board *owboard)
 : ow_aisearch(owboard)
{
	this->owboard = owboard;
	this->evalfunct = evalfunc_random;
}

/**
* Deletes the ow_aisearch_scout object
*/
ow_aisearch_scout::~ow_aisearch_scout()
{
	//Carefull with this is virtual method, executes both this and parent, shit.
	//cout << "delete ow_aisearch_negamax" << endl;
}




/**
* Scout search algoritm
* @param prinvar Principal variation found during the IA search.
* @param board Pointer to the board.
* @param boardside Side of the board where to look for best move.
* @param depth  Depth of the search algoritm
* @param nodecount Nodes (positions) analyzed during AI search.
* @return -1 if not found yet, >=0 if best move found, also it is the max eval value.
*/
int ow_aisearch_scout::_scout(ow_move_eval prinvar[], ow_board *board, int boardside, int depth, int *nodecount) {
	int idxrow,idxcol; int scfirstnode=1;
  int val; int best=-10001;prinvar[0].boardside = -1;
  int idxprvarchild;ow_move_eval prinvar_child[AI_MAXDEPTH];

	//board->draw_txt();
	(*nodecount)++;
	
	if ((!depth)||(!board->checkavailablemoves(boardside))) {
		return this->evalfunct(board,boardside);
  } else {

		//Begin analyzing regular moves
		for (idxrow=0;idxrow<BRD_ROWS_PER_PLAYER;idxrow++) {
			for (idxcol=0; idxcol<(board->getcolcount());idxcol++) {
				ow_board *newboard = new ow_board(board);
				if (newboard->move_d(boardside, idxcol, idxrow)!=-1) {
					if (scfirstnode) { // for first node
						best = -this->_scout(prinvar_child,newboard,(boardside==BRD_SIDE_NORTH)?BRD_SIDE_SOUTH:BRD_SIDE_NORTH,depth-1,nodecount);
						//cout << "FIRST = [" << best << "]" << endl;
						prinvar[0].boardside = boardside; prinvar[0].dir = 1; prinvar[0].x = idxcol; prinvar[0].y = idxrow;prinvar[0].veval=best;
						//append the best child principal variation	
						idxprvarchild=0;while (prinvar_child[idxprvarchild].boardside!=-1) {prinvar[idxprvarchild+1].x = prinvar_child[idxprvarchild].x;prinvar[idxprvarchild+1].y = prinvar_child[idxprvarchild].y;prinvar[idxprvarchild+1].boardside = prinvar_child[idxprvarchild].boardside;prinvar[idxprvarchild+1].dir = prinvar_child[idxprvarchild].dir;prinvar[idxprvarchild+1].veval = prinvar_child[idxprvarchild].veval;idxprvarchild++;}prinvar[idxprvarchild+1].boardside=-1;
						scfirstnode=0;
					} else {// for second node and further
						if (!this->_test_rand(newboard,(boardside==BRD_SIDE_NORTH)?BRD_SIDE_SOUTH:BRD_SIDE_NORTH,depth-1, -best)) {
							best = -this->_scout(prinvar_child,newboard,(boardside==BRD_SIDE_NORTH)?BRD_SIDE_SOUTH:BRD_SIDE_NORTH,depth-1,nodecount);
							//cout << "SECOND = [" << best << "]" << endl;
							prinvar[0].boardside = boardside;prinvar[0].dir = 1; prinvar[0].x = idxcol; prinvar[0].y = idxrow;prinvar[0].veval = best;
							//append the best child principal variation	
							idxprvarchild=0;while (prinvar_child[idxprvarchild].boardside!=-1) {prinvar[idxprvarchild+1].x = prinvar_child[idxprvarchild].x;prinvar[idxprvarchild+1].y = prinvar_child[idxprvarchild].y;prinvar[idxprvarchild+1].boardside = prinvar_child[idxprvarchild].boardside;prinvar[idxprvarchild+1].dir = prinvar_child[idxprvarchild].dir;prinvar[idxprvarchild+1].veval = prinvar_child[idxprvarchild].veval;idxprvarchild++;}prinvar[idxprvarchild+1].boardside=-1;
						}
					}
				}
				delete newboard;

			}
		}
		//End analyzing regular moves


		//Begin analyzing reverse capture moves
		for (idxrow=0;idxrow<BRD_ROWS_PER_PLAYER;idxrow++) {
			for (idxcol=0; idxcol<2;idxcol++) {
				ow_board *newboard = new ow_board(board);
				if (newboard->rcapture_d(boardside, idxcol, idxrow)!=-1) {
					if (scfirstnode) { // for first node
						best = -this->_scout(prinvar_child,newboard,(boardside==BRD_SIDE_NORTH)?BRD_SIDE_SOUTH:BRD_SIDE_NORTH,depth-1,nodecount);
						//cout << "FIRSTR = [" << best << "]" << endl;
						prinvar[0].boardside = boardside; prinvar[0].dir = -1; prinvar[0].x = idxcol; prinvar[0].y = idxrow;prinvar[0].veval = best;
						//append the best child principal variation	
						idxprvarchild=0;while (prinvar_child[idxprvarchild].boardside!=-1) {prinvar[idxprvarchild+1].x = prinvar_child[idxprvarchild].x;prinvar[idxprvarchild+1].y = prinvar_child[idxprvarchild].y;prinvar[idxprvarchild+1].boardside = prinvar_child[idxprvarchild].boardside;prinvar[idxprvarchild+1].dir = prinvar_child[idxprvarchild].dir;prinvar[idxprvarchild+1].veval = prinvar_child[idxprvarchild].veval;idxprvarchild++;}prinvar[idxprvarchild+1].boardside=-1;
						scfirstnode=0;
					} else {// for second node and further
						if (!this->_test_rand(newboard,(boardside==BRD_SIDE_NORTH)?BRD_SIDE_SOUTH:BRD_SIDE_NORTH,depth-1, -best)) {
							best = -this->_scout(prinvar_child,newboard,(boardside==BRD_SIDE_NORTH)?BRD_SIDE_SOUTH:BRD_SIDE_NORTH,depth-1,nodecount);
							//cout << "SECONDR = [" << best << "]" << endl;
							prinvar[0].boardside = boardside;prinvar[0].dir = -1; prinvar[0].x = idxcol; prinvar[0].y = idxrow;prinvar[0].veval = best;
							//append the best child principal variation	
							idxprvarchild=0;while (prinvar_child[idxprvarchild].boardside!=-1) {prinvar[idxprvarchild+1].x = prinvar_child[idxprvarchild].x;prinvar[idxprvarchild+1].y = prinvar_child[idxprvarchild].y;prinvar[idxprvarchild+1].boardside = prinvar_child[idxprvarchild].boardside;prinvar[idxprvarchild+1].dir = prinvar_child[idxprvarchild].dir;prinvar[idxprvarchild+1].veval = prinvar_child[idxprvarchild].veval;idxprvarchild++;}prinvar[idxprvarchild+1].boardside=-1;
						}
					}

				}
				delete newboard;

			}
		}
		//End analyzing reverse capture moves


	
	}

	return best;
}


/**
* Test() auxiliar probe for scout search algoritm
* @param board Pointer to the board.
* @param boardside Side of the board where to look for best move.
* @param depth  Depth of the search algoritm
* @param score Score achieved during the probe incursion.
* @return -1 if not found yet, >=0 if best move found, also it is the max eval value.
*/
int ow_aisearch_scout::_test(ow_board *board, int boardside, int depth, int score) {
	int idxrow,idxcol;

	if ((!depth)||(!board->checkavailablemoves(boardside))) {
		if (this->evalfunct(board,boardside)>score) return 1; else return 0;
  } else {
		// regular moves test
		for (idxrow=0;idxrow<BRD_ROWS_PER_PLAYER;idxrow++) {
			for (idxcol=0; idxcol<(board->getcolcount());idxcol++) {
				ow_board *newboard = new ow_board(board);
				if (newboard->move_d(boardside, idxcol, idxrow)!=-1) {
					if (!this->_test(newboard,(boardside==BRD_SIDE_NORTH)?BRD_SIDE_SOUTH:BRD_SIDE_NORTH,depth-1,-score)) {
						delete newboard; return 1;
					}
				}
				delete newboard;
			}
		}
		// end regular moves test

		// reverse capture moves test
		for (idxrow=0;idxrow<BRD_ROWS_PER_PLAYER;idxrow++) {
			for (idxcol=0; idxcol<2;idxcol++) {
				ow_board *newboard = new ow_board(board);
				if (newboard->rcapture_d(boardside, idxcol, idxrow)!=-1) {
					if (!this->_test(newboard,(boardside==BRD_SIDE_NORTH)?BRD_SIDE_SOUTH:BRD_SIDE_NORTH,depth-1,-score)) {
						delete newboard; return 1;
					}
				}
				delete newboard;
			}
		}
		// end reverse capture moves test

		return 0;
	}
	return 0;
}

/**
* Test() auxiliar probe for scout search algoritm
* Random version. When nodes get the same probe eval, it selects one of them randomly.
* @param board Pointer to the board.
* @param boardside Side of the board where to look for best move.
* @param depth  Depth of the search algoritm
* @param score Score achieved during the probe incursion.
* @return -1 if not found yet, >=0 if best move found, also it is the max eval value.
*/
int ow_aisearch_scout::_test_rand(ow_board *board, int boardside, int depth, int score) {
	int idxrow,idxcol; int vscore;

	if ((!depth)||(!board->checkavailablemoves(boardside))) {
		vscore = this->evalfunct(board,boardside);
		if (vscore==score) { //randomize
			srand((unsigned)time(0));
			if ((rand()%1000)>500) {return 1;} else {return 0;}
		} else {if (vscore>score) {return 1;} else {return 0;}}
  } else {
		for (idxrow=0;idxrow<BRD_ROWS_PER_PLAYER;idxrow++) {
			for (idxcol=0; idxcol<(board->getcolcount());idxcol++) {
				ow_board *newboard = new ow_board(board);
				if (newboard->move_d(boardside, idxcol, idxrow)!=-1) {
					if (!this->_test_rand(newboard,(boardside==BRD_SIDE_NORTH)?BRD_SIDE_SOUTH:BRD_SIDE_NORTH,depth-1,-score)) {
						delete newboard; return 1;
					}
				}
				delete newboard;
			}
		}
		// reverse capture moves test
		for (idxrow=0;idxrow<BRD_ROWS_PER_PLAYER;idxrow++) {
			for (idxcol=0; idxcol<2;idxcol++) {
				ow_board *newboard = new ow_board(board);
				if (newboard->rcapture_d(boardside, idxcol, idxrow)!=-1) {
					if (!this->_test_rand(newboard,(boardside==BRD_SIDE_NORTH)?BRD_SIDE_SOUTH:BRD_SIDE_NORTH,depth-1,-score)) {
						delete newboard; return 1;
					}
				}
				delete newboard;
			}
		}
		// end reverse capture moves test

		return 0;
	}
	return 0;
}

/**
* Abstract method to the search algoritm (scout)
* @param prinvar Principal variation found during the IA search.
* @param board Pointer to the board.
* @param boardside Side of the board where to look for best move.
* @param depth  Depth of the search algoritm
* @return -1 if not found yet, >=0 if best move found, also it is the max eval value.
*/
int ow_aisearch_scout::search(ow_move prinvar[], ow_board *board, int boardside, int depth) {
	struct timeval tv_ini;struct timeval tv_end;
  int usecs_consumed; float secs_consumed; int nodecount=0; char strspeed[100];
  int best; 

	ow_move_eval prinvar_tmp[AI_MAXDEPTH];

	gettimeofday(&tv_ini, NULL);
	best = this->_scout(prinvar_tmp, board, boardside, depth, &nodecount);
	gettimeofday(&tv_end, NULL);
	usecs_consumed = 1000000*(tv_end.tv_sec - tv_ini.tv_sec) + (tv_end.tv_usec - tv_ini.tv_usec);
	secs_consumed = (float)usecs_consumed/(float)1000000;

	int nspeed = usecs_consumed/nodecount; sprintf(strspeed," at %d usecs/node", nspeed);
	cout << "[scout] Nodes analyzed: " << nodecount << " (time " << secs_consumed << " secs)" << endl; //<< strspeed << ")" << endl;

	//int idxprvar=0;char strfeval[50];
	//cout << "PVScout:";while (prinvar_tmp[idxprvar].boardside!=-1) {cout << " -> " << ((prinvar_tmp[idxprvar].dir==1)?"mv(":"rc(") << this->owboard->translate_coord2name(prinvar_tmp[idxprvar].boardside, prinvar_tmp[idxprvar].x, prinvar_tmp[idxprvar].y) << ")[" << prinvar_tmp[idxprvar].veval << "]";	idxprvar++;} cout << strfeval << endl;

	int idxprvarchild=0;while (prinvar_tmp[idxprvarchild].boardside!=-1) {prinvar[idxprvarchild].x = prinvar_tmp[idxprvarchild].x;prinvar[idxprvarchild].y = prinvar_tmp[idxprvarchild].y;prinvar[idxprvarchild].boardside = prinvar_tmp[idxprvarchild].boardside;prinvar[idxprvarchild].dir = prinvar_tmp[idxprvarchild].dir;idxprvarchild++;}prinvar[idxprvarchild].boardside=-1;	

	return best;
}

