/***************************************************************************
 *   Copyright (C) 2007 by David Torres   *
 *   aiken669@gmail.com   *
 ***************************************************************************/
#include <SDL/SDL_image.h>
#include "ow_board.h"
#include "omweso.h"
#include <iostream>

using namespace std;

/**
* Creates a new 'ow_board' object, create board cells, and sets default board values.
* @param columns Number of columns the board has.
* @return New 'ow_board' object.
*/
ow_board::ow_board(int columns)
{
  ow_cell *newcell; string newcellname;
	int tabx = 72; int taby = 66; int taby2;
	int cellx, celly;
	int coordx, coordx2, coordy, coordy2;

	this->status=BRD_STATUS_READY; this->handseeds=0;this->turnchains=0;
	this->sowing_delay = 0; this->board_sowing_delay = BRD_SOWING_SPEED_FAST;
	this->gfxdata_loaded=0;
  taby2 = taby +2*(CELL_HEIGHT+CELL_SPACING)+6*CELL_SPACING;
	this->brd_num_cols = columns;
	this->movecount = 0; this->movecurrent = 0; this->undo_active = 0;
	this->backupsaved = 0;

	for (cellx=0;cellx<(this->brd_num_cols);cellx++) {
	  coordx2 = tabx+cellx*(CELL_WIDTH+CELL_SPACING);  
		coordx = tabx+((this->brd_num_cols)-cellx-1)*(CELL_WIDTH+CELL_SPACING);  
		for (celly=0;celly<BRD_ROWS_PER_PLAYER;celly++) {
			coordy = taby+(BRD_ROWS_PER_PLAYER-celly-1)*(CELL_HEIGHT+CELL_SPACING);
			coordy2 = taby2+celly*(CELL_HEIGHT+CELL_SPACING);
			newcell = new ow_cell(coordx,coordy,BRD_SIDE_NORTH,cellx,celly);
			newcellname = this->translate_coord2name(BRD_SIDE_NORTH, cellx, celly);
			newcell->setcellname(newcellname);
			this->bcells[BRD_SIDE_NORTH][cellx][celly] = newcell;
 			newcell = new ow_cell(coordx2,coordy2,BRD_SIDE_SOUTH,cellx,celly);
			newcellname = this->translate_coord2name(BRD_SIDE_SOUTH, cellx, celly);
			newcell->setcellname(newcellname);
			this->bcells[BRD_SIDE_SOUTH][cellx][celly] = newcell;
			//this->undodata[BRD_SIDE_NORTH][cellx][celly] = -1;
			//this->undodata[BRD_SIDE_SOUTH][cellx][celly] = -1;

    }
  }

}

/**
* Creates a new board, copying data from this board
* @return Pointer to a new board wi the same: cols, rows, cell contents.
*/
ow_board::ow_board(ow_board *board) {

  ow_cell *newcell;
	int tabx = 72; int taby = 66; int taby2;
	int cellx, celly;
	int coordx, coordx2, coordy, coordy2;

	this->status=BRD_STATUS_READY; this->handseeds=0;
	this->sowing_delay = 0; this->board_sowing_delay = BRD_SOWING_SPEED_FAST;
	this->gfxdata_loaded=0;
  taby2 = taby +2*(CELL_HEIGHT+CELL_SPACING)+6*CELL_SPACING;
	this->brd_num_cols = board->brd_num_cols;
	this->movecount = 0; this->movecurrent = 0; this->undo_active = 0;
	this->backupsaved = 0;

	for (cellx=0;cellx<(this->brd_num_cols);cellx++) {
	  coordx2 = tabx+cellx*(CELL_WIDTH+CELL_SPACING);  
		coordx = tabx+((this->brd_num_cols)-cellx-1)*(CELL_WIDTH+CELL_SPACING);  
		for (celly=0;celly<BRD_ROWS_PER_PLAYER;celly++) {
			coordy = taby+(BRD_ROWS_PER_PLAYER-celly-1)*(CELL_HEIGHT+CELL_SPACING);
			coordy2 = taby2+celly*(CELL_HEIGHT+CELL_SPACING);
			newcell = new ow_cell(coordx,coordy,BRD_SIDE_NORTH,cellx,celly);
			this->bcells[BRD_SIDE_NORTH][cellx][celly] = newcell;
 			newcell = new ow_cell(coordx2,coordy2,BRD_SIDE_SOUTH,cellx,celly);
			this->bcells[BRD_SIDE_SOUTH][cellx][celly] = newcell;
			//this->undodata[BRD_SIDE_NORTH][cellx][celly] = -1;
			//this->undodata[BRD_SIDE_SOUTH][cellx][celly] = -1;

    }
  }

	//Copying the data from the source board
	for (cellx=0;cellx<(this->brd_num_cols);cellx++) {
		for (celly=0;celly<BRD_ROWS_PER_PLAYER;celly++) {
			this->setcellseeds(board->bcells[BRD_SIDE_NORTH][cellx][celly]->countseeds(),BRD_SIDE_NORTH,cellx,celly);
			this->setcellseeds(board->bcells[BRD_SIDE_SOUTH][cellx][celly]->countseeds(),BRD_SIDE_SOUTH,cellx,celly);
		}
	}
}


/**
*	Setup the seeds in the board according to the beginners suggested setup.
*/
void ow_board::suggested_beginners_seeds_initial_setup() {

	for (int cellx=0;cellx<(this->brd_num_cols);cellx++) {
		for (int celly=0;celly<BRD_ROWS_PER_PLAYER;celly++) {
			this->bcells[BRD_SIDE_NORTH][cellx][celly]->emptyseeds();
			this->bcells[BRD_SIDE_SOUTH][cellx][celly]->emptyseeds();
    }
  }
	int idx;

	for (idx=0;idx<(this->brd_num_cols)-1; idx++) {
		this->bcells[BRD_SIDE_NORTH][idx][BRD_INNER_ROW]->addseeds(4);
		this->bcells[BRD_SIDE_SOUTH][idx][BRD_INNER_ROW]->addseeds(4);
	}
	this->bcells[BRD_SIDE_NORTH][idx][BRD_OUTER_ROW]->addseeds(4);
	this->bcells[BRD_SIDE_SOUTH][idx][BRD_OUTER_ROW]->addseeds(4);

}

/**
* Deletes an 'ow_board' object.
*/
ow_board::~ow_board() {
	int cellx, celly;

	for (cellx=0;cellx<(this->brd_num_cols);cellx++) {
		for (celly=0;celly<BRD_ROWS_PER_PLAYER;celly++) {
			delete this->bcells[BRD_SIDE_NORTH][cellx][celly];this->bcells[BRD_SIDE_NORTH][cellx][celly]=NULL;
			delete this->bcells[BRD_SIDE_SOUTH][cellx][celly];this->bcells[BRD_SIDE_SOUTH][cellx][celly]=NULL;
		}
	}	

	if (this->gfxdata_loaded) {
		SDL_FreeSurface(this->gfxdata[OWGFX_BACKGROUND]);
  	SDL_FreeSurface(this->gfxdata[OWGFX_CURSOR]);
  	SDL_FreeSurface(this->gfxdata[OWGFX_CELL]);
	}


}

/**
* Load board .png graphics for draw_gfx() to use
* @return 0 on success, -1 on any error loading
*/
int ow_board::load_gfxdata() {
	int cellx, celly;
  this->gfxdata[OWGFX_BACKGROUND] = IMG_Load(DATAFOLDER("gfx/fondo.png"));
  this->gfxdata[OWGFX_CURSOR] = IMG_Load(DATAFOLDER("gfx/cursor.png"));
  this->gfxdata[OWGFX_CELL] = IMG_Load(DATAFOLDER("gfx/casilla.png"));
  this->plt = this->gfxdata[OWGFX_BACKGROUND]->format->palette;
	this->font = TTF_OpenFont(DATAFOLDER("gfx/FreeSans.ttf"), 10);

	SDL_SetColorKey(this->gfxdata[OWGFX_CELL],SDL_SRCCOLORKEY | SDL_RLEACCEL, 0x0);

	for (cellx=0;cellx<(this->brd_num_cols);cellx++) {
		for (celly=0;celly<BRD_ROWS_PER_PLAYER;celly++) {
			this->bcells[BRD_SIDE_NORTH][cellx][celly]->set_gfxdata(this->gfxdata[OWGFX_CELL]);
			this->bcells[BRD_SIDE_SOUTH][cellx][celly]->set_gfxdata(this->gfxdata[OWGFX_CELL]);
    }
  }
	this->gfxdata_loaded = 1;
  return 0;
}

/**
* Returns gfx color palette for draw_gfx to use
* @return Color palette in SDL format
*/
SDL_Palette * ow_board::get_palette() {
  return this->plt;
}

/**
* Draws the gfx board using SDL library
* @param surface SDL Screen surface to draw
*/
void ow_board::draw_gfx(SDL_Surface *surface) {
	int cellx, celly; cellXY cellcoord;
  SDL_Rect src, dest;

  //Draw Background (fisrt half)
  src.x = 0; src.y = 0; dest.x = 0; dest.y = 0;
  src.h = 480; src.w = 72+(this->brd_num_cols)*(CELL_WIDTH+CELL_SPACING)+13;
  SDL_BlitSurface(this->gfxdata[OWGFX_BACKGROUND], &src, surface, &dest);

  //Draw Background (second half)
  src.x = 584; src.y = 0; dest.x = src.w; dest.y = 0;
  src.h = 480; src.w = 56;
  SDL_BlitSurface(this->gfxdata[OWGFX_BACKGROUND], &src, surface, &dest);

	//Draw the board cells
	for (cellx=0;cellx<(this->brd_num_cols);cellx++) {
		for (celly=0;celly<BRD_ROWS_PER_PLAYER;celly++) {
			this->bcells[BRD_SIDE_NORTH][cellx][celly]->draw(surface,this->font);
			this->bcells[BRD_SIDE_SOUTH][cellx][celly]->draw(surface,this->font);
    }
  }
/*
  //Draw the board cell names
	for (cellx=0;cellx<(this->brd_num_cols);cellx++) {
		for (celly=0;celly<BRD_ROWS_PER_PLAYER;celly++) {

			cellcoord = this->bcells[BRD_SIDE_NORTH][cellx][celly]->getcellcoordXY();
			dest.x=cellcoord.x+5; dest.y=cellcoord.y+2;
			text_surface=TTF_RenderText_Blended(this->font,"A8",color);
			SDL_BlitSurface(text_surface,NULL,surface,&dest);
			SDL_FreeSurface(text_surface);

			cellcoord = this->bcells[BRD_SIDE_SOUTH][cellx][celly]->getcellcoordXY();
			dest.x=cellcoord.x+5; dest.y=cellcoord.y+2;
			text_surface=TTF_RenderText_Blended(this->font,"F0",color);
			SDL_BlitSurface(text_surface,NULL,surface,&dest);
			SDL_FreeSurface(text_surface);
		}
  }
*/

}

/**
* Draws the txt board version
*/
void ow_board::draw_txt() {
	int cellx, celly;
	string blettersnorth;string bletterssouth;
	string horizdivider;
	int tmpseeds;
	
	blettersnorth = " [N] ";
	horizdivider  = "    o";

	switch (this->brd_num_cols) {
		case 8: blettersnorth+= "  H";horizdivider+="---";bletterssouth = "  h"+bletterssouth;
		case 7: blettersnorth+= "  G";horizdivider+="---";bletterssouth = "  g"+bletterssouth;
		case 6: blettersnorth+= "  F";horizdivider+="---";bletterssouth = "  f"+bletterssouth;
		case 5: blettersnorth+= "  E";horizdivider+="---";bletterssouth = "  e"+bletterssouth;
		case 4: blettersnorth+= "  D";horizdivider+="---";bletterssouth = "  d"+bletterssouth;
		case 3: blettersnorth+= "  C";horizdivider+="---";bletterssouth = "  c"+bletterssouth;
		case 2: blettersnorth+= "  B";horizdivider+="---";bletterssouth = "  b"+bletterssouth;
		case 1: blettersnorth+= "  A";horizdivider+="---";bletterssouth = "  a"+bletterssouth;
		break;
	}
	blettersnorth+= "  [N]";
	horizdivider+= "-o";
	bletterssouth = " [S] "+bletterssouth;
	bletterssouth+= "  [S]";

	cout << endl;
	cout << blettersnorth << endl;
	cout << horizdivider << endl;
	//Draw the board cells
	int countnorth=0;
	for (celly=(BRD_ROWS_PER_PLAYER-1);celly>=0;celly--) {
		cout << "  " << celly << " |";
		for (cellx=((this->brd_num_cols)-1);cellx>=0;cellx--) {
			tmpseeds = this->bcells[BRD_SIDE_NORTH][cellx][celly]->countseeds();
			cout << " "; if (tmpseeds<10) cout << "0";cout << tmpseeds;
			countnorth+=tmpseeds;
    }
		cout << " | " << celly << endl;
  }
	cout << horizdivider << endl;
	int countsouth=0;
	for (celly=0;celly<BRD_ROWS_PER_PLAYER;celly++) {
		cout << "  " << celly << " |";
		for (cellx=0;cellx<(this->brd_num_cols);cellx++) {
			tmpseeds = this->bcells[BRD_SIDE_SOUTH][cellx][celly]->countseeds();
			cout << " "; if (tmpseeds<10) cout << "0"; cout << tmpseeds;
			countsouth+=tmpseeds;
    }
		cout << " | " << celly << endl;
  }
	cout << horizdivider << endl;
	cout << bletterssouth << endl;
	cout << endl;
	cout << "N:"<< countnorth << " S:" << countsouth << endl;

}

/**
* Print an illegal movement error on txt_screen and on gfx board (if opened)
* @param boardside Side of the board (BRD_SIDE_NORTH, BRD_SIDE_SOUTH).
* @param cellx Xpos of the specified cell in the bcells array.
* @param celly Ypos of the specified cell in the bcells array.
*/
void ow_board::display_errormove(int boardside, int cellx, int celly) {
	//printf("[%i](%i,%i) -> [Ilegal move]\n",boardside,cellx,celly);
	this->status = BRD_STATUS_ERRORMOVE;
	this->sowing_delay = this->board_sowing_delay;
	this->bcells[boardside][cellx][celly]->setaction(CELL_ACTION_CANCEL,2*this->board_sowing_delay);
	this->bcells[boardside][cellx][celly]->setstatus(CELL_STATUS_RED,2*this->board_sowing_delay);
}


/**
* Takes all the seeds in the specified cell and execute a regular Mancala sow movement.
* @param boardside Side of the board (BRD_SIDE_NORTH, BRD_SIDE_SOUTH).
* @param cellx Xpos of the desired cell in the bcells array.
* @param celly Ypos of the desired cell in the bcells array.
* @return 1 on success, -1 on any error
*/
int ow_board::move(int boardside, int cellx, int celly) {
	int ferror;
	//cout << "owboard->move" << endl;	
	if ((ferror = this->checkmove(boardside,cellx,celly))<0) { 
		this->display_errormove(boardside,cellx,celly);return ferror;
	}
	this->movecurrent++;this->movecount = this->movecurrent;
	this->saveundo();
	return this->_move(boardside,cellx,celly);
}

/**
* Private move(), used by ow_board::move() to make chain moves.
* @param boardside Side of the board (BRD_SIDE_NORTH, BRD_SIDE_SOUTH).
* @param cellx Xpos of the desired cell in the bcells array.
* @param celly Ypos of the desired cell in the bcells array.
* @return 1 on success, -1 on any error
*/
int ow_board::_move(int boardside, int cellx, int celly) {
	int ferror;
	int actiondir=-1;

	//cout << "owboard->_move" << endl;

	this->moveini.x = cellx; this->moveini.y = celly; this->moveini.boardside = boardside;
	
	if ((ferror = this->checkmove(boardside,cellx,celly))<0) { 
		this->display_errormove(boardside,cellx,celly);return ferror;
	}
	this->bcells[boardside][cellx][celly]->setstatus(CELL_STATUS_NORM,-1);
	//Set action to ini cell
	if (moveini.boardside==BRD_SIDE_SOUTH) { if (moveini.y==BRD_INNER_ROW) actiondir=CELL_ACTION_MOVEL; else actiondir=CELL_ACTION_MOVER;}
	if (moveini.boardside==BRD_SIDE_NORTH) { if (moveini.y==BRD_INNER_ROW) actiondir=CELL_ACTION_MOVER; else actiondir=CELL_ACTION_MOVEL;}
	this->bcells[moveini.boardside][moveini.x][moveini.y]->setaction(actiondir,-1);


	//cout << "Sowing " << this->countseeds(boardside,cellx,celly) << " seeds ..." << endl;
	this->takeseeds(boardside,cellx,celly);
	this->movestep = this->moveini;
	this->status = BRD_STATUS_SOWING;
	this->sowing_delay = this->board_sowing_delay;
	return 1;
}

/**
* Move_d(irect): Makes the full move (chainmoves,captures,etc) before returning.
* Same that move() but doing the full move before returning, move() needs various update_status()
* calls to complete movement.
* Note: The Chainmove is limited to 100.
* @param boardside Side of the board (BRD_SIDE_NORTH, BRD_SIDE_SOUTH).
* @param cellx Xpos of the desired cell in the bcells array.
* @param celly Ypos of the desired cell in the bcells array.
* @return 1 on success, -1 on any error
*/
int ow_board::move_d(int boardside, int cellx, int celly) {
	int savedspeed;
	int bstatus, chaincount;
	if (this->move(boardside, cellx, celly)!=-1) {
		savedspeed = this->board_sowing_delay;this->setboardspeed(BRD_SOWING_SPEED_ULTRA);
		chaincount=0;
		while (!this->endofturn()) {
			bstatus=this->update_status();
			if (bstatus==BRD_STATUS_CHAINMOVE) chaincount++;
			if (chaincount>1000) {this->setboardspeed(savedspeed);cout << "move(chainout)" << endl;return -1;}
		}
		this->setboardspeed(savedspeed);
		//if (chaincount<1000) 
		return 1;
	}
	return -1;
}


/**
* Takes all the seeds in the specified cell and execute a 'reverse' Mancala sow movement.
* It checks if the movement is allowed, only allowed if the reverse movement ends with a capture.
* @param boardside Side of the board (BRD_SIDE_NORTH, BRD_SIDE_SOUTH).
* @param cellx Xpos of the desired cell in the bcells array.
* @param celly Ypos of the desired cell in the bcells array.
* @return 1 on success, -1 on any error
*/
int ow_board::rcapture(int boardside, int cellx, int celly) {
	int ferror;
	int actiondir=-1;

	this->moveini.x = cellx; this->moveini.y = celly; this->moveini.boardside = boardside;
	
	if ((ferror = this->checkrcapture(boardside,cellx,celly))<0) { 
		this->display_errormove(boardside,cellx,celly);return ferror;
	}
	this->movecurrent++;this->movecount = this->movecurrent;
	this->saveundo();
	this->bcells[boardside][cellx][celly]->setstatus(CELL_STATUS_NORM,-1);
	//Set action to ini cell
	if (moveini.boardside==BRD_SIDE_SOUTH) { if (moveini.y==BRD_INNER_ROW) actiondir=CELL_ACTION_MOVER; else actiondir=CELL_ACTION_MOVEL;}
	if (moveini.boardside==BRD_SIDE_NORTH) { if (moveini.y==BRD_INNER_ROW) actiondir=CELL_ACTION_MOVEL; else actiondir=CELL_ACTION_MOVER;}
	this->bcells[moveini.boardside][moveini.x][moveini.y]->setaction(actiondir,-1);


	//cout << "Sowing " << this->countseeds(boardside,cellx,celly) << " seeds ..." << endl;
	this->takeseeds(boardside,cellx,celly);
	this->movestep = this->moveini;
	this->status = BRD_STATUS_REVSOWING;
	this->sowing_delay = this->board_sowing_delay;
	return 1;
}

/**
* Rcapture_d(irect): Starts a rcapture and makes the full move (chainmoves,captures,etc) before returning.
* Same that rcapture() but doing the full move before returning, rcapture() needs various update_status()
* calls to complete movement.
* Note: The Chainmove is limited to 100.
* @param boardside Side of the board (BRD_SIDE_NORTH, BRD_SIDE_SOUTH).
* @param cellx Xpos of the desired cell in the bcells array.
* @param celly Ypos of the desired cell in the bcells array.
* @return 1 on success, -1 on any error
*/
int ow_board::rcapture_d(int boardside, int cellx, int celly) {
	int savedspeed;
	int bstatus, chaincount;
	//return -1;
	if (this->rcapture(boardside, cellx, celly)!=-1) {
		savedspeed = this->board_sowing_delay;this->setboardspeed(BRD_SOWING_SPEED_ULTRA);
		chaincount=0;
		while (!this->endofturn()) {
			bstatus=this->update_status();
			if (bstatus==BRD_STATUS_CHAINMOVE) chaincount++;
			if (chaincount>1000) {this->setboardspeed(savedspeed);cout << "rcapture(chainout)" << endl; return -1;}
		}
		this->setboardspeed(savedspeed);
		//if (chaincount<1000) 
		return 1;
	}
	return -1;
}


/**
* Check if starting a movement from the specified cell is a legal movement.
* @param boardside Side of the board (BRD_SIDE_NORTH, BRD_SIDE_SOUTH).
* @param cellx Xpos of the specified cell in the bcells array.
* @param celly Ypos of the specified cell in the bcells array.
* @return 1 on legal movement, -1 on illegal movement
*/
int ow_board::checkmove(int boardside, int cellx, int celly) {
	if (this->bcells[boardside][cellx][celly]->countseeds()<=1) { return -1; } // Solo hay 1 semilla, no mover.
	return 1;
}

/**
* Check if starting a reverse capture from the specified cell is a legal movement.
* Reverse movement is only allowed from 4 cells most left in each player board side.
* @param boardside Side of the board (BRD_SIDE_NORTH, BRD_SIDE_SOUTH).
* @param cellx Xpos of the specified cell in the bcells array.
* @param celly Ypos of the specified cell in the bcells array.
* @return 1 on legal movement, -1 on illegal movement
*/
int ow_board::checkrcapture(int boardside, int cellx, int celly) {
	int rseeds, markerseeds;
	cellXY movetmp;
	int rboardside, enemy_boardside;
	int opposite_colx;


	if (this->bcells[boardside][cellx][celly]->countseeds()<=1) { return -1; } // Only one seed, not allowed.
	if (cellx>1) {return -1;} // Reverse movement only allowed from 4 cells from the left.

	rseeds = this->bcells[boardside][cellx][celly]->countseeds();
	movetmp.x = cellx; movetmp.y = celly; movetmp.boardside = boardside;
  for (int idx=0;idx<rseeds;idx++) {
		//cout << "1"; 
		if (movetmp.y==BRD_OUTER_ROW) {
			movetmp.x--;
			if (movetmp.x<0) { movetmp.x=0;movetmp.y=BRD_INNER_ROW; }
		} else {
			movetmp.x++;
			if (movetmp.x>((this->brd_num_cols)-1)) { movetmp.x = ((this->brd_num_cols)-1); movetmp.y=BRD_OUTER_ROW; }
		}
	}
	//cout << endl;
	//cout << "movetmp.end(" << movetmp.x << "," << movetmp.y << ")" << endl;
	if (this->bcells[boardside][movetmp.x][movetmp.y]->countseeds()<1) {return -1;} // if no marker no capture

	//if both opposite cells are not empty then capture allowed  
	rboardside = movetmp.boardside;
	if (rboardside==BRD_SIDE_NORTH) enemy_boardside=BRD_SIDE_SOUTH; else enemy_boardside=BRD_SIDE_NORTH;
	opposite_colx = (this->brd_num_cols)-movetmp.x-1;
	if ((this->bcells[enemy_boardside][opposite_colx][BRD_OUTER_ROW]->countseeds()<1)||
		  (this->bcells[enemy_boardside][opposite_colx][BRD_INNER_ROW]->countseeds()<1)) {
		return -1; // At least one of the opposite cells are empty, so no capture.
	}

	return 1;
}

/**
* Executes a Capture in the cell where the last sow movement ended.
* @return Seeds of enemy captured cells taken, and a BRD_STATUS_REENTER_CAPTURE event launched.
* @return 1 on success
*/
int ow_board::capture() {
	int tablado, enemy_tablado;
	int opposite_colx;

	tablado = this->moveini.boardside;
	if (tablado==BRD_SIDE_NORTH) enemy_tablado=BRD_SIDE_SOUTH; else enemy_tablado=BRD_SIDE_NORTH;
	opposite_colx = (this->brd_num_cols)-this->movestep.x-1;
	//capture marker cell
	this->bcells[tablado][this->movestep.x][this->movestep.y]->setstatus(CELL_STATUS_GREEN,-1);
	this->bcells[tablado][this->movestep.x][this->movestep.y]->setaction(CELL_ACTION_CANCEL,-1);
	// capture ini cell
	this->bcells[tablado][this->moveini.x][this->moveini.y]->setstatus(CELL_STATUS_GREEN,-1);
	this->bcells[tablado][this->moveini.x][this->moveini.y]->setaction(CELL_ACTION_SELECT,-1);
	// captured cells
	this->bcells[enemy_tablado][opposite_colx][BRD_INNER_ROW]->setstatus(CELL_STATUS_RED,-1);
	this->bcells[enemy_tablado][opposite_colx][BRD_INNER_ROW]->setaction(CELL_ACTION_CAPTURE,-1);
	this->bcells[enemy_tablado][opposite_colx][BRD_OUTER_ROW]->setstatus(CELL_STATUS_RED,-1);
	this->bcells[enemy_tablado][opposite_colx][BRD_OUTER_ROW]->setaction(CELL_ACTION_CAPTURE,-1);

	//cout << "Capturing " << this->countseeds(enemy_tablado,opposite_colx,BRD_INNER_ROW)+this->countseeds(enemy_tablado,opposite_colx,BRD_OUTER_ROW) << " seeds ..." << endl;
	//cout << "Reentering " << this->countseeds(enemy_tablado,opposite_colx,BRD_INNER_ROW)+this->countseeds(enemy_tablado,opposite_colx,BRD_OUTER_ROW) << " seeds ..." << endl;
	this->takeseeds(enemy_tablado,opposite_colx,BRD_INNER_ROW);
	this->takeseeds(enemy_tablado,opposite_colx,BRD_OUTER_ROW);

	this->sowing_delay = 2*(this->board_sowing_delay);
	this->status = BRD_STATUS_REENTER_CAPTURE;

	return 1;
}

/**
* Remove all seeds from the specified cell, save them in the private 'this->handseeds' variable.
* @param boardside Side of the board (BRD_SIDE_NORTH, BRD_SIDE_SOUTH).
* @param cellx Xpos of the desired cell in the bcells array.
* @param celly Ypos of the desired cell in the bcells array.
* @return Number of seeds taken from the cell, -1 on error.
*/
int ow_board::takeseeds(int boardside, int cellx, int celly) {
	this->ntakeseeds(this->bcells[boardside][cellx][celly]->countseeds(),boardside,cellx,celly);
	return this->handseeds;
}

/**
* Remove N seeds from the specified cell, save them in the private 'this->handseeds' variable.
* @param seeds Number of seeds to be taken from the cell.
* @param boardside Side of the board (BRD_SIDE_NORTH, BRD_SIDE_SOUTH).
* @param cellx Xpos of the desired cell in the bcells array.
* @param celly Ypos of the desired cell in the bcells array.
* @return Number of seeds taken from the cell, -1 if cell does not have enought seeds to take.
*/
int ow_board::ntakeseeds(int seeds, int boardside, int cellx, int celly) {
	if (seeds<=this->bcells[boardside][cellx][celly]->countseeds()) {
		this->handseeds += seeds;
		this->bcells[boardside][cellx][celly]->takeseeds(seeds);
		return this->handseeds;
	}
	return -1;
}

/**
* Put seeds in the specified cell, taking them from private 'this->handseeds' variable.
* @param seeds Number of seeds to put in the specified cell
* @param boardside Side of the board (BRD_SIDE_NORTH, BRD_SIDE_SOUTH).
* @param cellx Xpos of the specified cell in the bcells array.
* @param celly Ypos of the specified cell in the bcells array.
* @return 1 on success, <=0 on error.
*/
int ow_board::nputseeds(int seeds, int boardside, int cellx, int celly) {
	if (seeds<= this->handseeds) {
		this->bcells[boardside][cellx][celly]->addseeds(seeds);
		this->bcells[boardside][cellx][celly]->setaction(CELL_ACTION_SELECT,-1);
		this->bcells[boardside][cellx][celly]->setstatus(CELL_STATUS_GREEN,-1);
		this->handseeds-=seeds;
		return 1;
	}
	return 0;
}

/**
* Returns the count of seeds in the specified cell.
* @param boardside Side of the board (BRD_SIDE_NORTH, BRD_SIDE_SOUTH).
* @param cellx Xpos of the specified cell in the bcells array.
* @param celly Ypos of the specified cell in the bcells array.
* @return Number of seeds
*/
int ow_board::countseeds(int boardside,int cellx, int celly) {
	return this->bcells[boardside][cellx][celly]->countseeds();
}

/**
* Returns the count of seeds in the specified board side.
* @param boardside Side of the board (BRD_SIDE_NORTH, BRD_SIDE_SOUTH).
* @return Number of seeds
*/
int ow_board::countseeds(int boardside) {
	int idx;
	int bcount = 0;
	for (idx=0;idx<(this->brd_num_cols); idx++) {
		bcount+=this->bcells[boardside][idx][BRD_INNER_ROW]->countseeds();
		bcount+=this->bcells[boardside][idx][BRD_OUTER_ROW]->countseeds();
	}
 	return bcount;
}


/**
* The board always keep a copy of the initial status of the cells of the board.
* This function returns the count of seeds in the specified cell, from the copy of the initial board status.
* You can get board initial status reading the content of all its cells with this function.
* To get the countseeds of cell of the board current status, you can use regular countseeds() instead.
* Note: This works saving initial board status to the undoqueue, so if you have undo option disabled probably it will not 
* work properly.
* @param undostep (0-N) specified the slot in the undo queue you want to query about.
* @param boardside Side of the board (BRD_SIDE_NORTH, BRD_SIDE_SOUTH).
* @param cellx Xpos of the specified cell in the board according to the undodata
* @param celly Ypos of the specified cell in the board according to the undodata
* @return Number of seeds, -1 on error (undo inactive, so no previous status data available)
*/
int ow_board::initialboard_countseeds(int boardside, int cellx, int celly) {
	if ((this->movecurrent < BRD_MAX_UNDO_MOVES)&&(this->movecurrent>=1)&&(this->undo_active)) {
		return undodata[0][boardside][cellx][celly];
	}
	if (this->movecount==0) {
		return this->countseeds(boardside,cellx,celly);
	}	
  return -1;
}

/**
* The board always keep a copy of the initial status of the cells of the board.
* This function returns the board to its initial status
* Note: This works saving initial board status to the undoqueue, so if you have undo option disabled probably it will not 
* work properly.
*/
int ow_board::initialboard_reset() {
	return this->undolastmove(this->movecurrent);
}

/**
* Set the current status of the board as 'initialboard'.
* As consecuence it deletes the undo queue if exists.
*/
int ow_board::initialboard_setcurrent() {
	this->movecount = 0; this->movecurrent = 0;
}

/**
* Setup board: Take N seeds from the specified cell, save them in the private 'this->handseeds' variable.
* @param seeds Number of seeds to be taken from the cell.
* @param boardside Side of the board (BRD_SIDE_NORTH, BRD_SIDE_SOUTH).
* @param cellx Xpos of the desired cell in the bcells array.
* @param celly Ypos of the desired cell in the bcells array.
* @return Number of seeds taken from the cell, -1 if cell does not have enought seeds to take.
*/
int ow_board::bsetup_takeseeds(int seeds, int boardside, int cellx, int celly) {
	int result;
	result = this->ntakeseeds(seeds,boardside, cellx, celly);
	this->bcells[boardside][cellx][celly]->setaction(CELL_ACTION_TAKE,2*this->board_sowing_delay);
	if (result>0) {
		this->bcells[boardside][cellx][celly]->setstatus(CELL_STATUS_CYAN,2*this->board_sowing_delay);
	} else {
		this->bcells[boardside][cellx][celly]->setstatus(CELL_STATUS_RED,2*this->board_sowing_delay);
	}
	return result;
}

/**
* Setup board: Put seeds in the specified cell. Controls if maxnumseeds on board has been reached.
* @param seeds Number of seeds to put in the specified cell
* @param boardside Side of the board (BRD_SIDE_NORTH, BRD_SIDE_SOUTH).
* @param cellx Xpos of the specified cell in the bcells array.
* @param celly Ypos of the specified cell in the bcells array.
* @return 1 on success, <=0 on error, if no more seeds allowed on board.
*/
int ow_board::bsetup_putseeds(int seeds, int boardside, int cellx, int celly) {
	int result;
	result = this->nputseeds(seeds,boardside, cellx, celly);
	this->bcells[boardside][cellx][celly]->setaction(CELL_ACTION_PUT,2*this->board_sowing_delay);		
	if (result>0) {
		this->bcells[boardside][cellx][celly]->setstatus(CELL_STATUS_CYAN,2*this->board_sowing_delay);
	} else {
		this->bcells[boardside][cellx][celly]->setstatus(CELL_STATUS_RED,2*this->board_sowing_delay);
	}
	return result;
}



/**
* Clears the cells status/action used by draw_gfx for the last movement.
*/
void ow_board::clear_status_actions() {
  int cellx,celly;
	
	//if (!this->sowing_delay) {
		if (this->status!=BRD_STATUS_ERRORMOVE) {
			for (cellx=0;cellx<(this->brd_num_cols);cellx++)  {
				for (celly=0;celly<BRD_ROWS_PER_PLAYER;celly++) {
					this->bcells[BRD_SIDE_NORTH][cellx][celly]->setstatus(CELL_STATUS_NORM,-1);
					this->bcells[BRD_SIDE_NORTH][cellx][celly]->setaction(CELL_ACTION_NONE,-1);
					this->bcells[BRD_SIDE_SOUTH][cellx][celly]->setstatus(CELL_STATUS_NORM,-1);
					this->bcells[BRD_SIDE_SOUTH][cellx][celly]->setaction(CELL_ACTION_NONE,-1);
				}
			}
		}
	//}
}

/**
* Update the cells status/action used by draw_gfx for the last movement.
* Basically used to update the cells who defined a 'timeout' for setaction() and setstatus().
*/
void ow_board:: update_status_actions() {
  int cellx,celly;
	for (cellx=0;cellx<(this->brd_num_cols);cellx++)  {
		for (celly=0;celly<BRD_ROWS_PER_PLAYER;celly++) {
			this->bcells[BRD_SIDE_NORTH][cellx][celly]->update_status_action();
			this->bcells[BRD_SIDE_SOUTH][cellx][celly]->update_status_action();
		}
	}
}

/**
* Executes the user commands in a delayed way, so the user is NOT blocked while the command ends.
* @return status of the board (BRD_STATUS_CHAINMOVE,BRD_STATUS_CAPTURE,BRD_STATUS_REENTER_CAPTURE, etc)
*/
int ow_board::update_status() {
	int idx;
	int enemy_tablado, opposite_colx;

	//Error no se puede mover
	if ((this->status==BRD_STATUS_ERRORMOVE)&&(!this->sowing_delay)) {
		this->status=BRD_STATUS_READY;this->turnchains=0; return -1;
	}
	
	if ((this->status==BRD_STATUS_CHAINMOVE)&&(!this->sowing_delay)) {
		this->clear_status_actions();this->turnchains++;
		this->_move(moveini.boardside, movestep.x, movestep.y); return 0;
	}
	
	if ((this->status==BRD_STATUS_CAPTURE)&&(!this->sowing_delay)) {
		this->clear_status_actions();
		this->capture(); 
		return 0;
	}

	if ((this->status==BRD_STATUS_REENTER_CAPTURE)&&(!this->sowing_delay)) {
		this->clear_status_actions();
		this->nputseeds(1,moveini.boardside,moveini.x,moveini.y);
		this->movestep = this->moveini;	
		this->status = BRD_STATUS_SOWING;
		this->sowing_delay=this->board_sowing_delay;
	}


	// Moving, sowing	
	if ((this->status==BRD_STATUS_SOWING)&&(!this->sowing_delay)) {
		if (this->handseeds==0) { this->status=BRD_STATUS_READY;this->turnchains=0; this->clear_status_actions(); return -1;}
		//Set delay for next semilla
		if (this->handseeds>1) this->sowing_delay=this->board_sowing_delay; else this->sowing_delay=2*(this->board_sowing_delay);

		if (movestep.y==BRD_INNER_ROW) {
			movestep.x--;
			if (movestep.x<0) { movestep.x=0;movestep.y=BRD_OUTER_ROW; }
		} else {
			movestep.x++;
			if (movestep.x>((this->brd_num_cols)-1)) { movestep.x = ((this->brd_num_cols)-1); movestep.y=BRD_INNER_ROW; }
		}
		this->nputseeds(1,moveini.boardside,movestep.x,movestep.y);

		//Aqui enlazar movimiento y/o capturar
		if ((this->handseeds==0)&&(this->countseeds(moveini.boardside,movestep.x,movestep.y)>1)) {
			this->status = BRD_STATUS_CHAINMOVE; //cout << "BRD_STATUS_CHAINMOVE" << endl;
			if (moveini.boardside==BRD_SIDE_NORTH) enemy_tablado=BRD_SIDE_SOUTH; else enemy_tablado=BRD_SIDE_NORTH;
			opposite_colx = (this->brd_num_cols)-movestep.x-1;
			if ((movestep.y==BRD_INNER_ROW)&&(this->countseeds(enemy_tablado,opposite_colx,BRD_INNER_ROW)>0)&&(this->countseeds(enemy_tablado,opposite_colx,BRD_OUTER_ROW)>0)) {
				this->status = BRD_STATUS_CAPTURE;
			}
		}

	} else {if (this->sowing_delay>0) {this->sowing_delay--;}}


	// Reverse movement. Reverse capture.
	if ((this->status==BRD_STATUS_REVSOWING)&&(!this->sowing_delay)) {
		if (this->handseeds==0) { this->status=BRD_STATUS_READY;this->turnchains=0; this->clear_status_actions(); return -1;}
		//Set delay for next semilla
		if (this->handseeds>1) this->sowing_delay=this->board_sowing_delay; else this->sowing_delay=2*(this->board_sowing_delay);

		if (movestep.y==BRD_OUTER_ROW) {
			movestep.x--;
			if (movestep.x<0) { movestep.x=0;movestep.y=BRD_INNER_ROW; }
		} else {
			movestep.x++;
			if (movestep.x>((this->brd_num_cols)-1)) { movestep.x = ((this->brd_num_cols)-1); movestep.y=BRD_OUTER_ROW; }
		}
		this->nputseeds(1,moveini.boardside,movestep.x,movestep.y);

		//Aqui enlazar movimiento y/o capturar
		if ((this->handseeds==0)&&(this->countseeds(moveini.boardside,movestep.x,movestep.y)>1)) {
			this->status = BRD_STATUS_CHAINMOVE;
			if (moveini.boardside==BRD_SIDE_NORTH) enemy_tablado=BRD_SIDE_SOUTH; else enemy_tablado=BRD_SIDE_NORTH;
			opposite_colx = (this->brd_num_cols)-movestep.x-1;
			if ((movestep.y==BRD_INNER_ROW)&&(this->countseeds(enemy_tablado,opposite_colx,BRD_INNER_ROW)>0)&&(this->countseeds(enemy_tablado,opposite_colx,BRD_OUTER_ROW)>0)) {
				this->status = BRD_STATUS_CAPTURE;
			}
		}

	} else {if (this->sowing_delay>0) {this->sowing_delay--;}}

	// Update the status and actions for the cells with timeout defined for setstatus() and setaction()
	this->update_status_actions();

	return (this->status);
}

/**
* Check if the board has finished executing last user commands and is ready for new commands.
* @return true if finished, false if board still executing last command.
*/
int ow_board::endofturn() {
	return (this->status==BRD_STATUS_READY);
}

/**
*	Checks if there is any allowed moves for the player with the turn.
* @param boardside The side of the board you want to check if there is remaining allowed movements.
*/
int ow_board::checkavailablemoves(int boardside) {

	//cout << "checkavailablemoves(" << boardside <<")";
	int cellx,celly;
	for (cellx=0;cellx<(this->brd_num_cols);cellx++)  {
		for (celly=0;celly<BRD_ROWS_PER_PLAYER;celly++) {
			if (this->bcells[boardside][cellx][celly]->countseeds()>1) {/*cout<< 1 << endl;*/return 1;}
		}
	}
	/*cout<< 0 << endl;*/return 0;
}

/**
*	Activate/Deactivate the UndoLastmove option. If deactived the board status is not saved 
* so UndoMove cannot be performed.
* @param status  1 to activate undo_mode, 0 to deactivate undo_mode.
*/
void ow_board::set_undo_mode(int status) {
	this->undo_active = status;
}

/**
*	Return the board to the status previous to the last move.
* @param nummoves Number of moves to undo.
* @return 1 is successful, -1 if cannot undo.
*/
int ow_board::undolastmove(int nummoves) {
	int cellx,celly;
	if ((this->movecurrent < BRD_MAX_UNDO_MOVES)&&(this->movecurrent>=nummoves)&&(this->undo_active)) {
		for (cellx=0;cellx<(this->brd_num_cols);cellx++)  {
			for (celly=0;celly<BRD_ROWS_PER_PLAYER;celly++) {
				this->setcellseeds(this->undodata[(this->movecurrent)-nummoves][BRD_SIDE_NORTH][cellx][celly], BRD_SIDE_NORTH, cellx, celly);
				this->setcellseeds(this->undodata[(this->movecurrent)-nummoves][BRD_SIDE_SOUTH][cellx][celly], BRD_SIDE_SOUTH, cellx, celly);
			}
		}
		this->movecurrent-=nummoves;
		return 1;
	}
	cout << "cannot undo" << endl;
	return -1;
}


/**
*	Save the cells content info to a backup private variable (undodata array) for future use with
* the undolasmove() function.
* @return 1 is successful, -1 if cannot save info.
*/
int ow_board::saveundo() {
	int cellx,celly;
	if ((this->undo_active)&&(this->movecurrent < BRD_MAX_UNDO_MOVES)) {
	  //cout << "saveundo" << endl;
		for (cellx=0;cellx<(this->brd_num_cols);cellx++)  {
			for (celly=0;celly<BRD_ROWS_PER_PLAYER;celly++) {
				this->undodata[(this->movecurrent)-1][BRD_SIDE_NORTH][cellx][celly] = this->bcells[BRD_SIDE_NORTH][cellx][celly]->countseeds();
				this->undodata[(this->movecurrent)-1][BRD_SIDE_SOUTH][cellx][celly] = this->bcells[BRD_SIDE_SOUTH][cellx][celly]->countseeds();
			}
		}
		return 1;
	}
	return 0;
}

/**
*	Save the cells content info to a backup private variable (backupdata array).
* You can use this function in conjunction with restorebackup to temporary save/restore
* the cell contents of the board with any purpose you like.
* @return 1 is successful, -1 if cannot save info.
*/
int ow_board::savebackup() {
	int cellx,celly;
	for (cellx=0;cellx<(this->brd_num_cols);cellx++)  {
		for (celly=0;celly<BRD_ROWS_PER_PLAYER;celly++) {
			this->backupdata[BRD_SIDE_NORTH][cellx][celly] = this->bcells[BRD_SIDE_NORTH][cellx][celly]->countseeds();
			this->backupdata[BRD_SIDE_SOUTH][cellx][celly] = this->bcells[BRD_SIDE_SOUTH][cellx][celly]->countseeds();
		}
	}
	this->backupsaved = 1;
	return 1;
}

/**
*	Restore the cells content info saved previously by savebackup()
* You can use this function in conjunction with savebackup to temporary save/restore
* the cell contents of the board with any purpose you like.
* @return 1 is successful, -1 if cannot restore info, no previously saved backup data.
*/
int ow_board::restorebackup() {
	int cellx,celly;
	if (!this->backupsaved) return -1;

	for (cellx=0;cellx<(this->brd_num_cols);cellx++)  {
		for (celly=0;celly<BRD_ROWS_PER_PLAYER;celly++) {
			this->setcellseeds(this->backupdata[BRD_SIDE_NORTH][cellx][celly], BRD_SIDE_NORTH, cellx, celly);
			this->setcellseeds(this->backupdata[BRD_SIDE_SOUTH][cellx][celly], BRD_SIDE_SOUTH, cellx, celly);
		}
	}
	return 1;
}


/**
* Gets the name of the board cell corresponding to the given cell coords.
* @param boardside Side of the board (BRD_SIDE_NORTH, BRD_SIDE_SOUTH).
* @param cellx Xpos of the specified cell in the bcells array.
* @param celly Ypos of the specified cell in the bcells array.
* @returns The name of the cell
*/
string ow_board::translate_coord2name(int boardside, int cellx, int celly) {
	string coord2name_translations[BRD_NUM_SIDES][2][8] = {
		{{"A0","B0","C0","D0","E0","F0","G0","H0"},{"A1","B1","C1","D1","E1","F1","G1","H1"}},
		{{"a0","b0","c0","d0","e0","f0","g0","h0"},{"a1","b1","c1","d1","e1","f1","g1","h1"}}
	};
	if ((boardside>=0)&&(boardside<BRD_NUM_SIDES)&&(celly<2)&&(cellx<8))
		return coord2name_translations[boardside][celly][cellx];
  else return "";
}


/**
* Gets the number of columns the board has.
* @return Number of columns.
*/
int ow_board::getcolcount() {
	return this->brd_num_cols;
}

/**
* Gets the number of seeds in the 'this->handseeds' variable.
* @return Number of seeds.
*/
int ow_board::counthandseeds() {
	return this->handseeds;
}

/**
* Set the number of seeds of the specified cell, overriding the previous value.
* This is used during board setup, while putseeds() is used during playgame and checks play rules.
* @param seeds Number of seeds to put in the specified cell
* @param boardside Side of the board (BRD_SIDE_NORTH, BRD_SIDE_SOUTH).
* @param cellx Xpos of the specified cell in the bcells array.
* @param celly Ypos of the specified cell in the bcells array.
* @return 1 on success, 0 on error.
*/
int ow_board::setcellseeds(int seeds, int boardside, int cellx, int celly) {
	this->bcells[boardside][cellx][celly]->emptyseeds();
	this->bcells[boardside][cellx][celly]->addseeds(seeds);
	return 1;
}

/*
int ow_board::setcellstatus(int status, int boardside, int cellx, int celly) {
	this->bcells[boardside][cellx][celly]->setstatus(status,2*this->board_sowing_delay);
}

int ow_board::setcellaction(int action, int boardside, int cellx, int celly) {
	this->bcells[boardside][cellx][celly]->setaction(action,2*this->board_sowing_delay);
}
*/

/**
* Set the speed of the board.
* Usefull to make it faster during batch games, or slower during human games.
* @param bspeed Desired speed (BRD_SOWING_SPEED_USLOW,BRD_SOWING_SPEED_SLOW,BRD_SOWING_SPEED_NORM,BRD_SOWING_SPEED_FAST,BRD_SOWING_SPEED_ULTRA)
*/
void ow_board::setboardspeed(int bspeed) {
	this->board_sowing_delay	= bspeed;
}

