/***************************************************************************
 *   Copyright (C) 2007 by David Torres   *
 *   aiken669@gmail.com   *
 ***************************************************************************/

#include <readline/readline.h>
#include <readline/history.h>
#include <SDL/SDL_image.h>
#include <SDL/SDL_ttf.h>
#include <iostream>
#include <fstream>
#include <string>
#include "ow_game.h"
#include "config.h"

using namespace std;

/**
* Creates a new 'ow_game' object, with the default values.
* @return New 'ow_game' object.
*/
ow_game::ow_game()
{
	this->status = OWGAME_STATUS_STOPPED;
	this->current_turn = 1;
	this->gfxboard_hidden = 1;
	this->gfxboard_dataloaded = 0;
	this->current_menu = OWGAME_MENU_APP;
	this->commandline_handler = NULL;
	this->commandline_handler_null = NULL;
	this->set_default_gameconfig(NULL);
	this->logmovescurrent = 0;
	this->logmovescount = 0;
	this->queue_pos = 0; this->queue_last = 0;
	this->logresults = "OFF";
	this->logresultsfile = "ow_results.log";
}

/**
* Sets the default values for game config (player names, numcols, etc)
* @param config If NULL the default values are applied to this->config, 
* if param not null, given variable is filled with default values
*	
*/
void ow_game::set_default_gameconfig(game_configuration *config) {
	game_configuration *cfg;
	if (config!=NULL) cfg = config; else cfg = &(this->config);
	cfg->p2name = "player2"; 
	cfg->p2aimode = "SIMPLE"; cfg->p2aidepth = "1"; cfg->p2aieval = "RANDOM";
	cfg->p1name = "player1"; 
	cfg->p1aimode = "NEGAMAX"; cfg->p1aidepth = "3"; cfg->p1aieval = "MATERIAL";
	cfg->numcols = 8;
	cfg->fturn_player = "P1";
	cfg->turn = 1;
	cfg->plays = "P1";
	cfg->gfxboard = "ON";


}

/**
* Deletes an existing 'ow_game' object, including its related resources.
*/
ow_game::~ow_game()
{
	rl_callback_handler_remove();
	printf("\n");
	this->endgame();
}


/**
* Set the next turn for the specified player
* @param player Player who you want to set the next turn to.
*/
void ow_game::setnewturn(ow_player *player) {
	char prompt[64]; char *playername;
	this->selected_player = player;
	rl_callback_handler_install("", this->commandline_handler_null);
	this->ow_commandline_getpromptstring(prompt);
	if (player->getaimode()==AI_HUMAN) rl_callback_handler_install(prompt, this->commandline_handler);
	if (player->getaimode()!=AI_HUMAN) cout << prompt << endl;
	player->wakeup();
}

/**
* Set turn to the specified player, used to setup the first turn of the game.
* @param player Specified player.
*/
void ow_game::setfirstturn(ow_player *player) {
	this->selected_player = player;
	player->wakeup();
}

/**
* Ends the turn of the current player, and sets the turn to the next player.
*/
void ow_game::changeturn() {
	ow_player *selected_player;
	this->selected_player->gosleep();
	if (this->selected_player->getplayernum() == this->p1->getplayernum()) selected_player = this->p2;
	else { selected_player = this->p1; }

	if (!this->owtab->checkavailablemoves(selected_player->getboardside())) {
		this->status=OWGAME_STATUS_WON;
		//this->current_menu = OWGAME_MENU_APP;
		cout << "Game finished. Player [" << this->selected_player->getplayername() << "] WON!";
		//this->logmovesfile << "# " << "Game finished. Player [" << this->selected_player->getplayername() << "] WON!" << endl;
		this->ow_commandline_refreshprompt();
		this->loggameresults();
	} else {
		if ((this->config.fturn_player=="P1")&&(selected_player==this->p1)) { this->current_turn++;}
		if ((this->config.fturn_player=="P2")&&(selected_player==this->p2)) { this->current_turn++;}
		cout << "Done." << endl;
		this->setnewturn(selected_player);
	}
}

/**
* Prints out the version info of the app.
*/
void ow_game::show_version() {
  cout << "Omweso Shell v-" << VERSION << " compiled on " << __DATE__ << " Copyright (C) 2007 by David Torres" << endl;
}

/**
* Shows the graphical board (or hiddes it if it is already shown).
*/
int ow_game::gfxboard_showhidde() {
	if (this->gfxboard_hidden) {
  	if ((SDL_Init(SDL_INIT_VIDEO)< 0)||(TTF_Init()==-1)) {cout << endl <<"Could not initialize SDL:" <<SDL_GetError() << endl;SDL_Quit();} else {
			if (!this->gfxboard_dataloaded) {
				if (!(this->owtab->load_gfxdata()>=0)||!(this->load_gfxdata()>=0)) {
      		cout << endl << "Could not read gfx data." << endl;
      		SDL_Quit();
    		} else {this->gfxboard_dataloaded=1;}
			}
			int screenX = 72+((this->owtab->getcolcount())*(CELL_WIDTH+CELL_SPACING))+13+56;
			gfxboard_screen = SDL_SetVideoMode(screenX,480,8,SDL_HWSURFACE|SDL_DOUBLEBUF);
			SDL_Palette *plt = this->owtab->get_palette();
			SDL_SetPalette(gfxboard_screen, SDL_LOGPAL|SDL_PHYSPAL, plt->colors, 0, 256);
			if (this->gfxboard_dataloaded) {this->gfxboard_hidden=0;}
			//cout << "this->gfxboard_hidden =" << this->gfxboard_hidden << endl;
		}
	} else { // Hidde the gfxboard
		TTF_Quit(); SDL_Quit();
		this->gfxboard_hidden=1;
	}
}


/**
* Draws in the screen surface a frame for the graphical board, and game items.
*/
void ow_game::draw_gfx() {
	if (!this->gfxboard_hidden) {
		this->owtab->draw_gfx(this->gfxboard_screen);
		this->draw();
		SDL_Flip(this->gfxboard_screen);
	}
}

/**
* Draws the graphical items of the game. Usually called from draw_gfx()
*/
void ow_game::draw() {
  SDL_Rect src, dest;
	SDL_Color color={255,255,255};
	SDL_Surface *text_surface;
  char textmsg[100];

	if (!this->gamesettingup()) {
  //Draw Player Marks
		if (this->selected_player->getboardside()==BRD_SIDE_NORTH) {dest.x=10;dest.y=10;} 
		else {dest.x=10;dest.y=443;}//else {dest.x=72+((this->owtab->getcolcount()-1)*(CELL_WIDTH+CELL_SPACING));dest.y=445;}
		if (this->selected_player->getplayernum()==PL_PLAYER1) {src.y=0;} else {src.y=30;}
		src.w=27;src.h=30;src.x=0;
  	SDL_BlitSurface(this->gfxdata[OWGFX_PLAYERMARKER], &src, this->gfxboard_screen, &dest);

	  //Draw Player Names
		dest.x =44;
		text_surface=TTF_RenderText_Blended(this->font,p1->getplayername(),color);
		if (p1->getboardside()==BRD_SIDE_NORTH) {dest.y=4;} else {dest.y=435;}
		SDL_BlitSurface(text_surface,NULL,this->gfxboard_screen,&dest);
		SDL_FreeSurface(text_surface);

		text_surface=TTF_RenderText_Blended(this->font,p2->getplayername(),color);
		if (p2->getboardside()==BRD_SIDE_NORTH) {dest.y=4;} else {dest.y=435;}
		SDL_BlitSurface(text_surface,NULL,this->gfxboard_screen,&dest);
		SDL_FreeSurface(text_surface);

	}
	// Shows SETUP MODE message if setting up
	if (this->gamesettingup()) {
		dest.x=10; dest.y=3;
		text_surface=TTF_RenderText_Blended(this->font,"SETUP MODE",color);
		SDL_BlitSurface(text_surface,NULL,this->gfxboard_screen,&dest);
		SDL_FreeSurface(text_surface);

		sprintf(textmsg,"Seeds to put: %d", this->owtab->counthandseeds());
		dest.x=10; dest.y=436;
		text_surface =TTF_RenderText_Blended(this->font,textmsg,color);
		SDL_BlitSurface(text_surface,NULL,this->gfxboard_screen,&dest);
		SDL_FreeSurface(text_surface);
	}

	// Draw WON message if game is won 
	if (this->checkgamewon()) {
		//cout << "DRAW_GFX_WON!!" << endl;
		int screenw = 72+((this->owtab->getcolcount())*(CELL_WIDTH+CELL_SPACING))+13+56;
		src.w = 320; src.h = 50; dest.w = 320; dest.h = 50;
		dest.x = (screenw/2)-160;
		if (this->selected_player->getboardside()==BRD_SIDE_NORTH) { dest.y=117;src.x = 0; src.y = 50; } 
		else { dest.y=305;src.x = 0; src.y = 0; } 
		SDL_BlitSurface(this->gfxdata[OWGFX_GAMEWON], &src, this->gfxboard_screen, &dest);
	}

}


/**
* Load the graphical items data files for the game.
*/  
int ow_game::load_gfxdata() {
	this->gfxdata[OWGFX_PLAYERMARKER] = IMG_Load(DATAFOLDER("gfx/logo_1p2p.png"));
	this->gfxdata[OWGFX_GAMEWON] = IMG_Load(DATAFOLDER("gfx/gamewon.png"));
	this->font = TTF_OpenFont(DATAFOLDER("gfx/FreeSans.ttf"), 26);
	//SDL_SetColorKey(this->gfxdata[OWGFX_PLAYERMARKER],SDL_SRCCOLORKEY | SDL_RLEACCEL, 0x0);
	return 0;
}

/**
* Check if Mouse button has been clicked and if so gets the mouse_x,mouse_y values.
* @param mouse_x Xpos of the mouse pointer when the button was clicked.
* @param mouse_y Ypos of the mouse pointer when the button was clicked.
* @return 1 if mouse button clicked, 0 if mouse button not clicked.
*/
int ow_game::MouseLeftButtonClicked(int *mouse_x, int *mouse_y) {
	static int mouse_released=1;
	int mouse_status=0;

	if (!this->gfxboard_hidden) {
		SDL_PumpEvents();
		if (SDL_GetMouseState(mouse_x, mouse_y) & SDL_BUTTON(SDL_BUTTON_LEFT)) mouse_status=1;
		if (mouse_status==0) {mouse_released=1; return 0;}
		else {if (mouse_released) {mouse_released=0;return 1;} else {return 0;}}
	}	
	else return 0;
}

/**
* Check if Right Mouse button has been clicked and if so gets the mouse_x,mouse_y values.
* @param mouse_x Xpos of the mouse pointer when the button was clicked.
* @param mouse_y Ypos of the mouse pointer when the button was clicked.
* @return 1 if right mouse button clicked, 0 if mouse button not clicked.
*/
int ow_game::MouseRightButtonClicked(int *mouse_x, int *mouse_y) {
	static int mouse_released=1;
	int mouse_status=0;

	if (!this->gfxboard_hidden) {
		SDL_PumpEvents();
		if (SDL_GetMouseState(mouse_x, mouse_y) & SDL_BUTTON(SDL_BUTTON_RIGHT)) mouse_status=1;
		if (mouse_status==0) {mouse_released=1; return 0;}
		else {if (mouse_released) {mouse_released=0;return 1;} else {return 0;}}
	}	
	else return 0;
}


/**
* Used to get the board cell 'under' the mouse cursor when the mouse button was clicked.
* @param coordx Xpos of the mouse pointer when the button was clicked.
* @param coordy Ypos of the mouse pointer when the button was clicked.
* @return info of the cell under the mouse cursor (x,y,boardside).
*/
ow_move ow_game::seleccionarcelda_bycoord(int coordx, int coordy) {
	int tablado;
	int cellx, celly;
	ow_move selected_cell;

	selected_cell.x = -1;
	selected_cell.y = -1;

	if (coordy>=owtab->bcells[BRD_SIDE_SOUTH][0][0]->getcellcoordXY().y) tablado = BRD_SIDE_SOUTH;
	else tablado = BRD_SIDE_NORTH;
	selected_cell.boardside = tablado;

	for (cellx=0;cellx<owtab->getcolcount();cellx++) {
		for (celly=0;celly<BRD_ROWS_PER_PLAYER;celly++) {
			if (owtab->bcells[tablado][cellx][celly]->isOverCoordXY(coordx,coordy)) {
				selected_cell.x = cellx;
				selected_cell.y = celly;
				break; //found
			}
    }
  }
	if (selected_cell.x<0) selected_cell.boardside = -1;
	return selected_cell;
}

/**
* Returns true if the game is running
* 'Running' is one of the game posible status.(OWGAME_STATUS_STOPPED,OWGAME_STATUS_RUNNING,OWGAME_STATUS_CREATED,OWGAME_STATUS_PLAYING)
* @return 1 if running, 0 if not running.
*/
int ow_game::gamerunning() {
	if (this->status>=OWGAME_STATUS_RUNNING) return 1;
	return 0;
}

/**
* Returns true if the game is created
* 'Created' is one of the game posible status.(OWGAME_STATUS_STOPPED,OWGAME_STATUS_RUNNING,OWGAME_STATUS_CREATED,OWGAME_STATUS_PLAYING)
* @return 1 if created, 0 if not created.
*/
int ow_game::gamecreated() {
	if (this->status>=OWGAME_STATUS_CREATED) return 1;
	return 0;
}

/**
* Returns true if the game is being played.
* 'Playing' is one of the game posible status.(OWGAME_STATUS_STOPPED,OWGAME_STATUS_RUNNING,OWGAME_STATUS_CREATED,OWGAME_STATUS_PLAYING)
* @return 1 if being played, 0 if not being played.
*/
int ow_game::gameplaying() {
	if ((this->status==OWGAME_STATUS_PLAYING)&&(this->current_menu==OWGAME_MENU_PLY)) return 1;
	else return 0;
}

/**
* Checks if we are in the setup board mode.
* @return 1 if setting up, 0 if not.
*/
int ow_game::gamesettingup() {
	if (this->current_menu==OWGAME_MENU_SETUPBOARD) return 1;
	else return 0;
}

/**
* Sets the application to the 'Running' status so it starts running.
* 'Running' is one of the game posible status.(OWGAME_STATUS_STOPPED,OWGAME_STATUS_RUNNING,OWGAME_STATUS_CREATED,OWGAME_STATUS_PLAYING)
*/
void ow_game::rungame() {
	this->status = OWGAME_STATUS_RUNNING;
}

/**
* Checks if any of the players won the game. (status=OWGAME_STATUS_WON)
*/
int ow_game::checkgamewon() {
	if (this->status == OWGAME_STATUS_WON) return 1;
  return 0;
}

/**
* Sets the application to the 'Stopped' so the application ends and exits.
* 'Stopped' is one of the game posible status.(OWGAME_STATUS_STOPPED,OWGAME_STATUS_RUNNING,OWGAME_STATUS_CREATED,OWGAME_STATUS_PLAYING)
*/
void ow_game::exitomweso() {
	this->status = OWGAME_STATUS_STOPPED;
}

/**
* Checks if the selected player has ended his turn and is sleeping waiting for his next turn.
*/
int ow_game::selplayer_endofturn() {
	return this->selected_player->endofturn();
}

/**
* Checks if the board has finished executing the orders inserted by the player.
* Some orders take various board_update_status() loops so you must wait before next order
* can be issued.
*/
int ow_game::board_endofturn() {
	return this->owtab->endofturn();
}

/**
* Gets the name of the board cell corresponding to the given cell coords.
* @param boardside Side of the board (BRD_SIDE_NORTH, BRD_SIDE_SOUTH).
* @param cellx Xpos of the specified cell in the bcells array.
* @param celly Ypos of the specified cell in the bcells array.
* @returns The name of the cell
*/
string ow_game::board_translate_coord2name(int boardside, int cellx, int celly) {
	return this->owtab->translate_coord2name(boardside,cellx,celly);
}

/**
* Updates the board status as it executes step by step the orders received.
* Some orders take various board_update_status() loops
* as example 'sowing' takes a loop for every seed sowm in the movement.
*/
void ow_game::board_update_status() {
		this->owtab->update_status();
}


/**
* Gets the selected player AI (Artificial Intelligence) mode. 
* @return AI mode used by the selected player.(AI_HUMAN,AI_COMPUTER,etc)
*/
int ow_game::selplayer_getaimode() {
	return this->selected_player->getaimode();
}

/**
* Checks if current selected player finished searching for best AI move
* @return -1 if no move available, 0 if still looking for, 1 if best AI move found
*/
int ow_game::selplayer_aimove_found() {
	return this->selected_player->foundbestmove_ai();
}

/**
* Looks for the best AI move for the current selected player.
* Execute it repeatly while not selplayer_aimove_found()
* @return when selplayer_aimove_found() this returns the cell coords to start the best AI move.
*/
ow_move ow_game::selplayer_aimove() {
	ow_move selected_cell=this->selected_player->searchbestmove_ai();
	return selected_cell;
}

int ow_game::getnewgame_fromqueue() {
	if (this->queue_pos < this->queue_last) {
		this->endgame();
		this->queue_pos++;
		this->createnewgame_useconfig();
		this->status = OWGAME_STATUS_PLAYING; // automatically go to play
		this->current_menu = OWGAME_MENU_PLY;  // automatically go to play
		return (this->queue_pos);
	}
	return 0;
}

/**
* Gets the board side where the selected player plays.
* @return Board side of the selected player (BRD_SIDE_NORTH,BRD_SIDE_SOUTH).
*/
int ow_game::selplayer_getboardside() {
	return this->selected_player->getboardside();
}

/**
* Orders a new sow movement for the board to execute.
* @param boardside Side of the board (BRD_SIDE_NORTH, BRD_SIDE_SOUTH).
* @param cellx Xpos of the specified cell in the board bcells array.
* @param celly Ypos of the specified cell in the board bcells array.
* @return 0 on success, -1 on any error.
*/
int ow_game::selplayer_makemovement(int boardside, int cellx, int celly) {
	int result = this->selected_player->makemovement(boardside, cellx, celly);
	if (result>=0) this->logmove(this->current_turn,this->selected_player,boardside,cellx,celly);
	return result;
}

/**
* Orders a new reverse movement ending with capture for the board to execute.
* @param boardside Side of the board (BRD_SIDE_NORTH, BRD_SIDE_SOUTH).
* @param cellx Xpos of the specified cell in the board bcells array.
* @param celly Ypos of the specified cell in the board bcells array.
* @return 0 on success, -1 on any error.
*/
int ow_game::selplayer_rcapture(int boardside, int cellx, int celly) {
	int result = this->selected_player->rcapture(boardside, cellx, celly);
	if (result>=0) this->logrcapture(this->current_turn,this->selected_player,boardside,cellx,celly);
	return result;
}

/**
* Write 'move' info to a log file specified by owgame->config.logmovesfile
* @param player Player that made the movement.
* @param boardside Side of the board (BRD_SIDE_NORTH, BRD_SIDE_SOUTH).
* @param cellx Xpos of the specified cell in the board bcells array.
* @param celly Ypos of the specified cell in the board bcells array.
*/
void ow_game::logmove(int turn, ow_player *player, int boardside, int cellx, int celly) {
	this->logmoves[this->logmovescurrent].turn = turn;
	this->logmoves[this->logmovescurrent].player = player;
	this->logmoves[this->logmovescurrent].stepcommand.name = "move";
	this->logmoves[this->logmovescurrent].stepcommand.param1 = this->ow_commandline_translate_coord2name(boardside, cellx, celly);
	this->logmovescurrent++; this->logmovescount = this->logmovescurrent; 
}

/**
* Write 'rcapture' info to a log file specified by owgame->config.logmovesfile
* @param player Player that made the movement.
* @param boardside Side of the board (BRD_SIDE_NORTH, BRD_SIDE_SOUTH).
* @param cellx Xpos of the specified cell in the board bcells array.
* @param celly Ypos of the specified cell in the board bcells array.
*/
void ow_game::logrcapture(int turn, ow_player *player, int boardside, int cellx, int celly) {
	this->logmoves[this->logmovescurrent].turn = turn;
	this->logmoves[this->logmovescurrent].player = player;
	this->logmoves[this->logmovescurrent].stepcommand.name = "rcapture";
	this->logmoves[this->logmovescurrent].stepcommand.param1 = this->ow_commandline_translate_coord2name(boardside, cellx, celly);
	this->logmovescurrent++; this->logmovescount = this->logmovescurrent; 
}

/**
* UnDo the last (nummoves) movement(s) executed
* @param nummoves Number of moves to UnDo.
* @returns -1 on fail, nummoves on success
*/
int ow_game::undolastmove(int nummoves) {
	//cout << "undolastmove(" << nummoves << ")" << endl;
	if ((this->logmovescurrent>=nummoves)&&(this->owtab->undolastmove(nummoves)>=0)) {
		this->logmovescurrent-=nummoves;
		this->current_turn = this->logmoves[this->logmovescurrent].turn;
		this->selected_player = this->logmoves[this->logmovescurrent].player;
		this->selected_player->wakeup();
		if (this->status == OWGAME_STATUS_WON) this->status = OWGAME_STATUS_PLAYING;
		return nummoves;
	}
	return -1;
}

/**
* ReDo the last move 'undone' by the 'undom' command.
* @param nummoves Number of moves to ReDo, if available.
* @return -1 on fail, Number of moves ReDone on success
*/
int ow_game::redolastundo(int nummoves) {
	cellXY mcell;
	//cout << "redolastundo(" << nummoves << ")" << endl;
	if ((this->logmovescurrent+nummoves)<=this->logmovescount) {
		for (int idx=1;idx<=nummoves;idx++) {
			this->current_turn = this->logmoves[this->logmovescurrent].turn;
			this->selected_player = this->logmoves[this->logmovescurrent].player;
			mcell = this->ow_commandline_translate_name2coord(this->logmoves[this->logmovescurrent].stepcommand.param1);
			this->owtab->set_undo_mode(0);
			if (this->logmoves[this->logmovescurrent].stepcommand.name=="move") {
				//cout << "REDOmove" << endl;
				this->owtab->move_d(mcell.boardside, mcell.x, mcell.y);
			}
			else if (this->logmoves[this->logmovescurrent].stepcommand.name=="rcapture") {
				//cout << "REDOcapture" << endl;
				this->owtab->rcapture_d(mcell.boardside, mcell.x, mcell.y);
			}
			this->owtab->set_undo_mode(1);
			this->selected_player->gosleep();
			this->logmovescurrent++;
		}
		//this->changeturn();

		return nummoves;
	}
	return -1;
}


/**
* Creates a new game (board,players,firstturn,etc) based on the this->config values.
*/
void ow_game::createnewgame_useconfig() {
	int depth=1;
	this->current_turn = 1;
	this->logmovescurrent = 0; this->logmovescount = 0;
  this->owtab 	= new ow_board(this->config.numcols);
	this->owtab->suggested_beginners_seeds_initial_setup();
	this->owtab->set_undo_mode(1);

  this->p1 = new ow_player(this->owtab, PL_PLAYER1, BRD_SIDE_SOUTH);
  this->p2 = new ow_player(this->owtab, PL_PLAYER2, BRD_SIDE_NORTH);

	if (this->config.fturn_player=="P1") this->setfirstturn(this->p1);
  else this->setfirstturn(this->p2);

	this->current_turn = this->config.turn;
	if (this->config.plays=="P1") this->setfirstturn(this->p1);
  else this->setfirstturn(this->p2);

	this->p1->setplayername(this->config.p1name.c_str());
	this->p2->setplayername(this->config.p2name.c_str());
	
	if (this->config.p1aimode == "SIMPLE") this->p1->setplayer_aimode(AI_SIMPLE);
	if (this->config.p2aimode == "SIMPLE") this->p2->setplayer_aimode(AI_SIMPLE);
	if (this->config.p1aimode == "NEGAMAX") this->p1->setplayer_aimode(AI_NEGAMAX);
	if (this->config.p2aimode == "NEGAMAX") this->p2->setplayer_aimode(AI_NEGAMAX);
	if (this->config.p1aimode == "ALPHABETA") this->p1->setplayer_aimode(AI_ALPHABETA);
	if (this->config.p2aimode == "ALPHABETA") this->p2->setplayer_aimode(AI_ALPHABETA);
	if (this->config.p1aimode == "SCOUT") this->p1->setplayer_aimode(AI_SCOUT);
	if (this->config.p2aimode == "SCOUT") this->p2->setplayer_aimode(AI_SCOUT);
	
	sscanf(this->config.p1aidepth.c_str(), "%d", &depth); this->p1->setplayer_aidepth(depth);
	sscanf(this->config.p2aidepth.c_str(), "%d", &depth); this->p2->setplayer_aidepth(depth);

	if (this->config.p1aieval == "RANDOM") this->p1->setplayer_aieval(AI_RANDOM_EVAL);
	if (this->config.p2aieval == "RANDOM") this->p2->setplayer_aieval(AI_RANDOM_EVAL);
	if (this->config.p1aieval == "MATERIAL") this->p1->setplayer_aieval(AI_MATERIAL_EVAL);
	if (this->config.p2aieval == "MATERIAL") this->p2->setplayer_aieval(AI_MATERIAL_EVAL);
	if (this->config.p1aieval == "DISPERSION") this->p1->setplayer_aieval(AI_DISPERSION_EVAL);
	if (this->config.p2aieval == "DISPERSION") this->p2->setplayer_aieval(AI_DISPERSION_EVAL);
	if (this->config.p1aieval == "MOBILITY") this->p1->setplayer_aieval(AI_MOBILITY_EVAL);
	if (this->config.p2aieval == "MOBILITY") this->p2->setplayer_aieval(AI_MOBILITY_EVAL);
	if (this->config.p1aieval == "REACH") this->p1->setplayer_aieval(AI_REACH_EVAL);
	if (this->config.p2aieval == "REACH") this->p2->setplayer_aieval(AI_REACH_EVAL);

	if (this->config.p1aieval == "F1COMB") this->p1->setplayer_aieval(AI_F1COMB_EVAL);
	if (this->config.p2aieval == "F1COMB") this->p2->setplayer_aieval(AI_F1COMB_EVAL);
	if (this->config.p1aieval == "F2COMB") this->p1->setplayer_aieval(AI_F2COMB_EVAL);
	if (this->config.p2aieval == "F2COMB") this->p2->setplayer_aieval(AI_F2COMB_EVAL);

	if ((this->config.gfxboard == "ON")&&(this->gfxboard_hidden)) { this->gfxboard_showhidde(); }
	if ((this->config.gfxboard == "OFF")&&(!this->gfxboard_hidden)) { this->gfxboard_showhidde(); }

}

/**
* Creates a new game and sets it up with the info loaded from the specified file.
* @param filename Name of the savegame file. See file format at app user guide.
* @return 0 on success, -1 on fileformat error, -2 any other error
*/
int ow_game::loadgame(string filename) {
	int boarddata[BRD_MAX_COLS_PER_PLAYER][(BRD_MAX_ROWS_PER_PLAYER*BRD_NUM_SIDES)];
	parsed_command logmsteps[BRD_MAX_UNDO_MOVES]; int logmstep = -1; string logmcommand; string logline;
	int splitpos,idxrow, idxcol; string::size_type notwhite;
	string boardline, boardcelldata;
	int i_boardcelldata, i_seeds_onboard;
	parsed_command cfgcommand;
	string cfgline;
	ow_player *enemy_player;
	int app_current_menu_backup = this->current_menu;
	int b_boarddataloaded = 0;
	int b_loaderror = 0;

	if (this->status>=OWGAME_STATUS_CREATED) {
		cout << "Error: Cannot load game. A game already exists." << endl;
		return -2;
	}

	this->set_default_gameconfig(NULL);

	ifstream filegame (filename.c_str());
	if (!filegame.is_open()) {
		cout << "Error loading, cannot open file '" << filename << "'." << endl;
		return -2;
	} else {
		while ((!filegame.eof())&&(!b_loaderror)) {
			getline(filegame,cfgline);
			if ((cfgline!="")&&(cfgline.substr(0,1)!="#")) {
				splitpos=cfgline.find(":",0);
				if (splitpos>0) {
					cfgcommand.param1 = cfgline.substr(0,splitpos);cfgcommand.param2 = cfgline.substr(splitpos+1,cfgline.length()-splitpos);
					// trim leading and trailing white spaces
					notwhite = cfgcommand.param1.find_first_not_of(" \t\n");cfgcommand.param1.erase(0,notwhite);notwhite = cfgcommand.param1.find_last_not_of(" \t\n"); cfgcommand.param1.erase(notwhite+1); 
					notwhite = cfgcommand.param2.find_first_not_of(" \t\n");cfgcommand.param2.erase(0,notwhite);notwhite = cfgcommand.param2.find_last_not_of(" \t\n"); cfgcommand.param2.erase(notwhite+1); 
					//cout << "[" << cfgcommand.param1 << "] ---> [" << cfgcommand.param2 << "]" << endl;

					if (cfgcommand.param1=="FIRSTTURN") {
						if ((cfgcommand.param2!="P1") && (cfgcommand.param2!="P2")) {
							cout << "Incorrect value for '" << cfgcommand.param1 << "', valid values are: (P1,P2)." << endl;
							b_loaderror = 1;
						} else { this->config.fturn_player = cfgcommand.param2;}
					}

					if (cfgcommand.param1=="TURN") {
						if (sscanf(cfgcommand.param2.c_str(), "%d", &this->config.turn)<=0) {
							cout << "TURN:[" << cfgcommand.param2 << "] Error loading value as integer." << endl;
							b_loaderror = 1;
						}
					}

					if (cfgcommand.param1=="PLAYS") {
						if ((cfgcommand.param2!="P1") && (cfgcommand.param2!="P2")) {
							cout << "Incorrect value for '" << cfgcommand.param1 << "', valid values are: (P1,P2)." << endl;
							b_loaderror = 1;
						} else { this->config.plays = cfgcommand.param2;}
					}


					if (cfgcommand.param1=="BRD_INIDATA") {
						idxrow = 0; i_seeds_onboard = 0;
						while ((!filegame.eof())&&(!b_loaderror)&&(idxrow<(BRD_ROWS_PER_PLAYER*BRD_NUM_SIDES))) {
							getline(filegame,boardline);
							idxcol=0;
							while ((boardline!="")&&(!b_loaderror)&&(idxcol<this->config.numcols)) {
								boardline.erase(0,boardline.find_first_not_of(" \t\n"));
								boardcelldata = boardline.substr(0, boardline.find_first_of(" \t\n"));							
								boardline.erase(0,boardline.find_first_of(" \t\n"));
								//cout << "[" << boardcelldata << "]";
								if(sscanf(boardcelldata.c_str(), "%d", &i_boardcelldata)>0) {
									if ((i_seeds_onboard+i_boardcelldata)<=(4*BRD_NUM_SIDES*this->config.numcols)) {
										boarddata[idxcol][idxrow] = i_boardcelldata;
										b_boarddataloaded=1; i_seeds_onboard+=i_boardcelldata;
									} else { cout << "BRD_INIDATA: Too many seeds on board. Max [" << 4*BRD_NUM_SIDES*this->config.numcols << "] seeds allowed." << endl; b_loaderror = 1; }
								} else { cout << "BRD_INIDATA: [" << boardcelldata << "] Error loading value as integer." << endl; b_loaderror = 1; }
								idxcol++;
							}
							//cout << endl; 
							idxrow++;
						}
					}	

					if (cfgcommand.param1=="LOGMOVES") {
						while ((!filegame.eof())&&(!b_loaderror)) {
							getline(filegame,logline);
							logline.erase(0,logline.find_first_not_of(" \t\n"));
							if ((logline!="")&&(logline.substr(0,1)!="#")) {
								logmcommand = logline.substr(0, logline.find_last_not_of(" \t\n")+1);							
								logmstep++;this->commandline_parse_command((char *)logmcommand.c_str(),&logmsteps[logmstep]);
							}
						}
					}	

					if ((cfgcommand.param1!="BRD_INIDATA")&&(cfgcommand.param1!="TURN")
						 &&(cfgcommand.param1!="LOGMOVES")&&(cfgcommand.param1!="PLAYS")) {
						cfgcommand.name = "SET";
						this->current_menu = OWGAME_MENU_APP_NEWGAME; // Hack to allow to set VARS
						if (this->ow_commandline_setvalue(&cfgcommand)==-1) {b_loaderror = 1;}
						this->current_menu = app_current_menu_backup; // Recover from hack
					}

				}
			}
		}
		filegame.close();
		if (!b_loaderror) { 
			int b_created = this->ow_commandline_creategame(NULL);
			if (b_created && b_boarddataloaded) this->setupboard_fromarray(boarddata);

			//Loading and running the moves found in the logmoves
			ow_player *tmpplayer; cellXY tmpcell; int tmpturn = 0; 
			ow_player *firstplayer;
			if (this->config.fturn_player=="P1") {firstplayer = this->p1;}else {firstplayer = this->p2;}


			for (int idx=0;idx<=logmstep;idx++) {
				//cout << "[" << logmsteps[idx].name << "][" << logmsteps[idx].param1 << "]" << endl;
				tmpcell = this->ow_commandline_translate_name2coord(logmsteps[idx].param1);
				//AQUII comprobar que el movimiento nuevo sea del boarside contrario al anterior
				if (this->p1->getboardside()==tmpcell.boardside) {tmpplayer = this->p1;} else {tmpplayer=this->p2;}
				if (tmpplayer->getplayernum()==firstplayer->getplayernum()) {tmpturn++;}
				if (logmsteps[idx].name=="move") {
					this->logmove(tmpturn, tmpplayer, tmpcell.boardside, tmpcell.x, tmpcell.y);
					this->owtab->move_d(tmpcell.boardside, tmpcell.x, tmpcell.y);
				}
				if (logmsteps[idx].name=="rcapture") {
					this->logrcapture(tmpturn, tmpplayer, tmpcell.boardside, tmpcell.x, tmpcell.y);
					this->owtab->rcapture_d(tmpcell.boardside, tmpcell.x, tmpcell.y);
				}
				
			}

			// Game not started yet, and player already WON!
			if (this->selected_player->getplayernum() == this->p1->getplayernum()) enemy_player = this->p2;
			else { enemy_player = this->p1; }
			if (!this->owtab->checkavailablemoves(enemy_player->getboardside())) {
				this->status=OWGAME_STATUS_WON;
			}
			
			if (b_created) return 1; else return -2;
		} else return -1;

	}
	
	return 0;
}

/**
* Sets up the number of seeds in each cell of the board from the info in the array provided.
* Usually the array is generated by loadgame() function.
* @param arraydata Array containing the number of seeds in each cell. Note: The array data  arraydata[column][row] is stored in the same order as the board
* is displayed by draw_txt(), not in the order stablished by the cell names.
* @return 0 on success, -1 on error setting up the board.
*/
int ow_game::setupboard_fromarray(int arraydata[BRD_MAX_COLS_PER_PLAYER][(BRD_MAX_ROWS_PER_PLAYER*BRD_NUM_SIDES)]) {
	int numrows = (BRD_ROWS_PER_PLAYER*BRD_NUM_SIDES);
	int numcols = this->config.numcols;
	int idxrow, idxcol, boardside, boardrow, boardcol;
	for (idxrow=0;idxrow<numrows;idxrow++) {
		if (idxrow<2) { boardside = BRD_SIDE_NORTH; boardrow=(BRD_MAX_ROWS_PER_PLAYER-idxrow-1);} else {boardside = BRD_SIDE_SOUTH;boardrow=idxrow-BRD_MAX_ROWS_PER_PLAYER;}
		for (idxcol=0;idxcol<numcols;idxcol++) {	
			//cout << "[" << arraydata[idxcol][idxrow] << "]";
			if (boardside==BRD_SIDE_NORTH) {boardcol = numcols-idxcol-1;} else { boardcol = idxcol; }
			this->owtab->setcellseeds(arraydata[idxcol][idxrow], boardside, boardcol, boardrow);
		}
		//cout << endl;
	}
	return 0;
}


/**
* Saves the current game into the specified file, overwriting the file if exists
* @param filename Name of the savegame file. See file format at app user guide.
* @return 0 on success, -1 on error saving savegame.
*/
int ow_game::savegame(string filename) {
	int numrows = (BRD_ROWS_PER_PLAYER*BRD_NUM_SIDES);
	int numcols = this->config.numcols;
	int idxrow, idxcol, boardside, boardrow, boardcol;
	int idxlog;
	int cseeds;

	ofstream filegame (filename.c_str());
	if (!filegame.is_open()) {return -1;} 
	else {
		filegame << "# Omweso Savegame" << endl;
		// Save player info
		filegame << "P1NAME: " << this->config.p1name << endl;
		filegame << "P1AI: " << this->config.p1aimode << endl;
		filegame << "P1DEPTH: " << this->config.p1aidepth << endl;
		filegame << "P1EVAL: " << this->config.p1aieval << endl;
		filegame << "P2NAME: " << this->config.p2name << endl;
		filegame << "P2AI: " << this->config.p2aimode << endl;
		filegame << "P2DEPTH: " << this->config.p2aidepth << endl;
		filegame << "P2EVAL: " << this->config.p2aieval << endl;

		// Game info
		filegame << "NUMCOLS: " << this->config.numcols << endl;
		filegame << "GFXBOARD: " << this->config.gfxboard << endl;
		filegame << "FIRSTTURN: " << this->config.fturn_player << endl;
		filegame << "TURN: " << this->current_turn << endl;
		if (this->selected_player->getplayernum()==PL_PLAYER1) filegame << "PLAYS: P1" << endl;
		else filegame << "PLAYS: P2" << endl;
 

  	// Save the cells contents
		filegame << "BRD_INIDATA:" << endl;
		for (idxrow=0;idxrow<numrows;idxrow++) {
			if (idxrow<2) { boardside = BRD_SIDE_NORTH; boardrow=(BRD_MAX_ROWS_PER_PLAYER-idxrow-1);} else {boardside = BRD_SIDE_SOUTH;boardrow=idxrow-BRD_MAX_ROWS_PER_PLAYER;}
			for (idxcol=0;idxcol<numcols;idxcol++) {	
				//cout << "[" << arraydata[idxcol][idxrow] << "]";
				if (boardside==BRD_SIDE_NORTH) {boardcol = numcols-idxcol-1;} else { boardcol = idxcol; }
				cseeds = this->owtab->initialboard_countseeds(boardside, boardcol, boardrow);
				if (idxcol>0) filegame << " ";
				if (cseeds<10) filegame << "0";
				filegame << cseeds;
				if (idxcol==(numcols-1)) filegame << endl;
			}
			//cout << endl;
		}
		filegame << "LOGMOVES:" << endl;
		for (idxlog=0;idxlog<=this->logmovescount;idxlog++) {
			filegame << this->logmoves[idxlog].stepcommand.name << " " << this->logmoves[idxlog].stepcommand.param1 << " " << this->logmoves[idxlog].stepcommand.param2 << endl;
		}
		filegame << "# EOF" << endl;
		filegame.close();
	}
	return 0;
}


/**
* Save the game results to the results log, when one of the players wins the current game.
*/
int ow_game::loggameresults() {
	int moves2win;
	ofstream logresults;
	
	if (this->logresults=="ON") {
		fstream checklogfile(this->logresultsfile.c_str(),ios::in);
		if (!checklogfile.is_open()) {
			logresults.open(this->logresultsfile.c_str());
			logresults << "P1,P1AIMODE,P1AIDEPTH,P1AIEVAL,P2,P2AIMODE,P2AIDEPTH,P2AIEVAL,NUMCOLS,FIRSTTURN,MOVES2WIN,WINNER" << endl;
			logresults.close();
		} else {checklogfile.close();}

		logresults.open(this->logresultsfile.c_str(),ios::app);
		logresults << "P1,";
		logresults << this->config.p1aimode << ",";
		logresults << this->config.p1aidepth << ",";
		logresults << this->config.p1aieval << ",";
		logresults << "P2,";
		logresults << this->config.p2aimode << ",";
		logresults << this->config.p2aidepth << ",";
		logresults << this->config.p2aieval << ",";
		logresults << this->config.numcols << ",";
		logresults << this->config.fturn_player << ",";
		if (this->selected_player->getplayernum()==PL_PLAYER1) {
			moves2win = (2*this->current_turn); if (this->config.fturn_player=="P1") { moves2win-=1;}
			logresults << moves2win << ",P1";
		} else {
			moves2win = (2*this->current_turn); if (this->config.fturn_player=="P2") { moves2win-=1;}
			logresults << moves2win << ",P2";
		}

		logresults << endl;
		logresults.close();

	} // end if saveresults=ON
}

/**
* Ends the current game, basically deletes board, players and other resources.
*/
void ow_game::endgame() {
	if (!this->gfxboard_hidden) {
		SDL_Quit();this->gfxboard_hidden=1;
	}
	delete this->owtab; this->gfxboard_dataloaded=0;
	delete this->p1; delete this->p2;
	this->owtab = NULL;
	this->p1 = NULL; this->p2 = NULL;
}

