/***************************************************************************
 *   Copyright (C) 2007 by David Torres   *
 *   aiken669@gmail.com   *
 ***************************************************************************/
#ifndef OW_GAME_H
#define OW_GAME_H

#include <readline/readline.h>
#include <SDL/SDL_ttf.h>
#include <string>
#include <fstream>
#include "omweso.h"
#include "ow_player.h"

using namespace std;

typedef struct {
	string name;
	string param1;
	string param2;
} parsed_command;

typedef struct {
	int turn;
	ow_player *player;
	parsed_command stepcommand;
} logm_step;

typedef struct {
	string p1name;
	string p1aimode;
	string p1aidepth;
	string p1aieval;
	string p2name;
	string p2aimode;
	string p2aidepth;
	string p2aieval;
	int numcols;
	string fturn_player;
	int turn;
	string plays;
	string gfxboard;
} game_configuration;

/**
	@author David Torres <aiken669@gmail.com>
*/
class ow_game{
private:
	game_configuration config;
	int status;
	int current_turn;
	int current_menu;
	int queue_pos; int queue_last;
	int gfxboard_hidden;
	int gfxboard_dataloaded;
	SDL_Surface *gfxboard_screen;
  SDL_Surface *gfxdata[2];
	TTF_Font *font;
	ow_player *p1;
	ow_player *p2;
	ow_board *owtab;
	ow_player *selected_player;
	rl_vcpfunc_t *commandline_handler;
	rl_vcpfunc_t *commandline_handler_null;
	rl_completion_func_t *commandline_completion_handler;
	rl_compentry_func_t *commandline_command_generator_handler;
	rl_compignore_func_t *commandline_savegames_filter_handler;

	logm_step logmoves[BRD_MAX_UNDO_MOVES];
	int logmovescurrent, logmovescount;
	void logmove(int turn, ow_player *player, int boardside, int cellx, int celly);
	void logrcapture(int turn, ow_player *player, int boardside, int cellx, int celly);
	int ow_commandline_showlogmoves(parsed_command *pcommand);

	string logresults;
	string logresultsfile;
	int loggameresults();
	int ow_commandline_setlogresults(parsed_command *pcommand);

	int undolastmove(int nummoves);
	int redolastundo(int nummoves);

	void commandline_parse_command(char *command, parsed_command *pcommand);
	void ow_commandline_getpromptstring(char *prompt);
	void ow_commandline_refreshprompt();
	int ow_commandline_newgame(parsed_command *pcommand);
	int ow_commandline_loadgame(parsed_command *pcommand);
	int ow_commandline_savegame(parsed_command *pcommand);
	int ow_commandline_setupboard(parsed_command *pcommand);
	int ow_commandline_setupboard_confirm(parsed_command *pcommand);
	int ow_commandline_setupboard_cancel(parsed_command *pcommand);
	int ow_commandline_endgame(parsed_command *pcommand);
	void ow_commandline_version(parsed_command *pcommand);
	int ow_commandline_quit(parsed_command *pcommand);
	void ow_commandline_help(parsed_command *pcommand);
	int ow_commandline_play(parsed_command *pcommand);
	int ow_commandline_batch(parsed_command *pcommand);
	int ow_commandline_setbatch(parsed_command *pcommand);
	void ow_commandline_showtxtboard(parsed_command *pcommand);
	int ow_commandline_setvalue(parsed_command *pcommand);
	int ow_commandline_setnextturn(string variablename, string playerIDstr);
	int ow_commandline_setnumcols(string numcols);
	int ow_commandline_setplayer_aimode(string player_ai, string aimode);
	int ow_commandline_setplayer_aidepth(string player_ai, string aidepth);
	int ow_commandline_setplayer_aieval(string player_ai, string aieval);
	int ow_commandline_setgfxboard_status(string gfxstatus);
	int ow_commandline_creategame(parsed_command *pcommand);
	void ow_commandline_setcommand_help(parsed_command *pcommand);
	int ow_commandline_move(parsed_command *pcommand);
	int ow_commandline_rcapture(parsed_command *pcommand);
	int ow_commandline_manageai(parsed_command *pcommand);
	int ow_commandline_mainmenu(parsed_command *pcommand);
	cellXY ow_commandline_translate_name2coord(string cellname);
	void createnewgame_useconfig();
	void set_default_gameconfig(game_configuration *config);
	int loadgame(string filename);
	int savegame(string filename);
	int setupboard_fromarray(int arraydata[BRD_MAX_COLS_PER_PLAYER][(BRD_MAX_ROWS_PER_PLAYER*BRD_NUM_SIDES)]);
	void setnewturn(ow_player *player);
	void setfirstturn(ow_player *player);
  int load_gfxdata();
	int gfxboard_showhidde();
	void draw();
	void endgame();
	void exitomweso();
	int ow_commandline_undolastmove(parsed_command *pcommand);
	int ow_commandline_redolastmove(parsed_command *pcommand);
	int ow_commandline_undolastturn(parsed_command *pcommand);


public:

	ow_game();
	void ow_setcommandlinehandlers(rl_vcpfunc_t *commhandler,rl_vcpfunc_t *commhandler_null, rl_completion_func_t *commhandler_completion,rl_compentry_func_t *commhandler_command,rl_compignore_func_t *commhandler_savegames_filter);
	char **commandline_completion(const char *text, int start, int end);
	char *commandline_command_generator(const char *text, int state);
	int commandline_savegames_filter(char **matches);
	void commandline_read();
	void commandline_parse_execute(char *command);
	string ow_commandline_translate_coord2name(int boardside, int cellx, int celly);
	int ow_commandline_setupboard_put(parsed_command *pcommand);
	int ow_commandline_setupboard_take(parsed_command *pcommand);
	void ow_commandline_showgameconfig(parsed_command *pcommand);

	int getnewgame_fromqueue();
	int selplayer_getboardside();
	int selplayer_makemovement(int boardside, int cellx, int celly);
	int selplayer_rcapture(int boardside, int cellx, int celly);
	int selplayer_getaimode();
	int selplayer_aimove_found();
	ow_move selplayer_aimove();
	int selplayer_endofturn();

	int board_endofturn();
	void board_update_status();
	string board_translate_coord2name(int boardside, int cellx, int celly);

	void changeturn();
	int gamerunning();
	int gamecreated();
	int gameplaying();
	int gamesettingup();
	void rungame();
	int checkgamewon();

	void show_version();
	void draw_gfx();
	int MouseLeftButtonClicked(int *mouse_x, int *mouse_y);
	int MouseRightButtonClicked(int *mouse_x, int *mouse_y);
	ow_move seleccionarcelda_bycoord(int coordx, int coordy);

	~ow_game();

};

#endif
