#include <readline/readline.h>
#include <readline/history.h>
#include <iostream>
#include "ow_game.h"

using namespace std;

typedef struct {
	string commname;
	string commshortname;
	string commshortdesc;
	string commlonghelp;
} command_help;


static command_help app_commands_help[OWGAME_MENU_APP_COMMANDS] = {
	{"newgame","new","create a new game.",""},
	{"loadgame","load","load an existing game.",""},
	{"savegame","save","save the current game.",""},
	{"setup","setup","edit the cell contents of the current board.",""},
	{"endgame","end","end the current game.",""},
	{"play","p","continue playing the current game.",""},
	{"manageai","ai","manage artificial intelligence.",""},
	{"version","ver","show program version and build.",""},
	{"help","h","show this help screen.",""},
	{"quit","q","exit from omweso.",""}
};

static command_help app_newgame_commands_help[OWGAME_MENU_APP_NEWGAME_COMMANDS] = {
	{"showcfg","cfg","show game current configuration.",""},	
	{"set","SET","set/change configuration value.",""},
	{"create","c","create new game with current configuration.",""},
	{"help","h","show this help screen.",""},
	{"quit","q","cancel creating new game, back to mainmenu.",""}
};

static command_help setupboard_commands_help[OWGAME_MENU_SETUPBOARD_COMMANDS] = {
	{"board","b","show the current board status.",""},
	{"gfxboard","gfx","show/hidde the graphic board.",""},
	{"put","p","put seeds in the board.",""},
	{"take","t","take seeds from the board.",""},
	{"confirm","ok","confirms setup and save it.",""},
	{"cancel","main","cancels setup and back to app mainmenu.",""},
	{"help","h","show this help screen.",""}
};

static command_help play_commands_help[OWGAME_MENU_PLY_COMMANDS] = {
	{"board","b","show the current board status.",""},
	{"gfxboard","gfx","show/hidde the graphic board.",""},
	{"move","mv","make movement during your turn.",""},
	{"rcapture","rc","make a reverse capture movement.",""},
	{"undo","ud","Returns game to the previous turn of the current player.",""},
	{"mainmenu","main","go to app main menu.",""},
	{"manageai","ai","manage artificial intelligence.",""},
	{"help","h","show this help screen.",""}
};

static command_help ai_commands_help[OWGAME_MENU_MAI_COMMANDS] = {
	{"showcfg","cfg","show game current configuration.",""},	
//	{"set","SET","set/change configuration value.",""},
//	{"eval","e","evaluate the current board using selected evalfunc.",""},
	{"logmoves","logm","show the list of moves executed.",""},
	{"undom","udm","UnDo of the last movement.",""},
	{"redom","rdm","ReDo the last 'undone' movement.",""},
//	{"batch","bat","create/edit current batch queue.",""},
	{"batch","bat","run various games with the current config.",""},
	{"logresult","logr","Save games results to a log file.",""},
	{"play","p","continue playing the current game.",""},
	{"mainmenu","main","go to app main menu.",""},
	{"help","h","show this help screen.",""}
};

static command_help ai_batch_commands_help[OWGAME_MENU_BAT_COMMANDS] = {
	{"batch","bat","show batch queue contents.",""},
	{"showcfg","cfg","game to be added to the batch queue.",""},	
	{"set","SET","set/change configuration value.",""},
	{"add","ADD","add games to batch queue using current cfg.",""},
	{"del","DEL","delete games from current batch queue.",""},
	{"logresult","logr","set on/off writting results to a log file.",""},
	{"runbatch","run","run batch queue.",""},
	{"mainmenu","main","go to app main menu.",""},
	{"help","h","show this help screen.",""}
};


/**
* Sets the commandline handlers
* and creates the command line prompt, so it is ready to receive the first order.
* @param commhandler Function that will handle the orders entered by the user throught the command line..
* @param commhandler_null Function that does nothing while the game is busy.
* @param commhandler_completion Function that manages the readline completion
* @param commhandler_command Function that manages readline 'appcommands' completion
* @param commhandler_savegames_filter function that manage savegame filenames completion.
*/
void ow_game::ow_setcommandlinehandlers(rl_vcpfunc_t *commhandler,rl_vcpfunc_t *commhandler_null, rl_completion_func_t *commhandler_completion,rl_compentry_func_t *commhandler_command,rl_compignore_func_t *commhandler_savegames_filter) {
	this->commandline_handler = commhandler;
	this->commandline_handler_null = commhandler_null;
	this->commandline_completion_handler = commhandler_completion;
	this->commandline_command_generator_handler = commhandler_command;
	this->commandline_savegames_filter_handler = commhandler_savegames_filter;
	rl_attempted_completion_function = this->commandline_completion_handler;
	char prompt[64];
	rl_callback_handler_remove();
	this->ow_commandline_getpromptstring(prompt);
	rl_callback_handler_install(prompt, this->commandline_handler);	
}

/**
* Parses every command line received searching for known commands.
* and executes the found commands.
* @param command Command line to parse.
*/
void ow_game::commandline_parse_execute(char *command) {
	string strcommand = command;
	parsed_command pcommand;
	int command_found = 0;

	if (!strcommand.empty()) {
		this->commandline_parse_command(command,&pcommand);

		switch (this->current_menu) {
			case OWGAME_MENU_APP:
				if ((pcommand.name == "newgame")||(pcommand.name == "new")) {this->ow_commandline_newgame(&pcommand);command_found=1;}
				if ((pcommand.name == "loadgame")||(pcommand.name == "load")) {this->ow_commandline_loadgame(&pcommand);command_found=1;}
				if ((pcommand.name == "savegame")||(pcommand.name == "save")) {this->ow_commandline_savegame(&pcommand);command_found=1;}
				if (pcommand.name == "setup") {this->ow_commandline_setupboard(&pcommand);command_found=1;}
				if ((pcommand.name == "endgame")||(pcommand.name == "end")) {this->ow_commandline_endgame(&pcommand);command_found=1;}
				if ((pcommand.name == "play")||(pcommand.name == "p")) {this->ow_commandline_play(&pcommand);command_found=1;}
				if ((pcommand.name == "manageai")||(pcommand.name == "ai")) {this->ow_commandline_manageai(&pcommand);command_found=1;}
				if ((pcommand.name == "version")||(pcommand.name == "ver")) {this->ow_commandline_version(&pcommand);command_found=1;}
				if ((pcommand.name == "quit")||(pcommand.name == "q")) {this->ow_commandline_quit(&pcommand);command_found=1;}
				if ((pcommand.name == "help")||(pcommand.name == "h")||(pcommand.name == "?")) {this->ow_commandline_help(&pcommand);command_found=1;}
				break;
			case OWGAME_MENU_APP_NEWGAME:
				if ((pcommand.name == "showcfg")||(pcommand.name == "cfg")) {this->ow_commandline_showgameconfig(&pcommand);command_found=1;}
				if ((pcommand.name == "set")||(pcommand.name == "SET")) {this->ow_commandline_setvalue(&pcommand);command_found=1;}
				if ((pcommand.name == "create")||(pcommand.name == "c")) {this->ow_commandline_creategame(&pcommand);command_found=1;}
				if ((pcommand.name == "help")||(pcommand.name == "h")||(pcommand.name == "?")) {this->ow_commandline_help(&pcommand);command_found=1;}
				if ((pcommand.name == "quit")||(pcommand.name == "q")) {this->ow_commandline_mainmenu(&pcommand);command_found=1;}
				break;
			case OWGAME_MENU_SETUPBOARD:
				if ((pcommand.name == "board")||(pcommand.name == "b")) {this->ow_commandline_showtxtboard(&pcommand);command_found=1;}
				if ((pcommand.name == "gfxboard")||(pcommand.name == "gfx")) {this->gfxboard_showhidde();command_found=1;}
				if ((pcommand.name == "put")||(pcommand.name == "p")) {this->ow_commandline_setupboard_put(&pcommand);command_found=1;}
				if ((pcommand.name == "take")||(pcommand.name == "t")) {this->ow_commandline_setupboard_take(&pcommand);command_found=1;}
				if ((pcommand.name == "help")||(pcommand.name == "h")||(pcommand.name == "?")) {this->ow_commandline_help(&pcommand);command_found=1;}
				if ((pcommand.name == "confirm")||(pcommand.name == "ok")) {this->ow_commandline_setupboard_confirm(&pcommand);command_found=1;}
				if ((pcommand.name == "cancel")||(pcommand.name == "main")||(pcommand.name == "..")) {this->ow_commandline_setupboard_cancel(&pcommand);command_found=1;}
				break;
			case OWGAME_MENU_PLY:
				if ((pcommand.name == "board")||(pcommand.name == "b")) {this->ow_commandline_showtxtboard(&pcommand);command_found=1;}
				if ((pcommand.name == "gfxboard")||(pcommand.name == "gfx")) {this->gfxboard_showhidde();command_found=1;}
				if ((pcommand.name == "move")||(pcommand.name == "mv")) {this->ow_commandline_move(&pcommand);command_found=1;}
				if ((pcommand.name == "rcapture")||(pcommand.name == "rc")) {this->ow_commandline_rcapture(&pcommand);command_found=1;}
				if ((pcommand.name == "undo")||(pcommand.name == "ud")) {this->ow_commandline_undolastturn(&pcommand);command_found=1;}
				if ((pcommand.name == "mainmenu")||(pcommand.name == "main")||(pcommand.name == "..")) {this->ow_commandline_mainmenu(&pcommand);command_found=1;}
				if ((pcommand.name == "manageai")||(pcommand.name == "ai")) {this->ow_commandline_manageai(&pcommand);command_found=1;}
				if ((pcommand.name == "help")||(pcommand.name == "h")||(pcommand.name == "?")) {this->ow_commandline_help(&pcommand);command_found=1;}
				break;
			case OWGAME_MENU_MAI:
				if ((pcommand.name == "showcfg")||(pcommand.name == "cfg")) {this->ow_commandline_showgameconfig(&pcommand);command_found=1;}
				if ((pcommand.name == "logmoves")||(pcommand.name == "logm")) {this->ow_commandline_showlogmoves(&pcommand);command_found=1;}
				if ((pcommand.name == "undom")||(pcommand.name == "udm")) {this->ow_commandline_undolastmove(&pcommand);command_found=1;}
				if ((pcommand.name == "redom")||(pcommand.name == "rdm")) {this->ow_commandline_redolastmove(&pcommand);command_found=1;}
				if ((pcommand.name == "batch")||(pcommand.name == "bat")) {this->ow_commandline_setbatch(&pcommand);command_found=1;}
				if ((pcommand.name == "logresult")||(pcommand.name == "logr")) {this->ow_commandline_setlogresults(&pcommand);command_found=1;}
				if ((pcommand.name == "mainmenu")||(pcommand.name == "main")||(pcommand.name == "..")) {this->ow_commandline_mainmenu(&pcommand);command_found=1;}
				if ((pcommand.name == "play")||(pcommand.name == "p")) {this->ow_commandline_play(&pcommand);command_found=1;}
				if ((pcommand.name == "help")||(pcommand.name == "h")||(pcommand.name == "?")) {this->ow_commandline_help(&pcommand);command_found=1;}
				break;
			case OWGAME_MENU_BAT:
				if ((pcommand.name == "mainmenu")||(pcommand.name == "main")) {this->ow_commandline_mainmenu(&pcommand);command_found=1;}
				if ((pcommand.name == "help")||(pcommand.name == "h")||(pcommand.name == "?")) {this->ow_commandline_help(&pcommand);command_found=1;}
				break;
		}
		if (!command_found) {
				cout << "Command not found. Please type 'h' for help on available commands." << endl;
		} else {
			add_history(command);
		}
	}
}

/**
* Parses the command line received, generating a parsed command to execute.
* @param command Command line to parse.
* @param pcommand pointer to the variable to hold the parsed command data (parameters,etc)
*/
void ow_game::commandline_parse_command(char *command, parsed_command *pcommand) {
	string strcommand = command;
	string words[3]; int wordidx; int inipos; int foundpos;
	//cout << strcommand << endl;
	pcommand->name 		= "";
	pcommand->param1 	= "";
	pcommand->param2 	= "";
	wordidx = -1; inipos = 0; foundpos=-1;
	while (((foundpos=strcommand.find(" ",inipos)))&&(wordidx<3)) {
			wordidx++;
			if (wordidx==0) { pcommand->name = strcommand.substr(inipos,foundpos-inipos);}
			if (wordidx==1) { pcommand->param1 = strcommand.substr(inipos,foundpos-inipos);}
			if (wordidx==2) { pcommand->param2 = strcommand.substr(inipos,foundpos-inipos);}
			inipos=foundpos+1;
			//cout << foundpos << endl;
			if (foundpos<0) break;
	}
	// Convert the command to lower case, to make it case insensitive	
	transform (pcommand->name.begin(),pcommand->name.end(),pcommand->name.begin(), ptr_fun(::tolower) );
}	// Param1 y param2 are NOT case insensitive (ex. move D1, move d1)

/**
* Manages the commandline (readline) completion.
* It manages if it is a 'appcommand' completion, or a savegame filename completion,
* calling commandline_command_generator() or commandline_savegames_filter() accordingly.
* For parameters definition, refer to : readline, rl_completion_func_t.
*/
char **ow_game::commandline_completion(const char *text, int start, int end) {
  char **matches;char **matches_tmp;
	matches = (char **)NULL;
	matches_tmp = (char **)NULL;
	
	if (start == 0) {
		rl_attempted_completion_over = 1;
		rl_ignore_some_completions_function = NULL;
		matches = rl_completion_matches (text, this->commandline_command_generator_handler);
	} else {
		if (strstr(rl_line_buffer,"load") || strstr(rl_line_buffer,"save")) {
			rl_attempted_completion_over = 0;
			rl_ignore_some_completions_function = this->commandline_savegames_filter_handler;
		} else {rl_attempted_completion_over = 1;}
	}
	return matches;
}

/**
* Filters the 'readline' standard filename completion, removing all but the savegame files.
* For parameters definition refer to: readline, rl_compignore_func_t.
*/
int ow_game::commandline_savegames_filter(char **matches) {

 char **newnames;
	int nidx; int idx;
	for (nidx = 1; matches[nidx]; nidx++) ; // counts the items in array
	newnames = (char **)malloc ((nidx + 1) * sizeof (char *));
	newnames[0] = matches[0];
  for (idx = nidx = 1; matches[idx]; idx++) {
		if (strstr(matches[idx],".sav")) {
			newnames[nidx++] = matches[idx];
		} else {free (matches[idx]);}
	}
	newnames[nidx] = (char *)NULL;
  for (nidx = 1; newnames[nidx]; nidx++) {
	  matches[nidx] = newnames[nidx];
		idx++;
	}
	if (nidx==1) matches[0] = newnames[0];	
  matches[nidx] = (char *)NULL;
  free (newnames);

	return 1;
}

/**
* Supports the 'readline' completion method generating the completion for 'appcommands'.
* For parameters definition, refer to: readline, rl_compentry_func_t.
*/
char *ow_game::commandline_command_generator(const char *text, int state) {
	static int list_index, len;
	string comname;
	char *r;

	if (!state) {list_index = 0;len = strlen (text);}
	switch (this->current_menu) {
		case OWGAME_MENU_APP:
			while (list_index<OWGAME_MENU_APP_COMMANDS) {
				comname = app_commands_help[list_index].commname; list_index++;
				if (strncmp (comname.c_str(), text, len) == 0) {r = (char *)malloc (strlen (comname.c_str()) + 1);strcpy (r, comname.c_str());return (r); }
			}
			break;
		case OWGAME_MENU_APP_NEWGAME: 
			while (list_index<OWGAME_MENU_APP_NEWGAME_COMMANDS) {
				comname = app_newgame_commands_help[list_index].commname; list_index++;
				if (strncmp (comname.c_str(), text, len) == 0) {r = (char *)malloc (strlen (comname.c_str()) + 1);strcpy (r, comname.c_str());return (r); }
			}
			break;
		case OWGAME_MENU_SETUPBOARD: 
			while (list_index<OWGAME_MENU_SETUPBOARD_COMMANDS) {
				comname = setupboard_commands_help[list_index].commname; list_index++; 
				if (strncmp (comname.c_str(), text, len) == 0) {r = (char *)malloc (strlen (comname.c_str()) + 1);strcpy (r, comname.c_str());return (r); }
			}
			break;
		case OWGAME_MENU_PLY:
			while (list_index<OWGAME_MENU_PLY_COMMANDS) { 
				comname = play_commands_help[list_index].commname; list_index++;
			if (strncmp (comname.c_str(), text, len) == 0) {r = (char *)malloc (strlen (comname.c_str()) + 1);strcpy (r, comname.c_str());return (r); }
			}
			break;
		case OWGAME_MENU_MAI:
			while (list_index<OWGAME_MENU_MAI_COMMANDS) { 
				comname = ai_commands_help[list_index].commname; list_index++;
				if (strncmp (comname.c_str(), text, len) == 0) {r = (char *)malloc (strlen (comname.c_str()) + 1);strcpy (r, comname.c_str());return (r); }
			}
			break;
		case OWGAME_MENU_BAT:
			while (list_index<OWGAME_MENU_BAT_COMMANDS) { 
				comname = ai_batch_commands_help[list_index].commname; list_index++;
				if (strncmp (comname.c_str(), text, len) == 0) {r = (char *)malloc (strlen (comname.c_str()) + 1);strcpy (r, comname.c_str());return (r); }
			}
			break;
		}

	return (char *)NULL;
}

/**
* Ends the game and deletes the associated resources (board,players,etc)
* Sets the application status to 'no game created' status.
* @param pcommand Command info and parameters for the 'endgame' command.
*/
int ow_game::ow_commandline_endgame(parsed_command *pcommand) {
	char prompt[64];
	rl_callback_handler_install("", this->commandline_handler_null);

	if (this->status>=OWGAME_STATUS_CREATED) {
		this->endgame();
		this->status = OWGAME_STATUS_RUNNING;
		cout << "Game ended successfully." << endl;
		if (this->queue_pos < this->queue_last) {
			this->queue_pos = 0;this->queue_last=0;
			cout << "There was a batch queue. It has been canceled because you ended the game." << endl;
		}
	} else cout << "Cannot end game. No game exists." << endl;	

	this->ow_commandline_getpromptstring(prompt);
	rl_callback_handler_install(prompt, this->commandline_handler);	
}

/**
* Goes into the 'create new game' configuration screen/menu.
* @param pcommand Command info and parameters (none) for the 'newgame' command.
* @return 1 on success, -1 on any error
*/
int ow_game::ow_commandline_newgame(parsed_command *pcommand) {
	char prompt[64];
	rl_callback_handler_install("", this->commandline_handler_null);

	if (this->status<OWGAME_STATUS_CREATED) {
		this->current_menu = OWGAME_MENU_APP_NEWGAME;
		this->ow_commandline_help(NULL);
	} else cout << "Cannot create game. A game already exists." << endl;	

	this->ow_commandline_getpromptstring(prompt);
	rl_callback_handler_install(prompt, this->commandline_handler);
	return 1;
}

/**
* Load a game into the current game, from the file specified as second parameter
* @param pcommand Command info (p1=filename)
* @return 1 on success, -1 on any error
*/
int ow_game::ow_commandline_loadgame(parsed_command *pcommand) {
	char prompt[64];
	int b_loaderror = -1;
	//rl_callback_handler_install("", this->commandline_handler_null);
	if (pcommand->param1.empty()||!pcommand->param2.empty()) {
		cout << "Incorrect syntax, try: loadgame <FILENAME>" << endl;
	} else {b_loaderror = this->loadgame(pcommand->param1);}
	//this->ow_commandline_getpromptstring(prompt);
	//rl_callback_handler_install(prompt, this->commandline_handler);
	if (b_loaderror==-1) cout << "Error loading savegame. Please check savegame file format." << endl;
	if (b_loaderror==-2) {b_loaderror=-1;} //-2 cannot open file
 	return b_loaderror;
}

/**
* Save a game the current game into the file specified as second parameter
* @param pcommand Command info (p2=filename)
* @return 1 on success, -1 on any error
*/
int ow_game::ow_commandline_savegame(parsed_command *pcommand) {
	char prompt[64];
	rl_callback_handler_install("", this->commandline_handler_null);
	if (pcommand->param1.empty()||!pcommand->param2.empty()) {
		cout << "Incorrect syntax, try: savegame <FILENAME>" << endl;
	} else {
		if (pcommand->param1.substr(pcommand->param1.length()-4,4) != ".sav") {
			cout << "Incorrect filename, savegame file must have '.sav' extension." << endl;
		} else {	
			if (this->status>=OWGAME_STATUS_CREATED) {
				if (this->savegame(pcommand->param1)!=-1) {cout << "Game saved sucessfully." << endl;} 
				else {cout << "Error saving, cannot create file '" << pcommand->param1 << "'." << endl;}
			} else {cout << "Cannot save game. No current game exists, create or load a new game first." << endl;}
		}
	}
	this->ow_commandline_getpromptstring(prompt);
	rl_callback_handler_install(prompt, this->commandline_handler);
	return 1;
}


/**
* Creates a new game (board,players,etc) based on the this->config defined in the 'newgame' menu.
* @param pcommand Command info and parameters (none) for the 'creategame' command.
*/
int ow_game::ow_commandline_creategame(parsed_command *pcommand) {
	int b_created = 0;
	char prompt[64];
	rl_callback_handler_install("", this->commandline_handler_null);

	if (this->status<OWGAME_STATUS_CREATED) {
		if (pcommand!=NULL) { /* called from newgame */
			this->config.turn = 1; this->config.plays = this->config.fturn_player;
		}
		this->createnewgame_useconfig(); b_created = 1;
		this->status = OWGAME_STATUS_CREATED; 
		this->current_menu = OWGAME_MENU_APP;  

		// If first player is HUMAN goes directly to play.
		if (this->selected_player->getaimode()==AI_HUMAN) {
			this->status = OWGAME_STATUS_PLAYING; // automatically go to play
			this->current_menu = OWGAME_MENU_PLY;  // automatically go to play
			if (pcommand!=NULL) /* called from newgame */ 
				cout << "Game created sucessfully. Game starts now!" << endl;//cout << "Game created sucessfully. Game begins!" << endl;
			else cout << "Game loaded sucessfully. Game continues now!" << endl;//cout << "Game loaded sucessfully. Game continues!" << endl;

		} else { //if not human players
			if (pcommand!=NULL) /* called from newgame */ 
				cout << "Game created sucessfully. Use 'play' to start game." << endl;//cout << "Game created sucessfully. Game begins!" << endl;
			else cout << "Game loaded sucessfully. Use 'play' to continue game." << endl;//cout << "Game loaded sucessfully. Game continues!" << endl;
		}

	} else {
		if (pcommand!=NULL) /* called from newgame */ 
			cout << "Error: Cannot create game. A game already exists." << endl;	
		else cout << "Error: Cannot load game. A game already exists." << endl;	
	}

	this->ow_commandline_getpromptstring(prompt);
	rl_callback_handler_install(prompt, this->commandline_handler);

	return b_created;
}

/**
* Refresh the commandline prompt display
*/
void ow_game::ow_commandline_refreshprompt() {
	char prompt[64];
	rl_callback_handler_install("", this->commandline_handler_null);
	cout << endl;
	this->ow_commandline_getpromptstring(prompt);
	rl_callback_handler_install(prompt, this->commandline_handler);
}

/**
* Shows the app version info
* @param pcommand Command info and parameters (none) for the 'version' command.
*/
void ow_game::ow_commandline_version(parsed_command *pcommand) {
	char prompt[64];
	rl_callback_handler_install("", this->commandline_handler_null);
	this->show_version();
	this->ow_commandline_getpromptstring(prompt);
	rl_callback_handler_install(prompt, this->commandline_handler);
}

/**
* Command line wrapper to the 'exitomweso' function.
* @param pcommand Command info and parameters (none) for the 'quit' command.
*/
int ow_game::ow_commandline_quit(parsed_command *pcommand) {
	rl_callback_handler_install("", this->commandline_handler_null);
	this->exitomweso();
}

/**
* Goes into the 'setupboard' mode and 'setup menu' screen.
* Sets the application to the OWGAME_STATUS_PLAYING status.
* @param pcommand Command info and parameters (none) for the 'setup' command.
* @return -1 if game no created and cannot go to setup. 1 if ok.
*/
int ow_game::ow_commandline_setupboard(parsed_command *pcommand) {
	char prompt[64];

	if (this->status>=OWGAME_STATUS_CREATED) {
		this->current_menu = OWGAME_MENU_SETUPBOARD;
		//this->status=OWGAME_STATUS_PLAYING;
		this->owtab->savebackup();
		this->ow_commandline_help(NULL);
		cout << "Setting up the board will set your setup as initial position of the board\nand will reset game deleting all history." << endl;
		rl_callback_handler_install("", this->commandline_handler_null);
		this->ow_commandline_getpromptstring(prompt);
		rl_callback_handler_install(prompt, this->commandline_handler);
		return 1;
	} else {
		cout << "No game created. To setup board, create a game first." << endl;
		return -1;
	}
}

/**
* Setups board. Adds seeds to the specified cell of the current game board.
* It will check if max number of seeds in board is reached before putting.
* @param pcommand Command put and parameters (param1=cellname, param2=numseeds) for the 'put' command.
*	@return 0 on sucess, -1 if no put can be performed.
*/
int ow_game::ow_commandline_setupboard_put(parsed_command *pcommand) {
	char prompt[64];
	int merror; cellXY tcell;
	int i_seeds, i_seedserror;
	rl_callback_handler_install("", this->commandline_handler_null);
	merror = -1;

	if (pcommand->param1.empty()||pcommand->param2.empty()) {
		cout << "Incorrect syntax, try: put <CELLNAME> <SEEDS>" << endl;
		this->ow_commandline_getpromptstring(prompt);
		rl_callback_handler_install(prompt, this->commandline_handler);
		return -1;
	}
	
	if (this->status == OWGAME_STATUS_WON) {
		cout << "Game finished. Player [" << this->selected_player->getplayername() << "] WON!." << endl << "Only running games could be setup." << endl;
		this->ow_commandline_getpromptstring(prompt);
		rl_callback_handler_install(prompt, this->commandline_handler);
		return -1;
	}
	
	if (this->owtab->endofturn()) {
		tcell = this->ow_commandline_translate_name2coord(pcommand->param1);
		i_seedserror = sscanf(pcommand->param2.c_str(),"%d",&i_seeds);
		if ((tcell.boardside>=0)&&(i_seedserror>0)) {
			if (this->owtab->bsetup_putseeds(i_seeds,tcell.boardside, tcell.x, tcell.y)<=0) merror=-1; else merror=0;
		}
		//cout << "Put(" << tcell.x << "," << tcell.y << "," << tcell.boardside << ") -> " << this->ow_commandline_translate_coord2name(tcell.boardside, tcell.x, tcell.y) << " result:" << merror << endl;
		if (merror<0) cout << "Illegal put action";
		if (tcell.boardside<0) cout << ": illegal cell name '" << pcommand->param1 << "'." << endl;
		if (i_seedserror<=0) cout << ": illegal number of seeds '" << pcommand->param2 << "'." << endl;
		if ((merror<0)&&(tcell.boardside>=0)&&(i_seedserror>0)) cout << ": board cannot hold '" << pcommand->param2 << "' more seed(s) ." << endl;
		if (merror>=0) { 
			cout << endl << "Putting ..." << endl;
			//cout << endl;
			//SHOW BOARD AGAIN draw_txt()
			this->ow_commandline_showtxtboard(NULL);
			cout << "Done:'" << pcommand->param2 << "' seed(s) put into '" << pcommand->param1 << "'." << endl << endl;
		}
	} else {
		cout << "Cannot PUT: Board busy. Wait until board is ready." << endl;
	}


	//if (merror<0) {
		this->ow_commandline_getpromptstring(prompt);
		rl_callback_handler_install(prompt, this->commandline_handler);
	//}

	return merror;
}


/**
* Show the list of moves recently executed. It is used by UnDo/ReDo commands.
* @param pcommand Command showlog and parameters (none) for the 'logm' command.
*	@return Number of moves saved in the logmoves list.
*/
int ow_game::ow_commandline_showlogmoves(parsed_command *pcommand) {
  cout << "List of game moves:" << endl;
	if (this->logmovescount<=0) { cout << "  No moves available." << endl; return -1;}
	for (int idx=0;idx<this->logmovescount; idx++) {
		cout << " ";
		if ((idx==this->logmovescurrent)&&(this->logmovescurrent!=this->logmovescount)) cout << "*"; else cout << " ";
		if (this->logmoves[idx].player->getplayernum()==PL_PLAYER1) cout << "P1"; else cout << "P2";
		if (this->logmoves[idx].player->getboardside()==BRD_SIDE_NORTH) cout << "[N"; else cout << "[S";
		cout << this->logmoves[idx].turn << "]: ";
		cout << this->logmoves[idx].stepcommand.name << " " << this->logmoves[idx].stepcommand.param1 << " " << this->logmoves[idx].stepcommand.param2 << endl;
	}	
	
	// if it is the last move, then print the 'waiting input' logline.
	if (this->logmovescurrent==this->logmovescount) {
		if (this->selected_player->getplayernum()==PL_PLAYER1) cout << " *P1"; else cout << " *P2";
		if (this->selected_player->getboardside()==BRD_SIDE_NORTH) cout << "[N"; else cout << "[S";
		cout << this->current_turn;
    cout << "]: Waiting input" << endl;
	}
	return this->logmovescount;
}

/*
* Sets the logmovesfile preferences.
* Logmoves preferences cannot be changed when game has already started.
* @param pcommand Command parameters (param1=ON|OFF, param2=logfilename).
*	@return 0 on sucess, -1 on any error
*/
/*
int ow_game::ow_commandline_setlogmoves(parsed_command *pcommand) {
	char prompt[64];
	int merror; cellXY tcell;
	int i_seeds, i_seedserror;
	rl_callback_handler_install("", this->commandline_handler_null);
	merror = -1;

	if (pcommand->param1.empty()) {
		cout << "Logmoves is " << this->config.logmoves << ", ";
		if (this->config.logmoves=="OFF") cout << "not ";
		cout << "writing moves to file: " << this->config.logmovesfile << endl;
		cout << "To change config use: logm [ON|OFF [<LOGMOVESFILE>]]" << endl;
		this->ow_commandline_getpromptstring(prompt);
		rl_callback_handler_install(prompt, this->commandline_handler);
		return 0;
	}

  if ((pcommand->param1=="ON")||(pcommand->param1=="OFF")) {
		if ((this->current_turn!=1)||
				((this->selected_player->getplayernum()==PL_PLAYER1)&&(this->config.fturn_player!="P1")) ||
				((this->selected_player->getplayernum()==PL_PLAYER2)&&(this->config.fturn_player!="P2"))
			 ) {
			cout << "The game is already running, you cannot change the logmovesfile preferences." << endl;
			this->ow_commandline_getpromptstring(prompt);
			rl_callback_handler_install(prompt, this->commandline_handler);
			return -1;
		}
		if (this->config.logmoves!=pcommand->param1) {
			this->config.logmoves = pcommand->param1;
			cout << "Logmoves changed to " << this->config.logmoves << endl;
			if ((this->config.logmoves=="OFF")&&(this->gamecreated())) this->logmovesfile.close();
			if ((this->config.logmoves=="ON")&&(this->gamecreated())) this->logmovesfile.open(this->config.logmovesfile.c_str());
		}
	} else { 
		cout << "Incorrect syntax, try: logm [ON|OFF [<LOGMOVESFILE>]]" << endl;
		this->ow_commandline_getpromptstring(prompt);
		rl_callback_handler_install(prompt, this->commandline_handler);
		return -1;
	}

	if (!pcommand->param2.empty()) {
		if (pcommand->param2.substr(pcommand->param2.length()-4,4) != ".smo") {
			cout << "Incorrect filename, logmoves file must have '.smo' extension." << endl;
			this->ow_commandline_getpromptstring(prompt);
			rl_callback_handler_install(prompt, this->commandline_handler);
			return -1;
		} else {
			if (this->config.logmovesfile!=pcommand->param2) {
				this->config.logmovesfile = pcommand->param2;
				cout << "Logmovesfile changed to '" << this->config.logmovesfile << "'" << endl;
				if ((this->config.logmoves=="ON")&&(this->gamecreated())) {
					this->logmovesfile.close();
					this->logmovesfile.open(this->config.logmovesfile.c_str());
				}
			}
		}	

	}

	cout << "Logmoves is " << this->config.logmoves << ", ";
	if (this->config.logmoves=="OFF") cout << "not ";
	cout << "writing moves to file: " << this->config.logmovesfile << endl;
	this->ow_commandline_getpromptstring(prompt);
	rl_callback_handler_install(prompt, this->commandline_handler);
	return 0;
}
*/



/**
* Sets the logresults file preferences.
* De/Activates saving results to a logfile, and additionally setups the logfile name.
* @param pcommand Command parameters (param1=ON|OFF, param2=logfilename).
*	@return 0 on sucess, -1 on any error
*/

int ow_game::ow_commandline_setlogresults(parsed_command *pcommand) {
	char prompt[64];
	int merror; cellXY tcell;
	int i_seeds, i_seedserror;
	rl_callback_handler_install("", this->commandline_handler_null);
	merror = -1;

	if (pcommand->param1.empty()) {
		cout << "Logresults is " << this->logresults << ", ";
		if (this->logresults=="OFF") cout << "not ";
		cout << "writing results to file: " << this->logresultsfile << endl;
		cout << "To change config use: logr [ON|OFF [<LOGRESULTS_FILENAME>]]" << endl;
		this->ow_commandline_getpromptstring(prompt);
		rl_callback_handler_install(prompt, this->commandline_handler);
		return 0;
	}

	//Convert the VARNAME to uppercase
	transform (pcommand->param1.begin(), pcommand->param1.end(), pcommand->param1.begin(), ptr_fun(::toupper) );

  if ((pcommand->param1=="ON")||(pcommand->param1=="OFF")) {
		if (this->logresults!=pcommand->param1) {
			this->logresults = pcommand->param1;
			cout << "Logresults changed to " << this->logresults << endl;
		}
	} else { 
		cout << "Incorrect syntax, try: logr [ON|OFF [<LOGRESULTS_FILENAME>]]" << endl;
		this->ow_commandline_getpromptstring(prompt);
		rl_callback_handler_install(prompt, this->commandline_handler);
		return -1;
	}

	if (!pcommand->param2.empty()) {
		if (pcommand->param2.substr(pcommand->param2.length()-4,4) != ".log") {
			cout << "Incorrect filename, logresults file must have '.log' extension." << endl;
			this->ow_commandline_getpromptstring(prompt);
			rl_callback_handler_install(prompt, this->commandline_handler);
			return -1;
		} else {
			if (this->logresultsfile!=pcommand->param2) {
				this->logresultsfile = pcommand->param2;
				cout << "Logresults file changed to '" << this->logresultsfile << "'" << endl;
			}
		}	

	}

	cout << "Logresults is " << this->logresults << ", ";
	if (this->logresults=="OFF") cout << "not ";
	cout << "writing results to file: " << this->logresultsfile << endl;
	this->ow_commandline_getpromptstring(prompt);
	rl_callback_handler_install(prompt, this->commandline_handler);
	return 0;
}






/**
* Setups board. Take seeds from the specified cell of the current game board.
* @param pcommand Command put and parameters (param1=cellname, param2=numseeds) for the 'take' command.
*	@return 0 on sucess, -1 if no put can be performed.
*/
int ow_game::ow_commandline_setupboard_take(parsed_command *pcommand) {
	char prompt[64];
	int merror; cellXY tcell;
	int i_seeds, i_seedserror;
	rl_callback_handler_install("", this->commandline_handler_null);
	merror = -1;

	if (pcommand->param1.empty()||pcommand->param2.empty()) {
		cout << "Incorrect syntax, try: take <CELLNAME> <SEEDS>" << endl;
		this->ow_commandline_getpromptstring(prompt);
		rl_callback_handler_install(prompt, this->commandline_handler);
		return -1;
	}
	
	if (this->status == OWGAME_STATUS_WON) {
		cout << "Game finished. Player [" << this->selected_player->getplayername() << "] WON!." << endl << "Only running games could be setup." << endl;
		this->ow_commandline_getpromptstring(prompt);
		rl_callback_handler_install(prompt, this->commandline_handler);
		return -1;
	}
	
	if (this->owtab->endofturn()) {
		tcell = this->ow_commandline_translate_name2coord(pcommand->param1);
		i_seedserror = sscanf(pcommand->param2.c_str(),"%d",&i_seeds);
		if ((tcell.boardside>=0)&&(i_seedserror>0)) {
			if (this->owtab->bsetup_takeseeds(i_seeds,tcell.boardside, tcell.x, tcell.y)<=0) merror=-1; else merror=0;
		}
		//cout << "Put(" << tcell.x << "," << tcell.y << "," << tcell.boardside << ") -> " << this->ow_commandline_translate_coord2name(tcell.boardside, tcell.x, tcell.y) << " result:" << merror << endl;
		if (merror<0) cout << "Illegal take action";
		if (tcell.boardside<0) cout << ": illegal cell name '" << pcommand->param1 << "'." << endl;
		if (i_seedserror<=0) cout << ": illegal number of seeds '" << pcommand->param2 << "'." << endl;
		if ((merror<0)&&(tcell.boardside>=0)&&(i_seedserror>0)) cout << ": cannot take '" << pcommand->param2 << "' seeds from board." << endl;
		if (merror>=0) { 
			cout << endl << "Taking ..." << endl;
			//cout << endl;
			//SHOW BOARD AGAIN draw_txt()
			this->ow_commandline_showtxtboard(NULL);
			cout << "Done:'" << pcommand->param2 << "' seed(s) taken from '" << pcommand->param1 << "'." << endl << endl;
		}
	} else {
		cout << "Cannot TAKE: Board busy. Wait until board is ready." << endl;
	}

	//if (merror<0) {
		this->ow_commandline_getpromptstring(prompt);
		rl_callback_handler_install(prompt, this->commandline_handler);
	//}	
	return merror;
}

/**
* Checks that the correct number of seeds are in the board and goes to the mainmenu.
* @param pcommand Command info and parameters (none) for the 'setupboard_confirm' command.
* @return -1 if cannot confirm setup. 0 if ok.
*/
int ow_game::ow_commandline_setupboard_confirm(parsed_command *pcommand) {
	char prompt[64];
	if (this->owtab->counthandseeds()==0) {
		//confirm setup delete undo queue
		this->logmovescurrent = 0;this->logmovescount = 0;
		//confirm setup save as initial position
		this->owtab->initialboard_setcurrent();
		// currentturn becomes firstturn, the same with fturn_player parameter
		this->current_turn = 1;
		if (this->selected_player->getplayernum()==PL_PLAYER1) {
			this->config.fturn_player="P1";
		} else {this->config.fturn_player="P2";}

		//back to mainmenu
		this->current_menu = OWGAME_MENU_APP;
		this->ow_commandline_help(NULL);
		rl_callback_handler_install("", this->commandline_handler_null);
		this->ow_commandline_getpromptstring(prompt);
		rl_callback_handler_install(prompt, this->commandline_handler);
		return 0;
	} else {
		cout << "Cannot confirm setup: '" << this->owtab->counthandseeds() << "' more seed(s) need(s) to be PUT in the board." << endl;
		return -1;
	}
}

/**
* Cancels the current setup, returning the board to the previous status.
* @param pcommand Command info and parameters (none) for the 'setupboard_cancel' command.
* @return -1 if cannot cancel. 0 if ok.
*/
int ow_game::ow_commandline_setupboard_cancel(parsed_command *pcommand) {
	char prompt[64];
	this->owtab->restorebackup();
	this->current_menu = OWGAME_MENU_APP;
	cout << "Board setup cancelled." << endl;
	rl_callback_handler_install("", this->commandline_handler_null);
	this->ow_commandline_getpromptstring(prompt);
	rl_callback_handler_install(prompt, this->commandline_handler);
	return 0;
}



/**
* UnDo the last move executed
* @param pcommand Command info and parameters (none) for the 'undom' command.
* @return -1 on fail, 1 on success
*/
int ow_game::ow_commandline_undolastmove(parsed_command *pcommand) {
	int result;
	result = this->undolastmove(1);
	if (result>=0) cout << "Last move UnDone successfully." << endl;
		else cout << "Cannot UnDo more moves from the undo queue." << endl;
	return result;
}

/**
* UnDo the last turn (usually 2 moves) executed
* @param pcommand Command info and parameters (none) for the 'undo' command.
* @return -1 on fail, 1 on success
*/
int ow_game::ow_commandline_undolastturn(parsed_command *pcommand) {
	char prompt[64];	int result;
	rl_callback_handler_install("", this->commandline_handler_null);
	result = this->undolastmove(2);
	if (result>=0) cout << "Last turn UnDone successfully." << endl;
		else cout << "Cannot UnDo more turns from the undo queue." << endl;
	this->ow_commandline_getpromptstring(prompt);
	rl_callback_handler_install(prompt, this->commandline_handler);
	return result;
}

/**
* ReDo the last move 'undone' by the 'undom' command.
* @param pcommand Command info and parameters (none) for the 'redo' command.
* @return -1 on fail, 1 on success
*/
int ow_game::ow_commandline_redolastmove(parsed_command *pcommand) {
	int result;
	result = this->redolastundo(1);
	if (result<0) cout << "Cannot ReDo more moves from the undo queue." << endl;
	else cout << "Last move ReDone successfully." << endl;
	return result;
}


/**
* Goes into the 'playgame' mode and 'play menu' screen.
* Sets the application to the OWGAME_STATUS_PLAYING status.
* @param pcommand Command info and parameters (none) for the 'play' command.
* @return -1 if game no created and cannot go to play. 1 if ok.
*/
int ow_game::ow_commandline_play(parsed_command *pcommand) {
	char prompt[64];

	if (this->status>=OWGAME_STATUS_CREATED) {
		this->current_menu = OWGAME_MENU_PLY;
		this->status=OWGAME_STATUS_PLAYING;
		this->ow_commandline_help(NULL);
		rl_callback_handler_install("", this->commandline_handler_null);
		this->ow_commandline_getpromptstring(prompt);
		rl_callback_handler_install(prompt, this->commandline_handler);
		return 1;
	} else {
		cout << "No game created. To play, create a game first." << endl;
		return -1;
	}
}


/**
* Goes into the 'show/edit batch games' mode/menu screen.
* Let's create a batch of games to be run
* @param pcommand Command info and parameters (none) for the 'batch' command.
* @return -1 if error. 1 if ok.
*/
int ow_game::ow_commandline_batch(parsed_command *pcommand) {
	char prompt[64];

	this->current_menu = OWGAME_MENU_BAT;
	this->ow_commandline_help(NULL);
	rl_callback_handler_install("", this->commandline_handler_null);
	this->ow_commandline_getpromptstring(prompt);
	rl_callback_handler_install(prompt, this->commandline_handler);
}

/**
* Sets to run various games with the current configuration.
* @param pcommand Command info and parameters (param1=number of games) for the 'batch' command.
* @return -1 if error. 1 if ok.
*/
int ow_game::ow_commandline_setbatch(parsed_command *pcommand) {
	char prompt[64];
	int ngames, i_ngameserror;
	
	i_ngameserror = sscanf(pcommand->param1.c_str(),"%d",&ngames);

	if (pcommand->param1.empty()||(i_ngameserror<=0)||(ngames<=0)) {
		cout << "Incorrect syntax, try: batch <NUMBER_OF_GAMES>" << endl;
		this->ow_commandline_getpromptstring(prompt); 
		rl_callback_handler_install(prompt, this->commandline_handler);
		return -1;
	}


	rl_callback_handler_install("", this->commandline_handler_null);
	this->queue_pos = 0; this->queue_last = ngames-1;

	// Draw the batch queue info
	if (this->queue_pos < this->queue_last) {
		cout << " The batch queue has been set to [" << this->queue_last - this->queue_pos + 1 << "] game(s)." << endl;
		if (this->gamecreated()) {
			cout << " The current game will run [" << this->queue_last - this->queue_pos + 1 << "] times, sequentially." << endl;
		} else {
		  cout << " The next game you create/play will run [" << this->queue_last - this->queue_pos + 1 << "] times, sequentially." << endl;
		}
  }

	this->ow_commandline_getpromptstring(prompt);
	rl_callback_handler_install(prompt, this->commandline_handler);
	
}


/**
* Goes into the 'manage artificial intelligence' mode and 'manageai menu' screen.
* @param pcommand Command info and parameters (none) for the 'manageai' command.
*/
int ow_game::ow_commandline_manageai(parsed_command *pcommand) {
	char prompt[64];
	this->current_menu = OWGAME_MENU_MAI;
	this->ow_commandline_help(NULL);
	rl_callback_handler_install("", this->commandline_handler_null);
	this->ow_commandline_getpromptstring(prompt);
	rl_callback_handler_install(prompt, this->commandline_handler);
}

/**
* Goes into the 'mainmenu' screen allowing to load/save and create new games.
* @param pcommand Command info and parameters (none) for the 'mainmenu' command.
*/
int ow_game::ow_commandline_mainmenu(parsed_command *pcommand) {
	char prompt[64];
	this->current_menu = OWGAME_MENU_APP;
	this->ow_commandline_help(NULL);
	rl_callback_handler_install("", this->commandline_handler_null);
	this->ow_commandline_getpromptstring(prompt);
	rl_callback_handler_install(prompt, this->commandline_handler);
}


/**
* Draws the board and game items in TXT format.
* @param pcommand Command info and parameters (none) for the 'board' command.
*/
void ow_game::ow_commandline_showtxtboard(parsed_command *pcommand) {
	//cout << endl << " OMWESO BOARD" << endl;
	this->owtab->draw_txt(); 
  if (this->gamesettingup()) {
		cout << "SETUP MODE. Seeds to put: " << this->owtab->counthandseeds() << endl;
	}
}

/**
* Sends a 'move/sow' order to the board, so it executes the movement order.
* It will check if the movement is allowed, and if no other move is being executed.
* @param pcommand Command info and parameters (pcommand.param1=cellname) for the 'move' command.
*	@return 0 on sucess, -1 if no movement can be performed.
*/
int ow_game::ow_commandline_move(parsed_command *pcommand) {
	char prompt[64];
	int merror; cellXY tcell;
	rl_callback_handler_install("", this->commandline_handler_null);
	merror = -1;

	if (pcommand->param1.empty()) {
		cout << "Incorrect syntax, try: move <CELLNAME>" << endl;
		this->ow_commandline_getpromptstring(prompt);
		rl_callback_handler_install(prompt, this->commandline_handler);
		return -1;
	}
	
	if (this->status == OWGAME_STATUS_WON) {
		cout << "Game finished. Player [" << this->selected_player->getplayername() << "] WON!." << endl << "End game and create a new one to play again." << endl;
		this->ow_commandline_getpromptstring(prompt);
		rl_callback_handler_install(prompt, this->commandline_handler);
		return -1;
	}
	
	if (this->owtab->endofturn()) {
		tcell = this->ow_commandline_translate_name2coord(pcommand->param1);
		if (tcell.boardside>=0) merror = this->selplayer_makemovement(tcell.boardside, tcell.x, tcell.y);
		//cout << "Move(" << tcell.x << "," << tcell.y << "," << tcell.boardside << ") -> " << this->ow_commandline_translate_coord2name(tcell.boardside, tcell.x, tcell.y) << " result:" << merror << endl;
		if (merror<0) cout << "Illegal movement";
		if (tcell.boardside<0) cout << ": illegal cell name '" << pcommand->param1 << "'.";
		if (merror>=0) cout << "Sowing ... "; else cout << endl;

	} else {
		cout << "Cannot move: Board busy. Wait until board is ready." << endl;
	}


	if (merror<0) {
		this->ow_commandline_getpromptstring(prompt);
		rl_callback_handler_install(prompt, this->commandline_handler);
	}

	return merror;
}


/**
* Sends a 'reverse capture' order to the board, so it executes reverse movement and capture.
* It will check if the reverse capture is allowed, and if no other move is being executed.
* @param pcommand Command info and parameters (pcommand.param1=cellname) for the 'rcapture' command.
*	@return 0 on sucess, -1 if no movement can be performed.
*/
int ow_game::ow_commandline_rcapture(parsed_command *pcommand) {
	char prompt[64];
	int merror; cellXY tcell;
	rl_callback_handler_install("", this->commandline_handler_null);
	merror = -1;
	
	if (this->status == OWGAME_STATUS_WON) {
		cout << "Game finished. Player [" << this->selected_player->getplayername() << "] WON!." << endl << "End game and create a new one to play again." << endl;
		this->ow_commandline_getpromptstring(prompt);
		rl_callback_handler_install(prompt, this->commandline_handler);
		return -1;
	}

	if (this->owtab->endofturn()) {
		tcell = this->ow_commandline_translate_name2coord(pcommand->param1);
		if (tcell.boardside>=0) merror = this->selplayer_rcapture(tcell.boardside, tcell.x, tcell.y);
		//cout << "Move(" << tcell.x << "," << tcell.y << "," << tcell.boardside << ") -> " << this->ow_commandline_translate_coord2name(tcell.boardside, tcell.x, tcell.y) << " result:" << merror << endl;
		if (merror<0) cout << "Illegal movement";
		if (tcell.boardside<0) cout << ": illegal cell name '" << pcommand->param1 << "'.";
		if (merror>=0) cout << "Sowing ...";
		cout << endl;
	} else {
		cout << "Cannot move: Board busy. Wait until board is ready." << endl;
	}
	if (merror<0) {
		this->ow_commandline_getpromptstring(prompt);
		rl_callback_handler_install(prompt, this->commandline_handler);
	}

	return merror;
}



/**
* Shows the help info for the command specified in pcommand parameter.
* @param pcommand Command info and parameters for the specified command.
*/
void ow_game::ow_commandline_help(parsed_command *pcommand) {
	
	cout << endl;

	switch (this->current_menu) {
		case OWGAME_MENU_APP:
				cout << " APPLICATION MAIN MENU" << endl << endl;
				for (int idx=0;idx<OWGAME_MENU_APP_COMMANDS;idx++) { 
					cout << "  " << app_commands_help[idx].commname << string(9-app_commands_help[idx].commname.length(),' ') << "[" << app_commands_help[idx].commshortname << "]" << string(6-app_commands_help[idx].commshortname.length(),' ') << app_commands_help[idx].commshortdesc << endl;
				}	
				break;
		case OWGAME_MENU_APP_NEWGAME:
				cout << " MAINMENU::NEWGAME MENU" << endl << endl;
				for (int idx=0;idx<OWGAME_MENU_APP_NEWGAME_COMMANDS;idx++) { 
					cout << "  " << app_newgame_commands_help[idx].commname << string(9-app_newgame_commands_help[idx].commname.length(),' ') << "[" << app_newgame_commands_help[idx].commshortname << "]" << string(4-app_newgame_commands_help[idx].commshortname.length(),' ') << app_newgame_commands_help[idx].commshortdesc << endl;
				}	
				break;
		case OWGAME_MENU_SETUPBOARD:
				cout << " MAINMENU::SETUPBOARD" << endl << endl;
				for (int idx=0;idx<OWGAME_MENU_SETUPBOARD_COMMANDS;idx++) { 
					cout << "  " << setupboard_commands_help[idx].commname << string(9-setupboard_commands_help[idx].commname.length(),' ') << "[" << setupboard_commands_help[idx].commshortname << "]" << string(5-setupboard_commands_help[idx].commshortname.length(),' ') << setupboard_commands_help[idx].commshortdesc << endl;
				}	
				break;
		case OWGAME_MENU_PLY:
				cout << " MAINMENU::PLAY MENU" << endl << endl;
				for (int idx=0;idx<OWGAME_MENU_PLY_COMMANDS;idx++) { 
					cout << "  " << play_commands_help[idx].commname << string(9-play_commands_help[idx].commname.length(),' ') << "[" << play_commands_help[idx].commshortname << "]" << string(5-play_commands_help[idx].commshortname.length(),' ') << play_commands_help[idx].commshortdesc << endl;
				}	
				break;
		case OWGAME_MENU_MAI:
				cout << " MAINMENU:: MANAGEAI MENU" << endl << endl;
				for (int idx=0;idx<OWGAME_MENU_MAI_COMMANDS;idx++) {
					cout << "  " << ai_commands_help[idx].commname << string(10-ai_commands_help[idx].commname.length(),' ') << "[" << ai_commands_help[idx].commshortname << "]" << string(5-ai_commands_help[idx].commshortname.length(),' ') << ai_commands_help[idx].commshortdesc << endl;
				}	
				break;
		case OWGAME_MENU_BAT:
				cout << " MANAGEAI:: BATCH MANAGEMENT MENU" << endl << endl;
				for (int idx=0;idx<OWGAME_MENU_BAT_COMMANDS;idx++) {
					cout << "  " << ai_batch_commands_help[idx].commname << string(10-ai_batch_commands_help[idx].commname.length(),' ') << "[" << ai_batch_commands_help[idx].commshortname << "]" << string(5-ai_batch_commands_help[idx].commshortname.length(),' ') << ai_batch_commands_help[idx].commshortdesc << endl;
				}	
				break;
	}

	cout << endl;

}


/**
* Shows this->config info, this info is usually used to create new games.
* @param pcommand Command info and parameters (none) for the specified command.
*/
void ow_game::ow_commandline_showgameconfig(parsed_command *pcommand) {
	cout << endl;
	cout << " Game configuration: " << endl;
	cout << "  P1NAME:           " << this->config.p1name << endl;
	cout << "  P1AI(P1DEPTH):    " << this->config.p1aimode << "(" << config.p1aidepth << ")" << endl;
	//cout << "  P1AI(DEPTH)/EVAL: " << this->config.p1aimode << "(" << config.p1aidepth << ")/" << config.p1aieval << endl;	
	cout << "  P1EVAL:           " << config.p1aieval << endl;
	cout << "  P2NAME:           " << this->config.p2name << endl;
	cout << "  P2AI(P2DEPTH):    " << this->config.p2aimode << "(" << config.p2aidepth << ")" << endl;
	cout << "  P2EVAL:           " << config.p2aieval << endl;
	cout << "  NUMCOLS:          " << this->config.numcols << endl;
	cout << "  FIRSTTURN:        " << this->config.fturn_player << endl;
	cout << "  GFXBOARD:         " << this->config.gfxboard << endl;
	cout << endl;

	// Draw the batch queue info
	if (this->queue_pos < this->queue_last) {
		cout << " The batch queue is set to [" << this->queue_last - this->queue_pos + 1 << "] game(s)." << endl;
		if (this->gamecreated()) {
			cout << " The current game will run [" << this->queue_last - this->queue_pos + 1 << "] times, sequentially." << endl;
		} else {
		cout << " The next game you create/play will run [" << this->queue_last - this->queue_pos + 1 << "] times, sequentially." << endl;
		}
		cout << endl;
  }

}

/**
* Sets the value of one of the configuration variables.
* Each variable can only be SET from the menu screen where it has scope.
* @param pcommand Command info and parameters (param1=VARNAME,param2=VARVALUE) for the specified command.
* @return -1 if fails setting, 1 if sucessfull.
*/
int ow_game::ow_commandline_setvalue(parsed_command *pcommand) {
	int command_set_exec = 0;
	int b_loaderror = -1;

	//Convert the VARNAME to uppercase
	transform (pcommand->param1.begin(), pcommand->param1.end(), pcommand->param1.begin(), ptr_fun(::toupper) );

	if ((pcommand->param1.empty())||(pcommand->param2.empty())) {
		this->ow_commandline_setcommand_help(pcommand);
	} else {
		switch (this->current_menu) {
			case OWGAME_MENU_APP_NEWGAME:
				if (pcommand->param1=="P1NAME") {b_loaderror = 1;this->config.p1name = pcommand->param2;command_set_exec=1;}
				if (pcommand->param1=="P1AI") {b_loaderror = this->ow_commandline_setplayer_aimode(pcommand->param1,pcommand->param2);command_set_exec=1;}
				if (pcommand->param1=="P1DEPTH") {b_loaderror = this->ow_commandline_setplayer_aidepth(pcommand->param1,pcommand->param2);command_set_exec=1;}
				if (pcommand->param1=="P1EVAL") {b_loaderror = this->ow_commandline_setplayer_aieval(pcommand->param1,pcommand->param2);command_set_exec=1;}
				if (pcommand->param1=="P2NAME") {b_loaderror = 1;this->config.p2name = pcommand->param2;command_set_exec=1;} 
				if (pcommand->param1=="P2AI") {b_loaderror = this->ow_commandline_setplayer_aimode(pcommand->param1,pcommand->param2);command_set_exec=1;}
				if (pcommand->param1=="P2DEPTH") {b_loaderror = this->ow_commandline_setplayer_aidepth(pcommand->param1,pcommand->param2);command_set_exec=1;}
				if (pcommand->param1=="P2EVAL") {b_loaderror = this->ow_commandline_setplayer_aieval(pcommand->param1,pcommand->param2);command_set_exec=1;}
				if (pcommand->param1=="NUMCOLS") {b_loaderror = this->ow_commandline_setnumcols(pcommand->param2);command_set_exec=1;}
				if (pcommand->param1=="FIRSTTURN") {b_loaderror = this->ow_commandline_setnextturn("FIRSTTURN",pcommand->param2);command_set_exec=1;}
				if (pcommand->param1=="GFXBOARD") {b_loaderror = this->ow_commandline_setgfxboard_status(pcommand->param2);command_set_exec=1;}
				break;
		}	

		if (pcommand->param2=="loadgame") /* it is called from loadgame() not from creategame() */ {
			if (pcommand->param1=="TURN") {}
			if (pcommand->param1=="NEXTTURN") {}	
		}
	
		if (!command_set_exec) {
				cout << "Variable '" << pcommand->param1 << "' not found at this level. Command not executed." << endl;
		}
	}
	return b_loaderror;
}

/**
* Command line command wrapper for the setnewturn() & setfirstturn() methods.
* @param variablename (FIRSTTURN/NEXTTURN)
* @param plyID Player StringIDs (P1,P2)
* @return 1 on success, -1 of failure.
*/
int ow_game::ow_commandline_setnextturn(string variablename, string plyID) {
	parsed_command pcommand;
	transform (plyID.begin(),plyID.end(),plyID.begin(), ptr_fun(::toupper) );
	if ((plyID!="P1")&&(plyID!="P2")) {
		pcommand.name = "SET"; pcommand.param1=variablename; pcommand.param2="";
		this->ow_commandline_setcommand_help(&pcommand);
		return -1;
	} else {
		this->config.fturn_player = plyID;
		return 1;
	}
}

/**
* Sets the number of columns in (this->config.numcols) the board.
* @param numcols Number of columns
* @return 1 on success, -1 on fail
*/
int ow_game::ow_commandline_setnumcols(string numcols) {
	parsed_command pcommand;
	if ((numcols!="3")&&(numcols!="4")&&(numcols!="5")&&(numcols!="6")&&(numcols!="7")&&(numcols!="8")) {
		pcommand.name = "SET"; pcommand.param1="NUMCOLS"; pcommand.param2="";
		this->ow_commandline_setcommand_help(&pcommand);
		return -1;
	} else {
		this->config.numcols = atoi(numcols.c_str());
		return 1;
	}
}

/**
* Sets the player AImode
* @param player_ai String selecting the player (P1AI,P2AI)
* @param aimode String selecting the aimode (HUMAN,SIMPLE,NEGAMAX,ALPHABETA,SCOUT)
* @return 1 on success, -1 on fail
*/
int ow_game::ow_commandline_setplayer_aimode(string player_ai, string aimode) {
	parsed_command pcommand;
	//Convert the AIMODE to uppercase
	transform (aimode.begin(), aimode.end(), aimode.begin(), ptr_fun(::toupper) );

	if ((aimode!="HUMAN")&&(aimode!="SIMPLE")&&(aimode!="NEGAMAX")&&(aimode!="ALPHABETA")&&(aimode!="SCOUT")) {
		pcommand.name = "SET"; pcommand.param1=player_ai; pcommand.param2="";
		this->ow_commandline_setcommand_help(&pcommand);
		return -1;
	} else {
		if (player_ai=="P1AI") {
			this->config.p1aimode = aimode; if (aimode=="HUMAN") {this->config.p1aieval="HUMAN";}
			if ((aimode!="HUMAN")&&(this->config.p1aieval=="HUMAN")) {this->config.p1aieval="RANDOM";}
			if ((aimode=="SIMPLE")||(aimode=="HUMAN")) {this->config.p1aidepth="1";}
		}
		if (player_ai=="P2AI") {
			this->config.p2aimode = aimode;if (aimode=="HUMAN") {this->config.p2aieval="HUMAN";}
			if ((aimode!="HUMAN")&&(this->config.p2aieval=="HUMAN")) {this->config.p2aieval="RANDOM";}
			if ((aimode=="SIMPLE")||(aimode=="HUMAN")) {this->config.p2aidepth="1";}
		}
		return 1;
	}
}

/**
* Sets the player AImode algoritm DEPTH
* @param player_ai String selecting the player (P1DEPTH,P2DEPTH)
* @param aidepth String selecting the aidepth (1,2,3)
* @return 1 on success, -1 on fail
*/
int ow_game::ow_commandline_setplayer_aidepth(string player_ai, string aidepth) {
	parsed_command pcommand;
	int depth, i_deptherror;
	i_deptherror = sscanf(aidepth.c_str(),"%d",&depth);

	if ((depth>0)&&(i_deptherror>0)) {
		if (player_ai=="P1DEPTH") {
			if ((depth>1)&&((this->config.p1aimode=="SIMPLE")||(this->config.p1aimode=="HUMAN"))) {
				pcommand.name = "SET"; pcommand.param1=player_ai; pcommand.param2="";
				this->ow_commandline_setcommand_help(&pcommand);
				return -1;
			} else this->config.p1aidepth=aidepth;
		}
		if (player_ai=="P2DEPTH") {
			if ((depth>1)&&((this->config.p2aimode=="SIMPLE")||(this->config.p2aimode=="HUMAN"))) {
				pcommand.name = "SET"; pcommand.param1=player_ai; pcommand.param2="";
				this->ow_commandline_setcommand_help(&pcommand);
				return -1;
			} else this->config.p2aidepth=aidepth;
		}
		return 1;
	} else {
		pcommand.name = "SET"; pcommand.param1=player_ai; pcommand.param2="";
		this->ow_commandline_setcommand_help(&pcommand);
		return -1;
	}
}

/**
* Sets the player AI evaluation function
* @param player_ai String selecting the player (P1EVAL,P2EVAL)
* @param aieval String selecting the aieval function (HUMAN,RANDOM,MATERIAL,DISPERSION,MOBILITY,REACH) 
* Or combined aieval functions (F1COMB,F2COMB)
* @return 1 on success, -1 on fail
*/
int ow_game::ow_commandline_setplayer_aieval(string player_ai, string aieval) {
	parsed_command pcommand;

	//Convert the AIEVAL to uppercase
	transform (aieval.begin(), aieval.end(), aieval.begin(), ptr_fun(::toupper) );

	if (player_ai=="P1EVAL") {
		if ((this->config.p1aimode=="HUMAN")&&(aieval!="HUMAN")) {
			pcommand.name = "SET"; pcommand.param1=player_ai; pcommand.param2="";
			this->ow_commandline_setcommand_help(&pcommand);
			return -1;
		}
		if ((this->config.p1aimode!="HUMAN")&&(aieval!="RANDOM")&&(aieval!="MATERIAL")&&(aieval!="DISPERSION")&&(aieval!="MOBILITY")&&(aieval!="REACH") 
			&&(aieval!="F1COMB")&&(aieval!="F2COMB")){ 
			pcommand.name = "SET"; pcommand.param1=player_ai; pcommand.param2="";
			this->ow_commandline_setcommand_help(&pcommand);
			return -1;
		}
		this->config.p1aieval = aieval;
	}

	if (player_ai=="P2EVAL") {
		if ((this->config.p2aimode=="HUMAN")&&(aieval!="HUMAN")) {
			pcommand.name = "SET"; pcommand.param1=player_ai; pcommand.param2="";
			this->ow_commandline_setcommand_help(&pcommand);
			return -1;
		}
		if ((this->config.p2aimode!="HUMAN")&&(aieval!="RANDOM")&&(aieval!="MATERIAL")&&(aieval!="DISPERSION")&&(aieval!="MOBILITY")&&(aieval!="REACH")
			&&(aieval!="F1COMB")&&(aieval!="F2COMB")) {  
			pcommand.name = "SET"; pcommand.param1=player_ai; pcommand.param2="";
			this->ow_commandline_setcommand_help(&pcommand);
			return -1;
		}
		this->config.p2aieval = aieval;
	}

	return 1;
}

/**
* Sets if the game is created and begins with the gfxboard on/off
* @param gfxstatus Status of the gfxboard (ON/OFF)
* @return 1 on success, -1 on fail
*/
int ow_game::ow_commandline_setgfxboard_status(string gfxstatus) {
	parsed_command pcommand;
	//Convert the gfxstatus to uppercase
	transform (gfxstatus.begin(), gfxstatus.end(), gfxstatus.begin(), ptr_fun(::toupper) );

	if ((gfxstatus!="ON")&&(gfxstatus!="OFF")) {
		pcommand.name = "SET"; pcommand.param1="GFXBOARD"; pcommand.param2="";
		this->ow_commandline_setcommand_help(&pcommand);
		return -1;
	} else {
		this->config.gfxboard = gfxstatus;
		return 1;
	}
}

/**
* Shows help screen for the SET command
* @param pcommand Command info and parameters (SET parameters) for the 'set' command.
*/
void ow_game::ow_commandline_setcommand_help(parsed_command *pcommand) {
	int variable_found = 0;
	if (pcommand->param1.empty()&&pcommand->param2.empty()) {
		cout << "Incorrect syntax, try: SET <VARNAME> <VARVALUE>" << endl;
	}
	if ((!pcommand->param1.empty())&&pcommand->param2.empty()) {
		switch (this->current_menu) {
			case OWGAME_MENU_APP_NEWGAME:
				if (pcommand->param1=="P1NAME") {
					variable_found=1;
					cout << "Incorrect value for '" << pcommand->param1 << "', valid values are: (Non empty string)." << endl;
				}
				if (pcommand->param1=="P1AI") {
					variable_found=1;
					cout << "Incorrect value for '" << pcommand->param1 << "', valid values are: (HUMAN,SIMPLE,NEGAMAX,ALPHABETA,SCOUT)." << endl;
				}
				if (pcommand->param1=="P1DEPTH") {
					variable_found=1;
					cout << "Incorrect value for '" << pcommand->param1 << "', valid values are: ";
					if ((this->config.p1aimode!="HUMAN")&&(this->config.p1aimode!="SIMPLE")) {cout << "(1,2,3,...).";}
					else {cout << "(1).";}
					cout << endl;
				}
				if (pcommand->param1=="P1EVAL") {
					variable_found=1;
					cout << "Incorrect value for '" << pcommand->param1 << "', valid values are: ";
					if (this->config.p1aimode=="HUMAN") cout << "(HUMAN)."; else cout << "(RANDOM,MATERIAL,DISPERSION,MOBILITY,REACH)\nor complex functions (F1COMB,F2COMB).";
					cout << endl;
				}
				if (pcommand->param1=="P2NAME") {
					variable_found=1;
					cout << "Incorrect value for '" << pcommand->param1 << "', valid values are: (Non empty string)." << endl;
				}
				if (pcommand->param1=="P2AI") {
					variable_found=1;
					cout << "Incorrect value for '" << pcommand->param1 << "', valid values are: (HUMAN,SIMPLE,NEGAMAX,ALPHABETA,SCOUT)." << endl;
				}
				if (pcommand->param1=="P2DEPTH") {
					variable_found=1;
					cout << "Incorrect value for '" << pcommand->param1 << "', valid values are: ";
					if ((this->config.p2aimode!="HUMAN")&&(this->config.p2aimode!="SIMPLE")) {cout << "(1,2,3,...).";}
					else {cout << "(1).";}
					cout << endl;
				}
				if (pcommand->param1=="P2EVAL") {
					variable_found=1;
					cout << "Incorrect value for '" << pcommand->param1 << "', valid values are: ";
					if (this->config.p2aimode=="HUMAN") cout << "(HUMAN)."; else cout << "(RANDOM,MATERIAL,DISPERSION,MOBILITY,REACH)\nor complex functions (F1COMB,F2COMB).";
					cout << endl;
				}
				if (pcommand->param1=="NUMCOLS") {
					variable_found=1;
					cout << "Incorrect value for '" << pcommand->param1 << "', valid values are: (3,4,5,6,7,8)." << endl;
				}
				if (pcommand->param1=="FIRSTTURN") {
					variable_found=1;
					cout << "Incorrect value for '" << pcommand->param1 << "', valid values are: (P1,P2)." << endl;
				}		
				if (pcommand->param1=="GFXBOARD") {
					variable_found=1;
					cout << "Incorrect value for '" << pcommand->param1 << "', valid values are: (ON,OFF)." << endl;
				}		
				break;
		}
		if (!variable_found) {
				cout << "Variable '" << pcommand->param1 << "' not found at this level. Command not executed." << endl;
		}

	}

}

/**
* Reads each keystroke of the command line 
* and generates a new 'command' string when and 'endl/intro/enter' is received.
* The commandline handler function is called by an event passing the new 'command' string generated.
*/
void ow_game::commandline_read() {
	fd_set input_fd; struct timeval rltimeval;
	rltimeval.tv_sec = 0; rltimeval.tv_usec = 0;
	FD_ZERO(&input_fd);
	FD_SET(0,&input_fd);
	if (select(1,&input_fd,NULL,NULL,&rltimeval)) {
		if (FD_ISSET (0, &input_fd)) rl_callback_read_char();
	}
}

/**
* Generates a different prompt string depending on the status or menu of the app where you are.
* @param prompt Pointer to the variable to hold the new prompt string. 
* The variable has to have room enought to hold generated string.
*/
void ow_game::ow_commandline_getpromptstring(char *prompt) {
	switch (this->current_menu) {
		case OWGAME_MENU_APP:
				sprintf(prompt,"omweso:/> ");
				break;
		case OWGAME_MENU_APP_NEWGAME:
				sprintf(prompt,"omweso:/newgame> ");
				break;
		case OWGAME_MENU_SETUPBOARD:
				sprintf(prompt,"omweso:/setup> ");
				break;
		case OWGAME_MENU_PLY:	
				char *s_boardside;
				switch (this->selected_player->getboardside()) { case BRD_SIDE_NORTH: s_boardside="N"; break; case BRD_SIDE_SOUTH: s_boardside="S"; break;}
				sprintf(prompt,"omweso[%s%d]:/%s> ",s_boardside, this->current_turn, this->selected_player->getplayername());
				break;
		case OWGAME_MENU_MAI:
				sprintf(prompt,"omweso:/ai> ");
				break;
		case OWGAME_MENU_BAT:
				sprintf(prompt,"omweso:/ai/batch> ");
				break;
	}
}

/**
* Translates the cell names (A1,A2,..,a1,a2) to the cell positions in the board cells array.
* @param cellname Name of the cell, usually entered by the player.
* @return cellXY info (x,y,boardside) of the cell represented by 'cellname'.
*/
cellXY ow_game::ow_commandline_translate_name2coord(string cellname) {
	string columnname, rowname;
	cellXY tcell;
	tcell.x = -1;
	tcell.y = -1;
	tcell.boardside = -1;

	if (cellname.length()==2) {
		columnname = cellname.substr(0,1);
		rowname = cellname.substr(1,1);

		if ((columnname=="a")||(columnname=="A")) {tcell.x = 0;}
		if ((columnname=="b")||(columnname=="B")) {tcell.x = 1;}
		if ((columnname=="c")||(columnname=="C")) {tcell.x = 2;}
		if ((columnname=="d")||(columnname=="D")) {tcell.x = 3;}
		if ((columnname=="e")||(columnname=="E")) {tcell.x = 4;}
		if ((columnname=="f")||(columnname=="F")) {tcell.x = 5;}
		if ((columnname=="g")||(columnname=="G")) {tcell.x = 6;}
		if ((columnname=="h")||(columnname=="H")) {tcell.x = 7;}

		if (rowname=="0") {tcell.y = 0;}
		if (rowname=="1") {tcell.y = 1;}

		if ((columnname>="A")&&(columnname<="H")) {tcell.boardside=BRD_SIDE_NORTH;}
		if ((columnname>="a")&&(columnname<="h")) {tcell.boardside=BRD_SIDE_SOUTH;}
	}

	if ((tcell.boardside==-1)||(tcell.x==-1)||(tcell.y== -1)) {
		tcell.boardside=-1; tcell.x=-1;tcell.y=-1;
	}


	return tcell;
}


/**
* Translates the cell position in the board cells array into the cellname (A1,A2,..,a1,a2..) used by the players to name the cells.
* @param boardside Side of the board (BRD_SIDE_NORTH, BRD_SIDE_SOUTH).
* @param cellx Xpos of the specified cell in the board bcells array.
* @param celly Ypos of the specified cell in the board bcells array.
* @return 'Name' string of the cell specified by (boardside,cellx,celly)
*/
string ow_game::ow_commandline_translate_coord2name(int boardside, int cellx, int celly) {
	string coord2name_translations[BRD_NUM_SIDES][2][8] = {
		{{"A0","B0","C0","D0","E0","F0","G0","H0"},{"A1","B1","C1","D1","E1","F1","G1","H1"}},
		{{"a0","b0","c0","d0","e0","f0","g0","h0"},{"a1","b1","c1","d1","e1","f1","g1","h1"}}
	};
	if ((boardside>=0)&&(boardside<BRD_NUM_SIDES)&&(celly<2)&&(cellx<8))
		return coord2name_translations[boardside][celly][cellx];
  else return "";
}
