/***************************************************************************
 *   Copyright (C) 2007 by David Torres   *
 *   aiken669@gmail.com   *
 ***************************************************************************/
#include "ow_player.h"
#include "ow_aisearch_simple.h"
#include "ow_aisearch_negamax.h"
#include "ow_aisearch_alphabeta.h"
#include "ow_aisearch_scout.h"

#include <iostream>
using namespace std;

/**
* Creates a new 'ow_player' object, based on the parameters.
* @param owboard A pointer to the ow_board previously created.
* @param playernum Player number (PL_PLAYER1,PL_PLAYER2).
* @param boardside Side of the board (BRD_SIDE_NORTH, BRD_SIDE_SOUTH).
* @return New 'ow_player' object.
* The player is created as AI_HUMAN by default, you have to use setplayer_aimode() to change it
*/
ow_player::ow_player(ow_board *owboard,  int playernum, int boardside)
{
	this->owboard = owboard;
	this->playernum = playernum;
	sprintf(this->playername,"player%d",playernum+1);
	this->aimode = AI_HUMAN;
	this->aidepth = 1;
	this->aieval = AI_HUMAN_EVAL;
	this->boardside = boardside;
	this->sleep = 1;
	this->aisearch = NULL;
	this->searchfound = 0;
}

/**
* Deletes the 'ow_player' object.
*/
ow_player::~ow_player()
{
	delete this->aisearch;
	this->aisearch = NULL;
}

/**
* Checks if player has ended his turn
* @return 1 if turn ended, 0 if turn still running
*/
int ow_player::endofturn() {
	return this->sleep;
}

/**
* Sends the player to sleep, this usually means the player's turn ends.
*/
void ow_player::gosleep() {
	this->sleep = 1;
}

/**
* Wake up from sleep, this usually means the player's turn begins.
*/
void ow_player::wakeup() {
	this->sleep = 0;
}

/**
* Returns the player number, used during the game.
* @return The player num.
*/
int ow_player::getplayernum() {
	return this->playernum;
}

/**
* Returns side of the board where the player plays the game.
* @return Side of the board (BRD_SIDE_NORTH, BRD_SIDE_SOUTH).
*/
int ow_player::getboardside() {
	return this->boardside;
}

/**
* Returns the AI mode used by the player.
* @return Player's Artificial Intelligence mode (AI_HUMAN, AI_COMPUTER, etc)
*/
int ow_player::getaimode() {
	return this->aimode;
}

/**
* Executes a movement. Usually called by AI_HUMAN players.
* @param boardside Side of the board (BRD_SIDE_NORTH, BRD_SIDE_SOUTH).
* @param cellx Xpos of the specified cell in the board bcells array.
* @param celly Ypos of the specified cell in the board bcells array.
* @return 0 on success, -1 on any error.
*/
int ow_player::makemovement(int boardside, int cellx, int celly) {
	int merror = -1;
	if ((cellx>=0)and(celly>=0)) {
		if (boardside==this->boardside) merror = this->owboard->move(boardside, cellx, celly);
		if (merror>=0) this->gosleep();
  }
	return merror;
}

/**
* Executes a reverse movement, ending in capture. Usually called by AI_HUMAN players.
* @param boardside Side of the board (BRD_SIDE_NORTH, BRD_SIDE_SOUTH).
* @param cellx Xpos of the specified cell in the board bcells array.
* @param celly Ypos of the specified cell in the board bcells array.
* @return 0 on success, -1 on any error.
*/
int ow_player::rcapture(int boardside, int cellx, int celly) {
	int merror = -1;
	if ((cellx>=0)and(celly>=0)) {
		if (boardside==this->boardside) merror = this->owboard->rcapture(boardside, cellx, celly);
		if (merror>=0) this->gosleep();
  }
	return merror;
}

/**
* Find best movement based on aimode.
* @return Cell to start the best AI move found
*/
ow_move ow_player::searchbestmove_ai() {
	int feval;ow_move prinvar[AI_MAXDEPTH];int idxprvar=0;
	char strfeval[50];
	this->searchfound=0;
	feval = this->aisearch->search(prinvar, this->owboard, this->boardside,this->aidepth);
	if (prinvar[0].boardside==this->boardside) {
		this->searchfound=1;
		if (feval<-1000) {sprintf(strfeval," -> [eval:%s LOSES]",this->getplayername(),feval);}
		if ((feval<1000)&&(feval>-1000)) {sprintf(strfeval," -> [eval:%d/1000]",feval);}
		if (feval>1000) {sprintf(strfeval," -> [eval:%s WINS]",this->getplayername(),feval);}
		cout << "Principal variation:";while (prinvar[idxprvar].boardside!=-1) {cout << " -> " << ((prinvar[idxprvar].dir==1)?"mv(":"rc(") << this->owboard->translate_coord2name(prinvar[idxprvar].boardside, prinvar[idxprvar].x, prinvar[idxprvar].y) << ")";	idxprvar++;} cout << strfeval << endl;
	}

/*
	// SCOUT search to TEST the algoritm is working fine
	if (this->aimode!=AI_SCOUT) {
		int feval2;ow_move prinvar2[AI_MAXDEPTH];int idxprvar2=0;
		ow_aisearch *checksearch = new ow_aisearch_scout(this->owboard);
		checksearch->setevalfunc(this->aieval);
		feval2 = checksearch->search(prinvar2, this->owboard, this->boardside,this->aidepth);
		delete checksearch;
		if (prinvar2[0].boardside==this->boardside) {
			this->searchfound=1;
			if (feval2<1000) {sprintf(strfeval," -> [eval:%d/1000]",feval2);}	
			if (feval2>1000) {sprintf(strfeval," -> [eval:%s WINS]",this->getplayername());}
			cout << "Principal variation:";while (prinvar2[idxprvar2].boardside!=-1) {cout << " -> " << ((prinvar2[idxprvar2].dir==1)?"mv(":"rc(") << this->owboard->translate_coord2name(prinvar2[idxprvar2].boardside, prinvar2[idxprvar2].x, prinvar2[idxprvar2].y) << ")";	idxprvar2++;} cout << strfeval << endl;
		}
		if (feval2!=feval) {cout << endl << "WARNING!WARNING! feval value mismatch!" << endl << endl;}
	}
*/


	return prinvar[0];
}

/**
* Checks if AI found a new move, if so it was returned by the last searchbestmove_ai() call.
* @return 1 if found, -1 if not found, 0 if still searching.
*/
int ow_player::foundbestmove_ai() {
	return this->searchfound;
}

/**
* Sets a new name for player.
* @param pname New name for the player.
*/
void ow_player::setplayername(const char *pname) {
	sprintf(this->playername,"%s",pname);
}

/**
* Gets the name of the player.
* @return the player's name.
*/
char *ow_player::getplayername() {
	return this->playername;
}

/**
* Changes the AIMODE of the player. 
* And creates the new aisearch() object for player to use.
* @param aimode aimode to set (AI_HUMAN, AI_SIMPLE, AI_NEGAMAX, AI_ALPHABETA, AI_SCOUT)
*/
void ow_player::setplayer_aimode(int aimode) {
	this->aimode = aimode;
	switch (this->aimode) {
		case AI_HUMAN: this->aisearch = NULL; break;
		case AI_SIMPLE:
				//cout << "setrandom" << endl;
				delete this->aisearch;
				this->aisearch = new ow_aisearch_simple(this->owboard);
				break;
		case AI_NEGAMAX:
				//cout << "setnegamax" << endl;
				delete this->aisearch;
				this->aisearch = new ow_aisearch_negamax(this->owboard);
				break;				
		case AI_ALPHABETA:
				//cout << "setalphabeta" << endl;
				delete this->aisearch;
				this->aisearch = new ow_aisearch_alphabeta(this->owboard);
				break;				
		case AI_SCOUT:
				//cout << "setscout" << endl;
				delete this->aisearch;
				this->aisearch = new ow_aisearch_scout(this->owboard);
				break;				
	}
}

/**
* Change the AIDEPTH of the player aisearch algoritm. 
* @param depth new aidepth value to set
*/
void ow_player::setplayer_aidepth(int depth) {
	this->aidepth = depth;
}

/**
* Change the AIEVAL function of the player. 
* And creates the new aisearch() object for player to use.
* @param aieval aieval function to set (AI_RANDOM_EVAL,AI_MATERIAL_EVAL,AI_DISPERSION_EVAL,AI_MOBILITY_EVAL,AI_REACH_EVAL)
* Or combined aieval functions (AI_COMBINED_EVAL)
*/
void ow_player::setplayer_aieval(int aieval) {
	this->aieval = aieval;
	this->aisearch->setevalfunc(aieval);
}
